C-----------------------------------------------------------------------
      PROGRAM CONDAPEH
C
C  THIS IS A B.E. PROGRAM THAT USES (CON)STANT ELEMENTS FOR THE
C  SOLUTION OF (D)YNAMIC (A)NTI(P)LANE (E)LASTIC OR VISCOELASTIC
C  TIME (H)ARMONIC PROBLEMS
C  PROGRAM NO. 1
C
C  SET DIMENSIONS
C  NX=MAXIMUM NUMBER OF BOUNDARY NODES
C  NFM=MAXIMUM NUMBER OF FREQUENCIES
C  NPM=MAXIMUN NUMBER OF INTERNAL POINTS
C
      PARAMETER(NX=50,NFM=10,NPM=20)
      CHARACTER*12 FILEIN,FILEOUT
C
      DIMENSION X(NX+1),Y(NX+1),XM(NX),YM(NX)
      DIMENSION KODE(NX),CX(NPM),CY(NPM),AFR(NFM)
      COMPLEX G(NX,NX),H(NX,NX),BC(NX),FI(NX),DFI(NX),SOL(NPM)
      COMPLEX CW,GE
      COMMON/MATG/ G
      COMMON/MATH/ H
      COMMON  N,L,CW,GE,FR,INP,IPR
C
C  ASSIGN NUMBERS FOR INPUT AND OUTPUT FILES
C
      INP=5
      IPR=6
C
C  READ NAMES AND OPEN  FILES FOR INPUT AND OUTPUT
C
      WRITE(*,' (A) ') ' NAME OF INPUT FILE (MAX. 12 CHART.)'
      READ(*,'  (A) ')FILEIN
      OPEN(INP,FILE=FILEIN,STATUS='OLD')
      WRITE(*,' (A) ') ' NAME OF OUTPUT FILE (MAX. 12 CHART.)'
      READ(*,' (A) ')FILEOUT
      OPEN(IPR,FILE=FILEOUT,STATUS='NEW')
C
C  READ DATA
C
      CALL INPUT1(CX,CY,X,Y,KODE,BC,NFR,AFR)

      DO 60 I=1,NFR
      FR=AFR(I)
      DO 40 J=1,N
   40 DFI(J)=BC(J)

C
C  FORM SYSTEM OF EQUATIONS
C
      CALL GHMAT1(X,Y,XM,YM,G,H,FI,DFI,KODE,NX)
C
C  SOLVE SYSTEM OF EQUATIONS
C
      CALL CSOLVER(H,FI,N,NX)
C
C  COMPUTE  DISPLACEMENT VALUES AT INTERNAL POINTS
C
      CALL INTER1(FI,DFI,KODE,CX,CY,X,Y,SOL)
C
C  PRINT RESULTS AT BOUNDARY NODES AND INTERNAL POINTS
C
      CALL OUTPUT1(XM,YM,FI,DFI,CX,CY,SOL)
   60 CONTINUE
C
C  CLOSE INPUT AND OUTPUT FILES
C
      CLOSE (INP)
      CLOSE (IPR)
      STOP
      END
C-----------------------------------------------------------------------
      SUBROUTINE INPUT1(CX,CY,X,Y,KODE,BC,NFR,AFR)
C
C  N= NUMBER OF BOUNDARY NODES (=NUMBER OF ELEMENTS)
C  L= NUMBER OF INTERNAL POINTS WHERE THE FUNCTION IS CALCULATED
C  CW= SHEAR WAVE VELOCITY
C  RO= DENSITY
C  GE= SHEAR MODULUS
C
      CHARACTER*80 TITLE
      DIMENSION CX(1),CY(1),X(1),Y(1),KODE(1),AFR(1)
      COMPLEX BC(1),CW,GE
      COMMON  N,L,CW,GE,FR,INP,IPR
      WRITE(IPR,100)
  100 FORMAT(' ',79('*'))
C
C  READ JOB TITLE
C
      READ(INP,'(A)') TITLE
      WRITE(IPR,'(A)') TITLE
C
C  READ NUMBER OF BOUNDARY ELEMENTS, INTERNAL POINTS AND FREQUENCIES
C  READ MATERIAL PROPERTIES
C
      READ(INP,*)N,L,NFR,RGE,DAM,RO
      GE=CMPLX(RGE,RGE*2*DAM)
      CW=CSQRT(GE/RO)
      WRITE(IPR,300)N,L,NFR,CW,GE,DAM,RO
  300 FORMAT(//' DATA'//2X,'NUMBER OF BOUNDARY ELEMENTS =',I3/2X,'NUMBER
     1 OF INTERNAL POINTS =',I3/2X,'NUMBER OF FREQUENCIES =',I3/2X,
     2'WAVE VELOCITY =',G14.7,',',G14.7/2X,
     3'SHEAR MODULUS =',G14.7,',',G14.7/2X,
     4'DAMPING RATIO=',G14.7/2X,
     5'DENSITY=',G14.7)
C
C  READ FREQUENCIES
C
      READ(INP,*)(AFR(I),I=1,NFR)
      WRITE(IPR,400) (AFR(I),I=1,NFR)
  400 FORMAT(/2X,'FREQUENCIES'/5(2X,G14.7))

C
C  READ COORDINATES OF EXTREME POINTS OF THE BOUNDARY ELEMENTS
C  IN ARRAYS X AND Y
C
      WRITE(IPR,500)
  500 FORMAT(//2X,'COORDINATES OF THE EXTREME POINTS OF THE BOUNDARY ELE
     1MENTS',//1X,'POINT',7X,'X',15X,'Y')
      READ(INP,*) (X(I),Y(I),I=1,N)
      DO 10 I=1,N
   10 WRITE(IPR,700)I,X(I),Y(I)
  700 FORMAT(2X,I3,2(2X,G14.5))
C
C  READ BOUNDARY CONDITIONS IN BC(I) VECTOR, IF KODE(I)=0 THE BC(I)
C  VALUE IS A KNOWN DISPLACEMENT;IF KODE(I)=1 THE BC(I) VALUE IS A
C  KNOWN TRACTION.
C
      WRITE(IPR,800)
  800 FORMAT(//2X,'BOUNDARY CONDITIONS'//2X,'NODE',6X,'CODE',14X,
     1'PRESCRIBED VALUE')
      DO 20 I=1,N
      READ(INP,*) KODE(I),BC(I)
   20 WRITE(IPR,950)I,KODE(I),BC(I)
  950 FORMAT(2X,I3,9X,I1,8X,G14.7,',',G14.7)
C
C  READ COORDINATES OF THE INTERNAL POINTS
C
      IF(L.EQ.0) GO TO 30
      READ(INP,*) (CX(I),CY(I),I=1,L)
   30 RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE GHMAT1(X,Y,XM,YM,G,H,FI,DFI,KODE,NX)
C
C  THIS SUBROUTINE COMPUTES THE G AND H MATRICES
C  AND FORMS THE SYSTEM OF EQUATIONS A X = F
C
      COMMON  N,L,CW,GE,FR,INP,IPR
      DIMENSION X(1),Y(1),XM(1),YM(1),KODE(1)
      COMPLEX FI(1),DFI(1),G(NX,NX),H(NX,NX),CH,CW,GE
C
C  COMPUTE THE NODAL COORDINATES AND STORE IN ARRAYS XM AND YM
C
      X(N+1)=X(1)
      Y(N+1)=Y(1)
      DO 10 I=1,N
      XM(I)=(X(I)+X(I+1))/2
   10 YM(I)=(Y(I)+Y(I+1))/2
C
C  COMPUTE THE COEFFICIENTS OF G AND H MATRICES
C
      DO 30 I=1,N
      DO 30 J=1,N
      KK=J+1
      IF(I-J)20,25,20
   20 CALL EXTIN1(XM(I),YM(I),X(J),Y(J),X(KK),Y(KK),H(I,J),G(I,J))
      GO TO 30
   25 CALL LOCIN1(X(J),Y(J),X(KK),Y(KK),G(I,J))
      H(I,J)=0.5
   30 CONTINUE
C
C  REORDER THE COLUMNS OF THE SYSTEM OF EQUATIONS IN ACCORDANCE
C  WITH THE BOUNDARY CONDITIONS AND FORM SYSTEM MATRIX A WHICH
C  IS STORED IN H
C
      DO 50 J=1,N
      IF(KODE(J))40,40,50
   40 DO 42 I=1,N
      CH=G(I,J)*GE
      G(I,J)=-H(I,J)
   42 H(I,J)=-CH
   50 CONTINUE
C
C  FORM THE RIGHT HAND SIDE VECTOR F WHICH IS STORED IN FI
C
      DO 60 I=1,N
      FI(I)=(0.,0.)
      DO 60 J=1,N
      FI(I)=FI(I)+G(I,J)*DFI(J)
   60 CONTINUE
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE EXTIN1(XP,YP,X1,Y1,X2,Y2,H,G)
C
C  THIS SUBROUTINE COMPUTES THE OFF DIAGONAL COEFFICIENTS OF
C  H AND G MATRICES USING GAUSS QUADRATURE
C
C  RA = RADIUS   = DISTANCE FROM THE COLLOCATION POINT TO THE
C                  GAUSS INTEGRATION POINTS ON THE BOUNDARY ELEMENT
C  RD1,RD2,RDN   = RADIUS DERIVATIVES
C  ETA1,ETA2     = COMPONENTS OF THE UNIT NORMAL TO THE ELEMENT
C  GI(I)         = GAUSS QUADRATURE POINTS
C  OME(I)        = GAUSS QUADRATURE WEIGHTS
C  XCO(I),YCO(I) = INTEGRATION POINTS ALONG THE ELEMENT
C
      COMMON  N,L,CW,GE,FR,INP,IPR
      COMPLEX G,H,Z,F0C,F1C,F2C,F0CM,F1CM,F2CM,CW,GE
      DIMENSION XCO(4),YCO(4),GI(4),OME(4)
      DATA GI/0.86113631,-0.86113631,0.33998104,-0.33998104/
      DATA OME/0.34785485,0.34785485,0.65214515,0.65214515/
C
      AX=(X2-X1)/2.
      BX=(X2+X1)/2.
      AY=(Y2-Y1)/2.
      BY=(Y2+Y1)/2.
      SR=SQRT(AX**2+AY**2)
      ETA1=(Y2-Y1)/(2.*SR)
      ETA2=(X1-X2)/(2.*SR)
      G=(0.,0.)
      H=(0.,0.)
C
C COMPUTE G AND H COEFFITIENTS
C
      DO 40 I=1,4
      XCO(I)=AX*GI(I)+BX
      YCO(I)=AY*GI(I)+BY
      RA=SQRT((XP-XCO(I))**2+(YP-YCO(I))**2)
      RD1=(XCO(I)-XP)/RA
      RD2=(YCO(I)-YP)/RA
      RDN=RD1*ETA1+RD2*ETA2
      ZR=FR*RA/CW
      Z=CMPLX(0.,ZR)
      CALL BESSEL(Z,F0C,F1C,F2C,F0CM,F1CM,F2CM)
      G=G+F0C*OME(I)*SR/(6.2831852*GE)
   40 H=H-Z/RA*RDN*F1C*OME(I)*SR/6.2831852
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE LOCIN1(X1,Y1,X2,Y2,G)
C
C  THIS SUBROUTINE COMPUTES THE VALUES OF THE DIAGONAL
C  COEFFICIENTS OF THE G MATRIX
C
      COMMON  N,L,CW,GE,FR,INP,IPR
      COMPLEX Z,S0C,G,CW,GE
      AX=(X2-X1)/2.
      AY=(Y2-Y1)/2.
      SR=SQRT(AX**2+AY**2)
      ZR=FR*SR/CW
      Z=CMPLX(0.,ZR)
      CALL INBESS(Z,S0C)
      G=(SR/Z)*S0C/(3.1415926*GE)
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE INTER1(FI,DFI,KODE,CX,CY,X,Y,SOL)
C
C  THIS SUBROUTINE COMPUTES THE VALUES OF DISPLACEMENT AT INTERNAL POINTS
C
      COMMON  N,L,CW,GE,FR,INP,IPR
      COMPLEX FI(1),DFI(1),SOL(1),CH,H,G,CW,GE
      DIMENSION KODE(1),CX(1),CY(1),X(1),Y(1)
C
C  REARRANGE THE FI AND DFI ARRAYS TO STORE ALL THE VALUES OF THE
C  DISPLACEMENT IN FI AND ALL THE VALUES OF THE TRACTION IN DFI
C
      DO 20 I=1,N
      IF(KODE(I)) 10,10,20
   10 CH=FI(I)*GE
      FI(I)=DFI(I)
      DFI(I)=CH
   20 CONTINUE
C
C  COMPUTE THE VALUES OF DISPLACEMENT AT INTERNAL POINTS
C
      IF(L.EQ.0) RETURN
      DO 40 K=1,L
      SOL(K)=(0.,0.)
      DO 30 J=1,N
      KK=J+1
      CALL EXTIN1(CX(K),CY(K),X(J),Y(J),X(KK),Y(KK),H,G)
   30 SOL(K)=SOL(K)+DFI(J)*G-FI(J)*H
   40 CONTINUE
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE OUTPUT1(XM,YM,FI,DFI,CX,CY,SOL)
C
C  THIS SUBROUTINE PRINTS THE VALUES OF THE DISPLACEMENT AND TRACTION
C  AT BOUNDARY NODES. IT ALSO PRINTS THE VALUES OF THE DISPLACEMENT
C  AT INTERNAL POINTS
C

      COMMON  N,L,CW,GE,FR,INP,IPR
      COMPLEX FI(1),DFI(1),SOL(1),CW,GE
      DIMENSION XM(1),YM(1),CX(1),CY(1)
      WRITE(IPR,50)
   50 FORMAT(' ',79('*')/)
      WRITE(IPR,700) FR
  700 FORMAT(30X,'FREQUENCY =',G14.7/)
      WRITE(IPR,100)
  100 FORMAT(' ',79('*')//1X,'RESULTS'//2X,'BOUNDARY NODES'//5X,'X',11
     1X,'Y',15X,'DISPLACEMENT',18X,'TRACTION'/)
      DO 10 I=1,N
   10 WRITE(IPR,200) XM(I),YM(I),FI(I),DFI(I)
  200 FORMAT(2(2X,G10.3),2(3X,G12.5,',',G12.5))
C
      IF(L.EQ.0) GO TO 30
      WRITE(IPR,300)
  300 FORMAT(//2X,'INTERNAL POINTS',//5X,'X',11X,'Y',15X,'DISPLACEMENT'
     1/)
      DO 20 K=1,L
   20 WRITE(IPR,400)CX(K),CY(K),SOL(K)
  400 FORMAT(2(2X,G10.3),3X,G12.5,',',G12.5)
   30 WRITE(IPR,500)
  500 FORMAT(' ',79('*'))
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE CSOLVER(A,B,N,NX)
C
      COMPLEX A(NX,NX),B(NX),X,CDOT
C
C     A=L*U  FACTORIZATION
C
      DO 30 K=1,N
      K1=K-1
      DO 10 I=K,N
      A(I,K)=A(I,K)-CDOT(K1,A(I,1),NX,A(1,K),1)
   10 CONTINUE
      X=A(K,K)
      IF(ABS(REAL(X))+ABS(AIMAG(X)).EQ.0.) THEN
      WRITE(6,*) ' SOLVER : STOP  SINGULARITY IN ROW ',K
      STOP ' SOLVER : STOP SINGULARITY '
      END IF
      X=(1.,0.)/X
      DO 20 J=K+1,N
      A(K,J)=X*(A(K,J)-CDOT(K1,A(K,1),NX,A(1,J),1))
   20 CONTINUE
   30 CONTINUE
C
C  FORWARD AND BACK SUBSTITUTION    L*U*X=B
C
C  SOLVE  L*Y=B
C
      DO 40 I=1,N
      B(I)=-(CDOT((I-1),A(I,1),NX,B,1)+B(I))/A(I,I)
   40 CONTINUE
C
C  SOLVE  U*X=Y
C
      DO 50 I=N,1,-1
      B(I)=-(CDOT((N-I),A(I,I+1),NX,B(I+1),1)+B(I))
   50 CONTINUE
C
      RETURN
      END
C-----------------------------------------------------------------------
      COMPLEX FUNCTION CDOT(N,U,IUX,V,IVX)
C
C THIS FUNCTION MAKES THE COMPLEX DOT PRODUCT OF TWO VECTORS
C
      COMPLEX U(1),V(1)
C
      CDOT=(0.,0.)
      IF(N.LE.0) RETURN
      IU=1
      IV=1
      IF(IUX.LT.0) IU=1-(N-1)*IUX
      IF(IVX.LT.0) IV=1-(N-1)*IVX
      DO 10 I=1,N
      CDOT=CDOT+U(IU)*V(IV)
      IU=IU+IUX
      IV=IV+IVX
10    CONTINUE
      RETURN
C
      END

C-----------------------------------------------------------------------
      SUBROUTINE BESSEL(Z,FK0,FK1,FK2,FK0M,FK1M,FK2M)
C
C
C  THIS SUBROUTINE COMPUTES THE MODIFIED BESSEL FUNCTIONS OF THE SECOND
C  KIND AND ORDERS ZERO, ONE AND TWO (FK0,FK1,FK2) AND COMPLEX ARGUMENT
C  Z USING A POLINOMIUM
C
C  THE COEFFICIENTS OF THE POLINOMIUM HAVE BEEN OBTAINED FROM THE
C  TERMS OF THE SERIES EXPANSION
C
C  TWO DIFFEERENT SERIES HAVE BEEN USED. ONE FOR MODULUS(Z).LT.5
C  AND THE OTHER FOR MODULUS(Z).GT.5
C
C  FOR SMALL ARGUMENTS THE PARTS OF THE MODIFIED BESSEL FUNCTIONS
C  TENDING TO ZERO WITH THE ARGUMENT ARE COMPUTED SEPARATEDLY FROM
C  THE BESSEL FUNCTIONS AND STORED IN FK0M,FK1M AND FK2M. THESE
C  PARTS ARE ONLY USED BY THE QUADRATIC ELEMENT CODES IN ORDER
C  TO COMPUTE THE DIFFERENCE BETWEEN THE DYNAMIC AND THE STATIC
C  FUNDAMENTAL SOLUTION TRACTIONS

C
      COMPLEX  Z,FK0,FK1,FK2,ZZ,SER,FK0M,FK1M,FK2M,FI0M,FI1M
      REAL     CI0P(15),CI1P(15),CK0P(15),CK0G(12),CK1P(15),CK1G(12)
C
      DATA CI0P/ 1.00000000E+00, 0.62500000E+01, 0.97656250E+01,
     &           0.67816840E+01, 0.26490953E+01, 0.66227383E+00,
     &           0.11497810E+00, 0.14665573E-01, 0.14321849E-02,
     &           0.11050809E-03, 0.69067559E-05, 0.35675392E-06,
     &           0.15484111E-07, 0.57263725E-09, 0.18260116E-10/
      DATA CI1P/ 0.50000000E+00, 0.15625000E+01, 0.16276042E+01,
     &           0.84771050E+00, 0.26490953E+00, 0.55189486E-01,
     &           0.82127211E-02, 0.91659834E-03, 0.79565828E-04,
     &           0.55254047E-05, 0.31394345E-06, 0.14864747E-07,
     &           0.59554274E-09, 0.20451330E-10, 0.60867054E-12/
      DATA CK0P/-0.57721566E+00, 0.26424021E+01, 0.90115658E+01,
     &           0.85185931E+01, 0.39898493E+01, 0.11299171E+01,
     &           0.21532918E+00, 0.29560538E-01, 0.30657944E-02,
     &           0.24883689E-03, 0.16242981E-04, 0.87142913E-06,
     &           0.39112788E-07, 0.14905278E-08, 0.48833881E-10/
      DATA CK1P/ 1.00000000E+00, 0.96519581E+00,-0.26280638E+02,
     &          -0.44329875E+02,-0.29269699E+02,-0.10636897E+02,
     &          -0.24689720E+01,-0.39918196E+00,-0.47620526E-01,
     &          -0.43685559E-02,-0.31795287E-03,-0.18814687E-04,
     &          -0.92322279E-06,-0.38181087E-07,-0.13490886E-08/
      DATA CK0G/ 0.12533141E+01,-0.31332853E-01, 0.35249460E-02,
     &          -0.73436375E-03, 0.22489890E-03,-0.91084054E-04,
     &           0.45921544E-04,-0.27716932E-04, 0.19488468E-04,
     &          -0.15644909E-04, 0.14119530E-04,-0.14151620E-04/
      DATA CK1G/ 0.12533141E+01, 0.93998560E-01,-0.58749100E-02,
     &           0.10281093E-02,-0.28915573E-03, 0.11132496E-03,
     &          -0.54270916E-04, 0.31981075E-04,-0.22086930E-04,
     &           0.17485486E-04,-0.15605797E-04, 0.15499393E-04/
C
      R=CABS(Z)
      RR=5.0
C
      IF(R-RR) 10,20,20
C
   10 ZZ=Z/CMPLX(RR,0.)
      ZZ=ZZ*ZZ
C
C  COMPUTE THE NUMBER OF TERMS TO BE USED IN THE SERIES
C  FOR SMALL ARGUMENTS
C
      N=MAX0(2,MIN1(15.,24.177/ALOG(RR/R)))
C
C  COMPUTE  I0(Z)-1.
C
      FI0M=SER(CI0P,ZZ,2,N)
C
C  COMPUTE I1(Z)-Z/2
C
      FI1M=Z*SER(CI1P,ZZ,2,N)
C
C  COMPUTE K0M(Z) AND K0(Z)
C
      FK0M=-FI0M*CLOG(Z/(2.,0.))+SER(CK0P,ZZ,2,N)
      FK0=FK0M-CLOG(Z/(2.,0.))+CMPLX(CK0P(1),0.)
C
C  COMPUTE K1M(Z) AND K1(Z)
C
      FK1M=(FI1M*Z*CLOG(Z/(2.,0.))+SER(CK1P,ZZ,2,N))/Z
     &+CMPLX(CK0P(1)+0.5,0.)*Z/(2.,0.)
      FK1=FK1M+(1.,0.)/Z+Z*(CLOG(Z/(2.,0.))-CMPLX(CK0P(1)+0.5,0.))/2
C
C  COMPUTE K2M(Z) AND K2(Z)
C
      FK2M=FK0M+FK1M*(2.,0.)/Z
      FK2=FK0+FK1*(2.,0.)/Z
      GO TO 30

   20 ZZ=CMPLX(RR,0.)/Z
C
C  COMPUTE THE NUMBER OF TERMS TO BE USED IN THE SERIES
C  FOR LARGE ARGUMENTS
C
C  IN THIS CASE FK0M,FK1M AND FK2M ARE COMPUTED IN ORDER
C  TO BE CONSISTENT IN THE NUMERICAL INTEGRATION PROCESS
C
      N=MAX0(2,MIN1(12.,12.088/ALOG(R/RR)))
C
C  COMPUTE K0(Z) AND K0M(Z)
C
      FK0=(CMPLX(CK0G(1),0.)+SER(CK0G,ZZ,2,N))*CEXP(-Z)/CSQRT(Z)
      FK0M=FK0+CLOG(Z/(2.,0.))-CMPLX(CK0P(1),0.)
C
C  COMPUTE K1(Z) AND K1M(Z)
C
      FK1=(CMPLX(CK1G(1),0.)+SER(CK1G,ZZ,2,N))*CEXP(-Z)/CSQRT(Z)
      FK1M=FK1-(1.,0.)/Z-Z*(CLOG(Z/(2.,0.))-CMPLX(CK0P(1)+0.5,0.))/2
C
C  COMPUTE K2(Z) AND K2M(Z)
C
      FK2=FK0+FK1*(2.,0.)/Z
      FK2M=FK0M+FK1M*(2.,0.)/Z
C
   30 RETURN
      END
C-----------------------------------------------------------------------
      COMPLEX FUNCTION SER(S,ZZ,N1,N2)
C
C  THIS FUNCTION COMPUTES THE VALUE OF A SERIES OF N2-N1+1
C  TERMS CONSISTING OF INCREASING POWERS OF ZZ TIMES THE
C  COEFFICIENTS IN ARRAY S
C
      COMPLEX  ZZ,ZZZ
      REAL     S(N2)
C
      SER=(0.,0.)
      ZZZ=(1.,0.)
      DO 10 I=N1,N2
      ZZZ=ZZZ*ZZ
      SER=SER+CMPLX(S(I),0.)*ZZZ
   10 CONTINUE
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE INBESS(Z,FINK0)
C
C  THIS SUBROUTINE COMPUTES THE INTEGRAL OF THE MODIFIED BESSEL FUNCTION
C  OF ZERO ORDER BETWEEN 0 AND Z USING A POLINOMIUM.
C
C  THE COEFFICIENTS OF THE POLINOMIUM HAVE BEEN OBTAINED FROM THE
C  TERMS OF THE SERIES EXPANSION
C
      COMPLEX  Z,FINK0,ZZ,SER
      DIMENSION S1(20),S2(20)
C
      DATA S0  / 0.42278434E+00/
      DATA S1  / 0.90734120E+01, 0.72756425E+02, 0.25901019E+03,
     &           0.52398212E+03, 0.68598066E+03, 0.62976149E+03,
     &           0.42826262E+03, 0.22451590E+03, 0.93540077E+02,
     &           0.31723213E+02, 0.89292258E+01, 0.21196877E+01,
     &           0.43013488E+00, 0.75474799E-01, 0.11565621E-01,
     &           0.15612002E-02, 0.18705605E-03, 0.20027866E-04,
     &           0.19277885E-05, 0.16772276E-06/
      DATA S2  / 0.12000000E+02, 0.64800000E+02, 0.18514286E+03,
     &           0.32400000E+03, 0.38173091E+03, 0.32300308E+03,
     &           0.20566727E+03, 0.10207750E+03, 0.40592223E+02,
     &           0.13221467E+02, 0.35916023E+01, 0.82606852E+00,
     &           0.16293265E+00, 0.27862515E-01, 0.41703893E-02,
     &           0.55091790E-03, 0.64704940E-04, 0.68008195E-05,
     &           0.64341868E-06, 0.55082916E-07/
C
      R=CABS(Z)
      RR=12.0
      IF(R-RR) 10,20,20
  10  N=MAX0(1,MIN1(20.,19.572/ALOG(RR/R)))
      GO TO 30
  20  N=20
      WRITE(6,*) ' INBESS: RESULTS MAY BE INACCURATE '
C
  30  ZZ=Z/CMPLX(RR,0.)
      ZZ=ZZ*ZZ
C
      FINK0=-CLOG(Z/(2.,0.))*((1.,0.)+SER(S2,ZZ,1,N))
      FINK0=Z*(CMPLX(S0,0.)+SER(S1,ZZ,1,N)+FINK0)
C
      RETURN
      END
