C-----------------------------------------------------------------------
      PROGRAM CONDPOEH
C
C  THIS IS A B.E. PROGRAM THAT USES (CON)STANT ELEMENTS FOR THE SOLUTION
C  OF (D)YNAMIC PLANE (PO)RO-(E)LASTIC TIME (H)ARMONIC PROBLEMS
C
C
      CHARACTER*12 FILEIN,FILEOUT
C
      PARAMETER(NX=90,NFRX=15,NPIX=10)
      COMPLEX G(NX,NX),H(NX,NX),BC(NX),FI(NX),DFI(NX),DSOL(3*NPIX)
      COMMON/MATG/G
      COMMON/MATH/H
      DIMENSION X(NX/3+1),Y(NX/3+1),XM(NX/3),YM(NX/3)
      DIMENSION KODE(NX),CX(NPIX),CY(NPIX),AFR(NFRX)
      COMPLEX AL(3),BL(3),CL(3),XL(3)
      COMPLEX CON1,CON2,ET,GAMMA,TRO,TK
      COMMON N,LN,INP,IPR,GE,XLAME,FR,XNU,ROS,ROF,ROA,AL,BL,CL
      COMMON BETA,SB,BQ,BR,RO11,RO12,RO22,XL,CON1,CON2,ET,GAMMA,TRO,TK
C
C  NX   = MAXIMUM DIMENSION OF THE SYSTEM OF EQUATIONS
C       = (3*MAXIMUM NUMBER OF BOUNDARY NODES)
C  NFRX = MAXIMUM NUMBER OF FREQUENCIES
C  NPIX = MAXIMUM NUMBER OF INTERNAL POINTS
C
C  ASSIGN NUMBERS FOR INPUT AND OUTPUT FILES
C
      INP=5
      IPR=6
C
C  READ NAMES AND OPEN FILES FOR INPUT AND OUTPUT
C
      WRITE(*,' (A) ') ' NAME OF THE INPUT  FILE (MAX. 12 CHART.) '
      READ(*,' (A) ') FILEIN
      OPEN(INP,FILE=FILEIN,STATUS='OLD')
      WRITE(*,' (A) ') ' NAME OF THE OUTPUT FILE (MAX. 12 CHART.) '
      READ(*,' (A) ') FILEOUT
      OPEN(IPR,FILE=FILEOUT,STATUS='NEW')
C
C  READ DATA
C
      CALL INPUT14(CX,CY,X,Y,KODE,BC,NFR,AFR)
      NN=3*N
      DO 60 I=1,NFR
      FR=AFR(I)
      DO 40 J=1,NN
  40  DFI(J)=BC(J)
C
C  COMPUTE G AND H MATRICES AND FORM SYSTEM OF EQUATIONS (A X = F)
C
      CALL GHMAT14(X,Y,XM,YM,G,H,FI,DFI,KODE,NX)
C
C  SOLVE SYSTEM OF EQUATIONS
C
      CALL CSOLVER(H,FI,NN,NX)
C
C  COMPUTE SOLID DISPLACEMENTS AND FLUID STRESS AT INTERNAL POINTS
C
      CALL INTER14(FI,DFI,KODE,CX,CY,X,Y,DSOL)
C
C  PRINT RESULTS AT BOUNDARY NODES AND INTERNAL POINTS
C
      CALL OUTPUT14(XM,YM,FI,DFI,CX,CY,DSOL)
   60 CONTINUE
      CLOSE (INP)
      CLOSE (IPR)
      STOP
      END
C-----------------------------------------------------------------------
      SUBROUTINE INPUT14(CX,CY,X,Y,KODE,BC,NFR,AFR)
C
      CHARACTER*80 TITLE
      DIMENSION CX(1),CY(1),X(1),Y(1),KODE(1),AFR(1)
      COMPLEX BC(1),XL(3),CON1,CON2,ET,GAMMA,TRO,TK
      COMPLEX AL(3),BL(3),CL(3)
      COMMON N,LN,INP,IPR,GE,XLAME,FR,XNU,ROS,ROF,ROA,AL,BL,CL
      COMMON BETA,SB,BQ,BR,RO11,RO12,RO22,XL,CON1,CON2,ET,GAMMA,TRO,TK

C
C  N= NUMBER OF BOUNDARY NODES (= NUMBER OF ELEMENTS)
C  LN= NUMBER OF INTERNAL POINTS WHERE DISPLACEMENT AND FLUID STRESS
C     ARE CALCULATED
C  ROS= SOLID DENSITY
C  ROF= FLUID DENSITY
C  ROA= ADDITIONAL DENSITY
C  BETA= POROSITY
C  NFR= NUMBER OF FREQUENCIES TO BE ANALYSED
C  AFR= ARRAY FOR STORAGE OF FREQUENCIES
C  FR=  FREQUENCY
C  GE=  SHEAR MODULUS
C  XNU= POISSONS RATIO
C  XLAME= LAME CONSTANT LAMBDA
C  XL(I)= WAVE NUMBER OF THE FIRST,THE SECOND OR THE TRANSVERSE WAVE
C
      WRITE(IPR,100)
  100 FORMAT(/' ',79('*'))
C
C  READ JOB TITLE
C
      READ(INP,'(A)') TITLE
      WRITE(IPR,'(A)') TITLE
C
C  READ NUMBER OF NODES, INTERNAL POINTS AND FREQUENCIES
C  READ MATERIAL PROPERTIES
C
      READ(INP,*) N,LN,NFR
      READ(INP,*) GE,XNU,ROS,ROF,ROA,BETA,SB,BQ,BR
      WRITE(IPR,300)N,LN,NFR,GE,XNU,ROS,ROF,ROA,BETA,SB,BQ,BR
  300 FORMAT(//' DATA'//2X,'NUMBER OF BOUNDARY ELEMENTS =',I3/2X,
     &'NUMBER OF INTERNAL POINTS =',I3/2X,
     &'NUMBER OF FREQUENCIES =',I3/2X,
     &'SHEAR MODULUS=',G14.7/2X,'POISSON RATIO=',G14.7/2X,
     &'SOLID DENSITY=',G14.7/2X,'FLUID DENSITY=',G14.7/2X,
     &'ADDITIONAL DENSITY=',G14.7/2X,'POROSITY=',G14.7/2X,
     &'BIOTS CONSTANT b RELATED TO PERMEABILITY=',G14.7/2X,
     &'BIOTS CONSTANT Q =',G14.7/2X,'BIOTS CONSTANT R =',G14.7/)

C
C  READ FREQUENCIES
C
      READ(INP,*) (AFR(I),I=1,NFR)
      WRITE(IPR,400) (AFR(I),I=1,NFR)
  400 FORMAT(/2X,'FREQUENCIES'/5(2X,G14.7))
C
C  READ COORDINATES OF THE EXTREME POINTS OF THE BOUNDARY
C  ELEMENTS IN ARRAYS X AND Y
C
      WRITE(IPR,500)
  500 FORMAT(//2X,'COORDINATES OF THE EXTREME POINTS OF ',
     1'THE BOUNDARY ELEMENTS'//4X,'POINT',10X,'X',18X,'Y')
      READ(INP,*) (X(I),Y(I),I=1,N)
      DO 10 I=1,N
   10 WRITE(IPR,700)I,X(I),Y(I)
  700 FORMAT(5X,I3,2(5X,G14.7))
C
C  READ BOUNDARY CONDITIONS IN BC(I) VECTOR; IF KODE(I)=0 THE
C  BC(I) VALUE IS A KNOWN COMPONENT OF THE SOLID DISPLACEMENT
C  OR IS A KNOWN VALUE OF THE FLUID STRESS;
C  IF KODE(I)=1 THE BC(I) VALUE IS A KNOWN COMPONENT OF THE SOLID
C  TRACTION OR IS A KNOWN VALUE OF THE FLUID NORMAL DISPLACEMENT
C
C  BOUNDARY CONDITIONS FOR THE SOLID
C
      WRITE(IPR,800)
  800 FORMAT(//2X,'BOUNDARY CONDITIONS FOR THE SOLID'//
     &18X,'PRESCRIBED VALUE',20X,'PRESCRIBED VALUE'/5X,
     &'NODE',11X,'X DIRECTION',10X,'CODE',11X,'Y DIRECTION',9X,'CODE')
      DO 20 I=1,N
      READ(INP,*) KODE(3*I-2),BC(3*I-2),KODE(3*I-1),BC(3*I-1)
   20 WRITE(IPR,900)I,BC(3*I-2),KODE(3*I-2),BC(3*I-1),KODE(3*I-1)
  900 FORMAT(5X,I3,3X,G14.7,',',G14.7,3X,I1,3X,G14.7,',',G14.7,3X,I1)
C
C  BOUNDARY CONDITIONS FOR THE FLUID
C
      WRITE(IPR,950)
  950 FORMAT(//25X,'BOUNDARY CONDITIONS FOR THE FLUID'//
     &23X,'NODE',9X,'PRESCRIBED VALUE',7X,'CODE')
      DO 30 I=1,N
      READ(INP,*) KODE(3*I),BC(3*I)
   30 WRITE(IPR,980)I,BC(3*I),KODE(3*I)
  980 FORMAT(23X,I3,3X,G14.7,',',G14.7,3X,I1)
C
C  READ COORDINATES OF THE INTERNAL POINTS
C
      IF(LN.EQ.0) RETURN
      READ(INP,*) (CX(I),CY(I),I=1,LN)
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE GHMAT14(X,Y,XM,YM,G,H,FI,DFI,KODE,NX)
C
C  THIS SUBROUTINE COMPUTES THE G AND H MATRICES AND
C  FORM THE SYSTEM OF EQUATIONS  A X = F
C
      COMPLEX G(NX,NX),H(NX,NX),GW(3,3),HW(3,3),FI(1),DFI(1),CH,XIM
      DIMENSION X(1),Y(1),XM(1),YM(1),KODE(1)
      COMPLEX AL(3),BL(3),CL(3),XL(3)
      COMPLEX CON1,CON2,ET,GAMMA,TRO,TK,T1,T2,T3,T4
      COMMON N,LN,INP,IPR,GE,XLAME,FR,XNU,ROS,ROF,ROA,AL,BL,CL
      COMMON BETA,SB,BQ,BR,RO11,RO12,RO22,XL,CON1,CON2,ET,GAMMA,TRO,TK

C
C  COMPUTE BASIC CONSTANTS
C

      XIM=CMPLX(0.,FR)
      RO1=ROS*(1.-BETA)
      RO2=ROF*BETA
      RO12=-ROA
      RO11=RO1+ROA
      RO22=RO2+ROA
      XLAME=2.*GE*XNU/(1.-2*XNU)
      CON1=1./(XIM*SB-FR*FR*RO22)
      CON2=CON1*(XIM*SB+FR*FR*RO12)
      ET=-SB*(1.+BQ/BR)+XIM*(RO12-RO22*BQ/BR)
      GAMMA=-BQ/BR-CON2
      TK=BR/(SB+XIM*RO22)
      TRO=(FR*FR*(RO12*RO12-RO11*RO22)+XIM*SB*(RO11+RO22+2.*RO12))*CON1
      T1=(TRO*FR*FR-XIM*GAMMA*ET)/(XLAME+2.*GE)-XIM/TK
      T2=-TRO*FR*FR*XIM/(XLAME+2.*GE)/TK
      XL(1)=CSQRT(0.5*(T1+CSQRT(T1*T1-4.*T2)))
      XL(2)=CSQRT(0.5*(T1-CSQRT(T1*T1-4.*T2)))
      XL(3)=CSQRT(TRO/GE)*FR
      T3=6.2831852*(XL(2)*XL(2)-XL(1)*XL(1))*(XLAME+2.*GE)
      AL(1)=(-1.-XIM/(TK*XL(1)*XL(1)))/T3
      AL(2)=(1.+XIM/(TK*XL(2)*XL(2)))/T3
      AL(3)=-1./(6.2831852*TRO*FR*FR)
      BL(1)=-1./T3
      BL(2)=-BL(1)
      BL(3)=(0.,0.)
      T4=T3/(XLAME+2.*GE)
      CL(1)=(TRO*FR*FR/(XLAME+2.*GE)-XL(1)*XL(1))/T4
      CL(2)=(XL(2)*XL(2)-TRO*FR*FR/(XLAME+2.*GE))/T4
      CL(3)=(0.,0.)
C
C  COMPUTE NODAL COORDINATES AND STORE IN ARRAYS XM AND YM
C
      X(N+1)=X(1)
      Y(N+1)=Y(1)
      DO 10 I=1,N
      XM(I)=(X(I)+X(I+1))/2
   10 YM(I)=(Y(I)+Y(I+1))/2
C
C  COMPUTE THE COEFFICIENTS OF G AND H MATRICES
C
      DO 30 I=1,N
      DO 30 J=1,N
   16 KK=J+1
   23 IF(I-J)20,25,20
   20 CALL EXTIN14(XM(I),YM(I),X(J),Y(J),X(KK),Y(KK),GW,HW)
      GO TO 26
   25 CALL LOCIN14(X(J),Y(J),X(KK),Y(KK),GW,HW)
   26 DO 28 II=1,3
      DO 28 JJ=1,3
      G(3*I-3+II,3*J-3+JJ)=GW(II,JJ)
   28 H(3*I-3+II,3*J-3+JJ)=HW(II,JJ)
   30 CONTINUE
C
C  REORDER THE COLUMNS OF THE SYSTEM OF EQUATIONS IN ACCORDANCE WITH THE
C  BOUNDARY CONDITIONS AND FORM SYSTEM MATRIX A WHICH IS STORED IN G
C
      NN=3*N
      DO 50 JN=1,N
      DO 45 JJ=1,3
      J=3*JN-3+JJ
      IF (KODE(J)) 35,35,40
   35 DO 38 I=1,NN
      CH=G(I,J)
      IF(JJ.LT.3) CH=CH*GE
      G(I,J)=-H(I,J)
   38 H(I,J)=-CH
      GO TO 45
   40 DO 42 I=1,NN
   42 IF(JJ.EQ.3) H(I,J)=H(I,J)*GE
   45 CONTINUE
   50 CONTINUE
C
C  FORM THE RIGHT HAND SIDE VECTOR F WHICH IS STORED IN FI
C
      DO 60 II=1,NN
      FI(II)=(0.,0.)
      DO 60 J=1,NN
      FI(II)=FI(II)+G(II,J)*DFI(J)
   60 CONTINUE
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE EXTIN14(XP,YP,X1,Y1,X2,Y2,GW,HW)
C
C  THIS SOUBROUTINE COMPUTES THE COEFFICIENTS OF THE MATRICES
C  G AND H THAT RELATE A COLLOCATION POINT WITH AN ELEMENT
C  TO WHICH THE POINT DOES NOT BELONG USING GAUSS QUADRATURE
C
C  ETA(I)        = COMPONENTS OF THE UNIT NORMAL TO THE ELEMENT
C  GI(I)         = GAUSS QUADRATURE POINTS
C  OME(I)        = GAUSS QUADRATURE WEIGHTS
C  XCO,YCO       = INTEGRATION POINTS ALONG THE ELEMENT
C
      DIMENSION GI(4),OME(4),ETA(2)
      COMPLEX GW(3,3),HW(3,3),U(3,3),T(3,3),AL(3),BL(3),CL(3)
      COMPLEX XL(3),CON1,CON2,ET,GAMMA,TRO,TK
      COMMON N,LN,INP,IPR,GE,XLAME,FR,XNU,ROS,ROF,ROA,AL,BL,CL
      COMMON BETA,SB,BQ,BR,RO11,RO12,RO22,XL,CON1,CON2,ET,GAMMA,TRO,TK
      DATA GI/0.86113631,-0.86113631,0.33998104,-0.33998104/
      DATA OME/0.34785485,0.34785485,0.65214515,0.65214515/

      AX=(X2-X1)/2
      BX=(X2+X1)/2
      AY=(Y2-Y1)/2
      BY=(Y2+Y1)/2
      SR=SQRT(AX**2+AY**2)
      ETA(1)=(Y2-Y1)/(2*SR)
      ETA(2)=(X1-X2)/(2*SR)

      DO 10 K=1,3
      DO 10 J=1,3
      GW(K,J)=(0.,0.)
   10 HW(K,J)=(0.,0.)

      DO 40 I=1,4
      XCO=AX*GI(I)+BX
      YCO=AY*GI(I)+BY
      CALL FSPORE(XP,YP,XCO,YCO,ETA,U,T)
      DO 30 K=1,3
      DO 30 J=1,3
      GW(K,J)=GW(K,J)+U(J,K)*OME(I)*SR
   30 HW(K,J)=HW(K,J)+T(J,K)*OME(I)*SR
   40 CONTINUE
      RETURN
      END

C-----------------------------------------------------------------------
      SUBROUTINE LOCIN14(X1,Y1,X2,Y2,GW,HW)
C
C  THIS SUBROUTINE COMPUTES THE COEFFICIENTS OF THE MATRICES G AND H
C  THAT RELATE AN ELEMENT WITH ITSELF
C
      DIMENSION ETA(2)
      COMPLEX GW(3,3),HW(3,3),AL(3),BL(3),CL(3),AUX
      COMPLEX XL(3),Z,FINK0,CON1,CON2,ET,GAMMA,XIM,TRO,TK
      COMPLEX F0Z,F1Z,F2Z,F0ZM,F1ZM,F2ZM
      COMMON N,LN,INP,IPR,GE,XLAME,FR,XNU,ROS,ROF,ROA,AL,BL,CL
      COMMON BETA,SB,BQ,BR,RO11,RO12,RO22,XL,CON1,CON2,ET,GAMMA,TRO,TK
C
C  COMPUTE BASIC PARAMETERS
C
      AX=(X2-X1)/2
      AY=(Y2-Y1)/2
      SR=SQRT(AX**2+AY**2)
      ETA(1)=(Y2-Y1)/(2.*SR)
      ETA(2)=(X1-X2)/(2.*SR)
      XIM=(0.,1.)

      DO 10 K=1,3
      DO 10 J=1,3
      GW(K,J)=(0.,0.)
   10 HW(K,J)=(0.,0.)
C
C  COMPUTE THE GW AND HW MATRICES BY SUMMATION
C  OVER THE THREE KINDS OF WAVES
C
      DO 90 L=1,3

      Z=XIM*XL(L)*SR
      CALL BESSEL(Z,F0Z,F1Z,F2Z,F0ZM,F1ZM,F2ZM)
      CALL INBESS(Z,FINK0)
      GW(1,1)=GW(1,1)+AL(L)*XL(L)*F1Z*2.*XIM
      GW(2,2)=GW(2,2)-AL(L)*XL(L)*2.*XIM*(FINK0+F1Z)
      IF(L.NE.3) GO TO 20
      AUX=FINK0/(XIM*XL(3)*3.1415926*GE)
      GW(1,1)=GW(1,1)+AUX
      GW(2,2)=GW(2,2)+AUX
   20 HW(2,3)=HW(2,3)-XL(L)*BL(L)*FR*ET*(FINK0+F1Z)
      GW(3,3)=GW(3,3)-CL(L)*2.*FINK0/(XIM*XL(L))
      HW(3,2)=HW(3,2)-XL(L)*BL(L)*GAMMA*2.*XIM*((XLAME+2.*GE)*FINK0
     &+2.*GE*F1Z)

   90 CONTINUE

      HW(2,3)=HW(2,3)*2.*CON1-GW(2,2)*CON2
      HW(3,3)=-CON1*0.5
      HW(1,1)=(0.5,0.)
      HW(2,2)=(0.5,0.)
      HW(3,2)=HW(3,2)-GW(3,3)*BQ/BR
C
C  CHANGE THE GW AND HW MATRICES INTO GLOBAL CO-ORDINATES
C

      HW(1,3)=HW(2,3)*ETA(1)
      HW(2,3)=HW(2,3)*ETA(2)

      GW(1,2)=(GW(2,2)-GW(1,1))*ETA(1)*ETA(2)
      GW(2,1)=GW(1,2)

      AUX=GW(1,1)
      GW(1,1)=GW(1,1)*ETA(2)*ETA(2)+GW(2,2)*ETA(1)*ETA(1)
      GW(2,2)=GW(2,2)*ETA(2)*ETA(2)+AUX*ETA(1)*ETA(1)

      HW(3,1)=HW(3,2)*ETA(1)
      HW(3,2)=HW(3,2)*ETA(2)

      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE FSPORE(XP,YP,XCO,YCO,ETA,U,T)
C
C  THIS SUBROUTINE COMPUTES THE FUNDAMENTAL SOLUTION TENSORS
C  THAT RELATE THE COLLOCATION POINT AND A POINT ON THE BOUNDARY
C
      COMPLEX A(3,3),B(3,3),C(3,3),D(3,3),E(3,3)
      COMPLEX AL(3),BL(3),CL(3),XL(3)
      COMPLEX XIM,Z,CON1,CON2,ET,GAMMA,TRO,TK,TAU(3,2)
      COMPLEX U(3,3),T(3,3),STN(2,3),ST(2,2,3),DILA(3)
      COMPLEX F0Z,F1Z,F2Z,F0ZM,F1ZM,F2ZM
      DIMENSION RD(3),ETA(2)
      COMMON N,LN,INP,IPR,GE,XLAME,FR,XNU,ROS,ROF,ROA,AL,BL,CL
      COMMON BETA,SB,BQ,BR,RO11,RO12,RO22,XL,CON1,CON2,ET,GAMMA,TRO,TK
C
C  RA =RADIUS
C  RD(I) = RADIUS DERIVATIVES
C  A,B,C,D,E = MATRICES OF COEFFICIENTS OF THE TERMS THAT FORM
C              THE FUNDAMENTAL SOLUTION MATRIX.
C  A,B,C,D AND E ARE DIFFERENT FOR EACH KIND OF WAVE
C
      XIM=(0.,1.)
      RA=SQRT((XCO-XP)**2+(YCO-YP)**2)
      RD(1)=(XCO-XP)/RA
      RD(2)=(YCO-YP)/RA
      RD(3)=0.
      DO 10 K=1,3
      DO 10 J=1,3
      A(J,K)=(0.,0.)
      B(J,K)=(0.,0.)
      C(J,K)=(0.,0.)
      D(J,K)=(0.,0.)
      E(J,K)=(0.,0.)
      U(J,K)=(0.,0.)
   10 T(J,K)=(0.,0.)
C
C  SUMMATION OVER THE THREE KINDS OF WAVES
C
      DO 90 L=1,3
C
C  SET THE VALUES OF THE A,B,C,D AND E MATRICES FOR THE CORRESPONDING L
C
      A(3,3)=CL(L)
      IF(L.NE.3) GO TO 20
      A(1,1)=1./(6.2831852*GE)
      A(2,2)=A(1,1)
   20 B(3,1)=-XL(L)*BL(L)*FR*ET
      B(3,2)=B(3,1)
      C(1,3)=XIM*XL(L)*BL(L)*GAMMA
      C(2,3)=C(1,3)
      D(1,1)=XIM*XL(L)*AL(L)
      D(2,2)=D(1,1)
      E(1,1)=XL(L)*XL(L)*AL(L)
      E(1,2)=E(1,1)
      E(2,1)=E(1,1)
      E(2,2)=E(1,1)

      Z=XIM*XL(L)*RA
      CALL BESSEL(Z,F0Z,F1Z,F2Z,F0ZM,F1ZM,F2ZM)

      DO 70 J=1,3
C
C  COMPUTE DISPLACEMENTS FOR THE SOLID AND STRESS FOR THE FLUID
C
      DO 30 K=1,2
   30 U(K,J)=U(K,J)+A(K,J)*F0Z+(B(K,J)*RD(J)+C(K,J)*RD(K)+D(K,J)/RA)*F1Z
     1+E(K,J)*RD(K)*RD(J)*F2Z
      U(3,J)=U(3,J)-A(3,J)*F0Z-(B(3,J)*RD(J)+C(3,J)*RD(3)+D(3,J)/RA)*F1Z
     1-E(3,J)*RD(3)*RD(J)*F2Z

C
C  COMPUTE THE STRAIN TENSOR AND THE DILATATION OF THE SOLID
C
      DILA(J)=(0.,0.)
      DO 50 K=1,2
      STN(K,J)=(0.,0.)
      DO 40 M=1,2
      IF(J.EQ.M) DJM=1.
      IF(J.NE.M) DJM=0.
      IF(J.EQ.K) DJK=1.
      IF(J.NE.K) DJK=0.
      IF(K.EQ.M) DKM=1.
      IF(K.NE.M) DKM=0.
      ST(K,M,J)=RD(K)*(-A(M,J)*XIM*XL(L)*F1Z+E(K,J)*DJM*F2Z/RA
     &-D(M,J)*XIM*XL(L)*F2Z/RA)+RD(M)*(-A(K,J)*XIM*XL(L)*F1Z
     &+E(M,J)*DJK*F2Z/RA-D(K,J)*XIM*XL(L)*F2Z/RA)
     &+RD(J)*(E(K,J)+E(M,J))*DKM*F2Z/RA
     &-RD(K)*RD(M)*(C(K,J)+C(M,J))*XIM*XL(L)*F2Z
     &-RD(K)*RD(J)*B(M,J)*XIM*XL(L)*F2Z
     &-RD(M)*RD(J)*B(K,J)*XIM*XL(L)*F2Z
     &-RD(M)*RD(K)*RD(J)*(E(K,J)+E(M,J))*(4.*F2Z/RA+XIM*XL(L)*F1Z)
     &+(B(K,J)*DJM+B(M,J)*DJK+C(K,J)*DKM+C(M,J)*DKM)*F1Z/RA
   40 STN(K,J)=STN(K,J)+ST(K,M,J)*ETA(M)/2.
   50 DILA(J)=DILA(J)+ST(K,K,J)/2.
C
C  COMPUTE THE TRACTIONS ON THE SOLID
C
      DO 55 K=1,2
   55 T(K,J)=T(K,J)+XLAME*DILA(J)*ETA(K)+2.*GE*STN(K,J)
C
C  COMPUTE THE NORMAL DISPLACEMENT OF THE FLUID
C
      IF(J.NE.3) D3J=0.
      IF(J.EQ.3) D3J=1.
      DO 60 M=1,2
      IF(J.NE.M) DJM=0.
      IF(J.EQ.M) DJM=1.
   60 TAU(J,M)=-A(3,J)*F1Z*XIM*XL(L)*RD(M)+B(3,J)*F1Z*DJM/RA
     1-B(3,J)*F2Z*RD(M)*RD(J)*XIM*XL(L)
   70 T(3,J)=T(3,J)-CON1*(TAU(J,1)*ETA(1)+TAU(J,2)*ETA(2))
   90 CONTINUE
      DO 110 J=1,3
      T(1,J)=T(1,J)-U(3,J)*BQ*ETA(1)/BR
      T(2,J)=T(2,J)-U(3,J)*BQ*ETA(2)/BR
  110 T(3,J)=T(3,J)-(U(1,J)*ETA(1)+U(2,J)*ETA(2))*CON2
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE INTER14(FI,DFI,KODE,CX,CY,X,Y,DSOL)
C
C  THIS SUBROUTINE COMPUTES THE VALUES OF THE DISPLACEMENT COMPONENTS
C  FOR THE SOLID AND THE FLUID STRESS AT INTERNAL POINTS
C
      COMPLEX GW(3,3),HW(3,3),FI(1),DFI(1),DSOL(1),CH
      DIMENSION X(1),Y(1),CX(1),CY(1),KODE(1)
      COMPLEX AL(3),BL(3),CL(3),XL(3)
      COMPLEX CON1,CON2,ET,GAMMA,TRO,TK
      COMMON N,LN,INP,IPR,GE,XLAME,FR,XNU,ROS,ROF,ROA,AL,BL,CL
      COMMON BETA,SB,BQ,BR,RO11,RO12,RO22,XL,CON1,CON2,ET,GAMMA,TRO,TK

C
C  REARRANGE FI AND DFI ARRAYS TO STORE ALL THE VALUES OF THE
C  SOLID DISPLACEMENTS AND FLUID STRESS IN FI AND ALL THE
C  VALUES OF THE TRACTIONS AND FLUID NORMAL DISPLACEMENT IN DFI
C
      DO 23 IN=1,N
      DO 20 II=1,3
      I=3*IN-3+II
      IF(KODE(I)) 10,10,15
   10 CH=FI(I)
      IF(II.LT.3) CH=CH*GE
      FI(I)=DFI(I)
      DFI(I)=CH
      GO TO 20
   15 IF(II.EQ.3) FI(I)=FI(I)*GE
   20 CONTINUE
   23 CONTINUE
C
C  COMPUTE THE VALUES OF THE SOLID DISPLACEMENT AND FLUID STRESS
C  AT INTERNAL POINTS
C
      IF(LN.LE.0) RETURN
C
      DO 40 K=1,LN
      DSOL(3*K-2)=(0.,0.)
      DSOL(3*K-1)=(0.,0.)
      DSOL(3*K)=(0.,0.)
      DO 30 J=1,N
      KK=J+1
      CALL EXTIN14(CX(K),CY(K),X(J),Y(J),X(KK),Y(KK),GW,HW)
      DO 25 II=1,3
      MR=3*(K-1)+II
      DO 25 JJ=1,3
      MC=3*(J-1)+JJ
   25 DSOL(MR)=DSOL(MR)+DFI(MC)*GW(II,JJ)-FI(MC)*HW(II,JJ)
   30 CONTINUE
   40 DSOL(3*K)=-DSOL(3*K)/CON1
      RETURN
      END
C ----------------------------------------------------------------------
      SUBROUTINE OUTPUT14(XM,YM,FI,DFI,CX,CY,DSOL)
C
C  THIS SUBROUTINE PRINTS THE VALUES OF THE DISPLACEMENTS AND
C  TRACTIONS AND FLUID STRESS AT BOUNDARY NODES.
C  IT ALSO PRINTS THE VALUES OF THE SOLID DISPLACEMENTS
C  AND FLUID STRESS AT INTERNAL POINTS
C
      COMPLEX FI(1),DFI(1),DSOL(1)
      DIMENSION XM(1),YM(1),CX(1),CY(1)
      COMPLEX AL(3),BL(3),CL(3),XL(3)
      COMPLEX CON1,CON2,ET,GAMMA,TRO,TK
      COMMON N,LN,INP,IPR,GE,XLAME,FR,XNU,ROS,ROF,ROA,AL,BL,CL
      COMMON BETA,SB,BQ,BR,RO11,RO12,RO22,XL,CON1,CON2,ET,GAMMA,TRO,TK

C
      WRITE(IPR,50)
   50 FORMAT(' ',79('*')/)
      WRITE(IPR,700) FR
  700 FORMAT(30X,'FREQUENCY=',G14.7/)
      WRITE(IPR,100)
  100 FORMAT(' ',79('*')//1X,'RESULTS'//2X,'BOUNDARY NODES'//6X,'X',11X
     &,'Y',8X,'SOLID DISPLACEMENT X',4X,'SOLID DISPLACEMENT Y'/)
      DO 10 I=1,N
   10 WRITE(IPR,200) XM(I),YM(I),FI(3*I-2),FI(3*I-1)
      WRITE(IPR,120)
  120 FORMAT(/6X,'X',11X,'Y',12X,'FLUID STRESS'/)
      DO 12 I=1,N
   12 WRITE(IPR,200) XM(I),YM(I),FI(3*I)
      WRITE(IPR,150)
  150 FORMAT(/6X,'X',11X,'Y',10X,'SOLID TRACTION X',8X,
     &'SOLID TRACTION Y'/)
      DO 15 I=1,N
   15 WRITE(IPR,200) XM(I),YM(I),DFI(3*I-2),DFI(3*I-1)
      WRITE(IPR,170)
  170 FORMAT(/6X,'X',11X,'Y',5X,'FLUID NORMAL DISPLACEMENT'/)
      DO 17 I=1,N
   17 WRITE(IPR,200) XM(I),YM(I),DFI(3*I)
  200 FORMAT(2(1X,G11.4),2(1X,G11.4,',',G11.4))
C
      IF(LN.LE.0) GO TO 60
      WRITE(IPR,300)
  300 FORMAT(//2X,'INTERNAL POINTS'//6X,'X',11X,'Y',8X,'SOLID
     &DISPLACEMENT X',7X,'SOLID DISPLACEMENT Y')
      DO 20 K=1,LN
   20 WRITE(IPR,200)CX(K),CY(K),DSOL(3*K-2),DSOL(3*K-1)
      WRITE(IPR,400)
  400 FORMAT(/6X,'X',11X,'Y',12X,'FLUID STRESS')
      DO 30 K=1,LN
   30 WRITE(IPR,200)CX(K),CY(K),DSOL(3*K)
   60 WRITE(IPR,500)
  500 FORMAT(' ',79('*'))
      RETURN
      END

C-----------------------------------------------------------------------
      SUBROUTINE BESSEL(Z,FK0,FK1,FK2,FK0M,FK1M,FK2M)
C
C
C  THIS SUBROUTINE COMPUTES THE MODIFIED BESSEL FUNCTIONS OF THE SECOND
C  KIND AND ORDERS ZERO, ONE AND TWO (FK0,FK1,FK2) AND COMPLEX ARGUMENT
C  Z USING A POLINOMIUM
C
C  THE COEFFICIENTS OF THE POLINOMIUM HAVE BEEN OBTAINED FROM THE
C  TERMS OF THE SERIES EXPANSION
C
C  TWO DIFFEERENT SERIES HAVE BEEN USED. ONE FOR MODULUS(Z).LT.5
C  AND THE OTHER FOR MODULUS(Z).GT.5
C
C  FOR SMALL ARGUMENTS THE PARTS OF THE MODIFIED BESSEL FUNCTIONS
C  TENDING TO ZERO WITH THE ARGUMENT ARE COMPUTED SEPARATEDLY FROM
C  THE BESSEL FUNCTIONS AND STORED IN FK0M,FK1M AND FK2M. THESE
C  PARTS ARE ONLY USED BY THE QUADRATIC ELEMENT CODES IN ORDER
C  TO COMPUTE THE DIFFERENCE BETWEEN THE DYNAMIC AND THE STATIC
C  FUNDAMENTAL SOLUTION TRACTIONS

C
      COMPLEX  Z,FK0,FK1,FK2,ZZ,SER,FK0M,FK1M,FK2M,FI0M,FI1M
      REAL     CI0P(15),CI1P(15),CK0P(15),CK0G(12),CK1P(15),CK1G(12)
C
      DATA CI0P/ 1.00000000E+00, 0.62500000E+01, 0.97656250E+01,
     &           0.67816840E+01, 0.26490953E+01, 0.66227383E+00,
     &           0.11497810E+00, 0.14665573E-01, 0.14321849E-02,
     &           0.11050809E-03, 0.69067559E-05, 0.35675392E-06,
     &           0.15484111E-07, 0.57263725E-09, 0.18260116E-10/
      DATA CI1P/ 0.50000000E+00, 0.15625000E+01, 0.16276042E+01,
     &           0.84771050E+00, 0.26490953E+00, 0.55189486E-01,
     &           0.82127211E-02, 0.91659834E-03, 0.79565828E-04,
     &           0.55254047E-05, 0.31394345E-06, 0.14864747E-07,
     &           0.59554274E-09, 0.20451330E-10, 0.60867054E-12/
      DATA CK0P/-0.57721566E+00, 0.26424021E+01, 0.90115658E+01,
     &           0.85185931E+01, 0.39898493E+01, 0.11299171E+01,
     &           0.21532918E+00, 0.29560538E-01, 0.30657944E-02,
     &           0.24883689E-03, 0.16242981E-04, 0.87142913E-06,
     &           0.39112788E-07, 0.14905278E-08, 0.48833881E-10/
      DATA CK1P/ 1.00000000E+00, 0.96519581E+00,-0.26280638E+02,
     &          -0.44329875E+02,-0.29269699E+02,-0.10636897E+02,
     &          -0.24689720E+01,-0.39918196E+00,-0.47620526E-01,
     &          -0.43685559E-02,-0.31795287E-03,-0.18814687E-04,
     &          -0.92322279E-06,-0.38181087E-07,-0.13490886E-08/
      DATA CK0G/ 0.12533141E+01,-0.31332853E-01, 0.35249460E-02,
     &          -0.73436375E-03, 0.22489890E-03,-0.91084054E-04,
     &           0.45921544E-04,-0.27716932E-04, 0.19488468E-04,
     &          -0.15644909E-04, 0.14119530E-04,-0.14151620E-04/
      DATA CK1G/ 0.12533141E+01, 0.93998560E-01,-0.58749100E-02,
     &           0.10281093E-02,-0.28915573E-03, 0.11132496E-03,
     &          -0.54270916E-04, 0.31981075E-04,-0.22086930E-04,
     &           0.17485486E-04,-0.15605797E-04, 0.15499393E-04/
C
      R=CABS(Z)
      RR=5.0
C
      IF(R-RR) 10,20,20
C
   10 ZZ=Z/CMPLX(RR,0.)
      ZZ=ZZ*ZZ
C
C  COMPUTE THE NUMBER OF TERMS TO BE USED IN THE SERIES
C  FOR SMALL ARGUMENTS
C
      N=MAX0(2,MIN1(15.,24.177/ALOG(RR/R)))
C
C  COMPUTE  I0(Z)-1.
C
      FI0M=SER(CI0P,ZZ,2,N)
C
C  COMPUTE I1(Z)-Z/2
C
      FI1M=Z*SER(CI1P,ZZ,2,N)
C
C  COMPUTE K0M(Z) AND K0(Z)
C
      FK0M=-FI0M*CLOG(Z/(2.,0.))+SER(CK0P,ZZ,2,N)
      FK0=FK0M-CLOG(Z/(2.,0.))+CMPLX(CK0P(1),0.)
C
C  COMPUTE K1M(Z) AND K1(Z)
C
      FK1M=(FI1M*Z*CLOG(Z/(2.,0.))+SER(CK1P,ZZ,2,N))/Z
     &+CMPLX(CK0P(1)+0.5,0.)*Z/(2.,0.)
      FK1=FK1M+(1.,0.)/Z+Z*(CLOG(Z/(2.,0.))-CMPLX(CK0P(1)+0.5,0.))/2
C
C  COMPUTE K2M(Z) AND K2(Z)
C
      FK2M=FK0M+FK1M*(2.,0.)/Z
      FK2=FK0+FK1*(2.,0.)/Z
      GO TO 30

   20 ZZ=CMPLX(RR,0.)/Z
C
C  COMPUTE THE NUMBER OF TERMS TO BE USED IN THE SERIES
C  FOR LARGE ARGUMENTS
C
C  IN THIS CASE FK0M,FK1M AND FK2M ARE COMPUTED IN ORDER
C  TO BE CONSISTENT IN THE NUMERICAL INTEGRATION PROCESS
C
      N=MAX0(2,MIN1(12.,12.088/ALOG(R/RR)))
C
C  COMPUTE K0(Z) AND K0M(Z)
C
      FK0=(CMPLX(CK0G(1),0.)+SER(CK0G,ZZ,2,N))*CEXP(-Z)/CSQRT(Z)
      FK0M=FK0+CLOG(Z/(2.,0.))-CMPLX(CK0P(1),0.)
C
C  COMPUTE K1(Z) AND K1M(Z)
C
      FK1=(CMPLX(CK1G(1),0.)+SER(CK1G,ZZ,2,N))*CEXP(-Z)/CSQRT(Z)
      FK1M=FK1-(1.,0.)/Z-Z*(CLOG(Z/(2.,0.))-CMPLX(CK0P(1)+0.5,0.))/2
C
C  COMPUTE K2(Z) AND K2M(Z)
C
      FK2=FK0+FK1*(2.,0.)/Z
      FK2M=FK0M+FK1M*(2.,0.)/Z
C
   30 RETURN
      END
C-----------------------------------------------------------------------
      COMPLEX FUNCTION SER(S,ZZ,N1,N2)
C
C  THIS FUNCTION COMPUTES THE VALUE OF A SERIES OF N2-N1+1
C  TERMS CONSISTING OF INCREASING POWERS OF ZZ TIMES THE
C  COEFFICIENTS IN ARRAY S
C
      COMPLEX  ZZ,ZZZ
      REAL     S(N2)
C
      SER=(0.,0.)
      ZZZ=(1.,0.)
      DO 10 I=N1,N2
      ZZZ=ZZZ*ZZ
      SER=SER+CMPLX(S(I),0.)*ZZZ
   10 CONTINUE
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE INBESS(Z,FINK0)
C
C  THIS SUBROUTINE COMPUTES THE INTEGRAL OF THE MODIFIED BESSEL FUNCTION
C  OF ZERO ORDER BETWEEN 0 AND Z USING A POLINOMIUM.
C
C  THE COEFFICIENTS OF THE POLINOMIUM HAVE BEEN OBTAINED FROM THE
C  TERMS OF THE SERIES EXPANSION
C
      COMPLEX  Z,FINK0,ZZ,SER
      DIMENSION S1(20),S2(20)
C
      DATA S0  / 0.42278434E+00/
      DATA S1  / 0.90734120E+01, 0.72756425E+02, 0.25901019E+03,
     &           0.52398212E+03, 0.68598066E+03, 0.62976149E+03,
     &           0.42826262E+03, 0.22451590E+03, 0.93540077E+02,
     &           0.31723213E+02, 0.89292258E+01, 0.21196877E+01,
     &           0.43013488E+00, 0.75474799E-01, 0.11565621E-01,
     &           0.15612002E-02, 0.18705605E-03, 0.20027866E-04,
     &           0.19277885E-05, 0.16772276E-06/
      DATA S2  / 0.12000000E+02, 0.64800000E+02, 0.18514286E+03,
     &           0.32400000E+03, 0.38173091E+03, 0.32300308E+03,
     &           0.20566727E+03, 0.10207750E+03, 0.40592223E+02,
     &           0.13221467E+02, 0.35916023E+01, 0.82606852E+00,
     &           0.16293265E+00, 0.27862515E-01, 0.41703893E-02,
     &           0.55091790E-03, 0.64704940E-04, 0.68008195E-05,
     &           0.64341868E-06, 0.55082916E-07/
C
      R=CABS(Z)
      RR=12.0
      IF(R-RR) 10,20,20
  10  N=MAX0(1,MIN1(20.,19.572/ALOG(RR/R)))
      GO TO 30
  20  N=20
      WRITE(6,*) ' INBESS: RESULTS MAY BE INACCURATE '
C
  30  ZZ=Z/CMPLX(RR,0.)
      ZZ=ZZ*ZZ
C
      FINK0=-CLOG(Z/(2.,0.))*((1.,0.)+SER(S2,ZZ,1,N))
      FINK0=Z*(CMPLX(S0,0.)+SER(S1,ZZ,1,N)+FINK0)
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE CSOLVER(A,B,N,NX)
C
      COMPLEX A(NX,NX),B(NX),X,CDOT
C
C     A=L*U  FACTORIZATION
C
      DO 30 K=1,N
      K1=K-1
      DO 10 I=K,N
      A(I,K)=A(I,K)-CDOT(K1,A(I,1),NX,A(1,K),1)
   10 CONTINUE
      X=A(K,K)
      IF(ABS(REAL(X))+ABS(AIMAG(X)).EQ.0.) THEN
      WRITE(6,*) ' SOLVER : STOP  SINGULARITY IN ROW ',K
      STOP ' SOLVER : STOP SINGULARITY '
      END IF
      X=(1.,0.)/X
      DO 20 J=K+1,N
      A(K,J)=X*(A(K,J)-CDOT(K1,A(K,1),NX,A(1,J),1))
   20 CONTINUE
   30 CONTINUE
C
C  FORWARD AND BACK SUBSTITUTION    L*U*X=B
C
C  SOLVE  L*Y=B
C
      DO 40 I=1,N
      B(I)=-(CDOT((I-1),A(I,1),NX,B,1)+B(I))/A(I,I)
   40 CONTINUE
C
C  SOLVE  U*X=Y
C
      DO 50 I=N,1,-1
      B(I)=-(CDOT((N-I),A(I,I+1),NX,B(I+1),1)+B(I))
   50 CONTINUE
C
      RETURN
      END
C-----------------------------------------------------------------------
      COMPLEX FUNCTION CDOT(N,U,IUX,V,IVX)
C
C THIS FUNCTION MAKES THE COMPLEX DOT PRODUCT OF TWO VECTORS
C
      COMPLEX U(1),V(1)
C
      CDOT=(0.,0.)
      IF(N.LE.0) RETURN
      IU=1
      IV=1
      IF(IUX.LT.0) IU=1-(N-1)*IUX
      IF(IVX.LT.0) IV=1-(N-1)*IVX
      DO 10 I=1,N
      CDOT=CDOT+U(IU)*V(IV)
      IU=IU+IUX
      IV=IV+IVX
10    CONTINUE
      RETURN
C
      END
