C-----------------------------------------------------------------------
      PROGRAM CONDTHEH
C
C  THIS IS A B.E. PROGRAM THAT USES (CON)STANT ELEMENTS FOR THE
C  SOLUTION OF (D)YNAMIC (TH)REE-DIMENSIONAL (E)LASTIC OR VISCOELASTIC
C  TIME (H)ARMONIC PROBLEMS
C
C
      PARAMETER(NE=57,NX=3*NE,NPIX=10,NFRX=10,NCOX=80)
      CHARACTER*12 FILEIN,FILEOUT
C
      COMPLEX     G(NX,NX),H(NX,NX)
      COMPLEX     BC(NX),FI(NX),DFI(NX),DSOL(3*NPIX)
      COMMON/MATG/G
      COMMON/MATH/H
      COMPLEX     CP,CS,GE,C1,C2,C3
      DIMENSION   DELTA(3,3)
      COMMON      N,NP,L,CP,CS,GE,RO,XNU,FR,INP,IPR
      COMMON      C1,C2,C3,C4,C5,C6,DELTA
      DIMENSION   X(NCOX),Y(NCOX),Z(NCOX),XM(NE),YM(NE),ZM(NE)
      INTEGER     CONE(NE,4)
      DIMENSION   KODE(NX),CX(NPIX),CY(NPIX),CZ(NPIX),AFR(NFRX)
C
C  NE   = MAXIMUM NUMBER OF BOUNDARY ELEMENT
C  NX   = MAXIMUM DIMENSION OF THE SYSTEM OF EQUATIONS
C  NPIX = MAXIMUM NUMBER OF INTERNAL POINTS
C  NFRX = MAXIMUM NUMBER OF FREQUENCIES
C  NCOX = MAXIMUM NUMBER OF CORNER POINTS
C
C  ASSIGN NUMBERS FOR INPUT AND OUTPUT FILES
C
      INP=5
      IPR=6
C
C  READ NAMES AND OPEN FILES FOR INPUT AND OUTPUT
C
      WRITE(*,'(A,$)') ' NAME OF THE INPUT  FILE (MAX. 12 CHART.) : '
      READ(*,'(A)') FILEIN
      OPEN(INP,FILE=FILEIN,STATUS='OLD')
      WRITE(*,'(A,$)') ' NAME OF THE OUTPUT FILE (MAX. 12 CHART.) : '
      READ(*,'(A)') FILEOUT
      OPEN(IPR,FILE=FILEOUT,STATUS='NEW')
C
C  READ DATA
C
      CALL INPUT7(CX,CY,CZ,X,Y,Z,KODE,BC,NFR,AFR,NE,CONE,XM,YM,ZM)
      NN=3*N
      DO 60 I=1,NFR
      FR=AFR(I)
      DO 40 J=1,NN
      DFI(J)=BC(J)
  40  CONTINUE
C
C  FORM SYSTEM OF EQUATIONS
C
       CALL GHMAT7(X,Y,Z,XM,YM,ZM,G,H,FI,DFI,KODE,NX,NE,CONE)
C
C  SOLVE SYSTEM OF EQUATIONS
C
      CALL CSOLVER(H,FI,NN,NX)
C
C  COMPUTE DISPLACEMENT OF INTERNAL POINTS
C
      CALL INTER7(FI,DFI,KODE,CX,CY,CZ,X,Y,Z,DSOL,NE,CONE)
C
C  PRINT RESULTS AT BOUNDARY NODES AND INTERNAL POINTS
C
      CALL OUTPUT7(FI,DFI,DSOL)
   60 CONTINUE
C
      CLOSE (INP)
      CLOSE (IPR)
C
      STOP 'END CONDTHEH'
      END
C-----------------------------------------------------------------------
      SUBROUTINE INPUT7(CX,CY,CZ,X,Y,Z,KODE,BC,NFR,AFR,NE,CONE,XM,YM,ZM)
C
      CHARACTER*80 TITLE
      DIMENSION    CX(1),CY(1),CZ(1),X(1),Y(1),Z(1),KODE(1),AFR(1)
      DIMENSION    XM(1),YM(1),ZM(1)
      INTEGER      CONE(NE,4)
      COMPLEX      BC(1)
      COMPLEX      CP,CS,GE,C1,C2,C3
      DIMENSION    DELTA(3,3)
      COMMON       N,NP,L,CP,CS,GE,RO,XNU,FR,INP,IPR
      COMMON       C1,C2,C3,C4,C5,C6,DELTA
C
C  N=   NUMBER OF BOUNDARY ELEMENTS
C  NP=  NUMBER OF ELEMENT CORNER POINTS
C  L=   NUMBER OF INTERNAL POINTS WHERE DISPLACEMENTS
C       ARE CALCULATED
C  CP=  P-WAVE VELOCITY
C  CS=  S-WAVE VELOCITY
C  RO=  DENSITY
C  NFR= NUMBER OF FREQUENCIES TO BE ANALYSED
C  AFR= ARRAY FOR STORAGE OF FREQUENCIES
C  FR=  FREQUENCY
C  GE=  SHEAR MODULUS
C  XNU= POISSON MODULUS
C
      WRITE(IPR,100)
  100 FORMAT(/' ',79('*'))
C
C  READ JOB TITLE
C
      READ(INP,'(A)') TITLE
      WRITE(IPR,'(/A)') TITLE
      WRITE(IPR,100)
C
C  READ NUMBER OF ELEMENTS, CORNER POINTS, INTERNAL POINTS AND FREQUENCIES
C  READ MATERIAL PROPERTIES
C
      READ(INP,*) N,NP,L,NFR,RGE,DAM,RO,XNU
      GE=CMPLX(RGE,RGE*2.*DAM)
      CS=CSQRT(GE/RO)
      CP=CS*SQRT((2.-2.*XNU)/(1.-2.*XNU))
      WRITE(IPR,300) N,NP,L,NFR,CP,CS,GE,DAM,RO,XNU
  300 FORMAT(//' DATA'//2X,
     1'NUMBER OF BOUNDARY ELEMENTS =',I3/2X,
     2'NUMBER OF ELEMENT CORNER POINTS    =',I3/2X,
     3'NUMBER OF INTERNAL POINTS   =',I3/2X,
     4'NUMBER OF FREQUENCIES       =',I3/2X,
     5'P-WAVE VELOCITY             =',G14.7,',',G14.7/2X,
     6'S-WAVE VELOCITY             =',G14.7,',',G14.7/2X,
     7'SHEAR MODULUS               =',G14.7,',',G14.7/2X,
     8'DAMPING RATIO               =',G14.7/2X,
     9'DENSITY                     =',G14.7/2X,
     A'POISSON RATIO               =',G14.7)
C
C  READ FREQUENCIES
C
      READ(INP,*) (AFR(I),I=1,NFR)
      WRITE(IPR,400) (AFR(I),I=1,NFR)
  400 FORMAT(/2X,'FREQUENCIES'/5(2X,G14.7))
C
C  READ COORDINATES OF THE CORNER POINTS OF THE BOUNDARY
C  ELEMENTS IN ARRAYS X, Y AND Z
C
      WRITE(IPR,500)
  500 FORMAT(//2X,'COORDINATES OF THE CORNER POINTS OF ',
     1'THE BOUNDARY ELEMENTS'//4X,'POINT',10X,'X',18X,'Y',17X,' Z')
      READ(INP,*) (X(I),Y(I),Z(I),I=1,NP)
      WRITE(IPR,700) (I,X(I),Y(I),Z(I),I=1,NP)
  700 FORMAT(5X,I3,3G19.7)
C
C  READ THE CONNECTIVITY OF EACH ELEMENT
C  TRIANGULAR ELEMENTS HAVE THE SAME CONNECTIVITY FOR THE LAST
C  TWO CORNERS
C
      WRITE(IPR,740)
  740 FORMAT(//2X,'CONECTIVITY OF THE BOUNDARY ELEMENTS'//
     11X,'ELEMENT',1X,'CORNER I',1X,'CORNER J',1X,'CORNER K'
     2,1X,'CORNER L',4X,'MID - X',5X,'MID - Y',5X,'MID - Z')
      DO 18 I=1,N
      READ(INP,*) (CONE(I,J),J=1,4)
C
C  COMPUTE NODAL COORDINATES AND STORE IN ARRAYS XM, YM AND ZM
C
      IF(CONE(I,4).EQ.0) CONE(I,4)=CONE(I,3)
      N1=CONE(I,1)
      N2=CONE(I,2)
      N3=CONE(I,3)
      N4=CONE(I,4)
      IF(N3-N4) 12,14,12
C
C  QUADRILATERAL ELEMENTS
C
   12 XM(I)=(X(N1)+X(N2)+X(N3)+X(N4))*0.25
      YM(I)=(Y(N1)+Y(N2)+Y(N3)+Y(N4))*0.25
      ZM(I)=(Z(N1)+Z(N2)+Z(N3)+Z(N4))*0.25
      GO TO 16
C
C  TRIANGULAR ELEMENTS
C
   14 XM(I)=(X(N1)+X(N2)+X(N3))*0.3333333
      YM(I)=(Y(N1)+Y(N2)+Y(N3))*0.3333333
      ZM(I)=(Z(N1)+Z(N2)+Z(N3))*0.3333333
   16 CONTINUE
      WRITE(IPR,780) I,(CONE(I,J),J=1,4),XM(I),YM(I),ZM(I)
  780 FORMAT(I6,4I9,2X,3G12.4)
   18 CONTINUE
C
C  READ BOUNDARY CONDITIONS IN BC(I) VECTOR; IF KODE(I)=0 THE
C  BC(I) VALUE IS A KNOWN DISPLACEMENT, IF KODE(I)=1 THE BC(I)
C  VALUE IS A KNOWN TRACTION
C
      WRITE(IPR,800)
  800 FORMAT(//2X,'BOUNDARY CONDITIONS'//8X,3('PRESCRIBED  VALUE',8X)
     1/1X,'ELEM',6X,'X DIRECTION',4X,'CODE',6X,'Y DIRECTION',4X,'CODE',
     26X,'Z DIRECTION',4X,'CODE')
      DO 20 I=1,N
      READ(INP,*) (KODE(3*(I-1)+J),BC(3*(I-1)+J),J=1,3)
   20 WRITE(IPR,950) I,(BC(3*(I-1)+J),KODE(3*(I-1)+J),J=1,3)
  950 FORMAT(I4,3(1X,G10.3,',',G10.3,I3))
C
C  READ COORDINATES OF THE INTERNAL POINTS
C
      IF(L) 40,40,30
   30 WRITE(IPR,1000)
 1000 FORMAT(//2X,'COORDINATES OF THE INTERNAL POINTS ',
     1//4X,'POINT',10X,'X',18X,'Y',17X,' Z')
      READ(INP,*) (CX(I),CY(I),CZ(I),I=1,L)
      WRITE(IPR,700) (I,CX(I),CY(I),CZ(I),I=1,L)
   40 CONTINUE
C
      WRITE(IPR,100)
c
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE GHMAT7(X,Y,Z,XM,YM,ZM,G,H,FI,DFI,KODE,NX,NE,CONE)
C
C  THIS SUBROUTINE COMPUTES THE G AND H MATRICES AND
C  FORM THE SYSTEM OF EQUATIONS  A X = F
C
      COMPLEX   G(NX,NX),H(NX,NX),FI(1),DFI(1),CH
      COMPLEX   GW(3,3),HW(3,3),HDIF(3,3)
      DIMENSION X(1),Y(1),Z(1),XM(1),YM(1),ZM(1)
      DIMENSION KODE(1),ETA(3),CO(4,3),HS(3,3)
      INTEGER   CONE(NE,4)
      COMPLEX   CP,CS,GE,C1,C2,C3
      DIMENSION DELTA(3,3)
      COMMON    N,NP,L,CP,CS,GE,RO,XNU,FR,INP,IPR
      COMMON    C1,C2,C3,C4,C5,C6,DELTA
      DATA      PI/3.141592654/
C
C  COMPUTE GENERAL CONSTANTS
C
      C1=(CP/CS)**2
      C2=(CS/CP)**2
      C3=(1.,0.)/(CMPLX(4.*PI,0.)*GE)
      C4=1./(4.*PI)
      C5=-1./(8.*PI*(1.-XNU))
      C6=1.-2.*XNU
C
      DO 25 I=1,3
      DO 25 J=1,3
      IF(I-J) 10,20,10
   10 DELTA(I,J)=0.
      GOTO 25
   20 DELTA(I,J)=1.
   25 CONTINUE
C
      DO 50 J=1,N
      DO 50 I=1,N
      II=3*(I-1)
      DO 50 JN=1,3
      DO 50 IN=1,3
      G(II+IN,II+JN)=(0.,0.)
   50 H(II+IN,II+JN)=(0.,0.)
C
C  COMPUTE THE COEFICIENTS OF G AND H MATRICES
C
      DO 220 J=1,N
C
C  COMPUTE THE COMPONENTS OF THE NORMAL VECTOR TO A FLAT ELEMENT
C  BY USING THE VECTOR PRODUCT OF SIDES 1-2 AND 1-3
C
      N1=CONE(J,1)
      N2=CONE(J,2)
      N3=CONE(J,3)
      N4=CONE(J,4)
      A=(Y(N2)-Y(N1))*(Z(N3)-Z(N1))-(Z(N2)-Z(N1))*(Y(N3)-Y(N1))
      B=(Z(N2)-Z(N1))*(X(N3)-X(N1))-(X(N2)-X(N1))*(Z(N3)-Z(N1))
      C=(X(N2)-X(N1))*(Y(N3)-Y(N1))-(Y(N2)-Y(N1))*(X(N3)-X(N1))
      R=SQRT(A*A+B*B+C*C)
      ETA(1)=A/R
      ETA(2)=B/R
      ETA(3)=C/R
C
C  STORE COORDINATES OF THE CORNER POINTS IN ARRAY CO
C
      CO(1,1)=X(N1)
      CO(1,2)=Y(N1)
      CO(1,3)=Z(N1)
      CO(2,1)=X(N2)
      CO(2,2)=Y(N2)
      CO(2,3)=Z(N2)
      CO(3,1)=X(N3)
      CO(3,2)=Y(N3)
      CO(3,3)=Z(N3)
      CO(4,1)=X(N4)
      CO(4,2)=Y(N4)
      CO(4,3)=Z(N4)
      JJ=3*(J-1)
C
      DO 200 I=1,N
C
      IF(I-J) 120,140,120
  120 IF(N3-N4) 125,130,125
C
C  QUADRILATERAL ELEMENTS
C
  125 CALL EXTIN7A(GW,HW,HDIF,HS,CO,XM(I),YM(I),ZM(I),ETA,0)
      GO TO 150
C
C  TRIANGULAR ELEMENTS
C
  130 CALL EXTIN7B(GW,HW,HS,CO,XM(I),YM(I),ZM(I),ETA)
      GO TO 150
  140 CALL LOCIN7(GW,HW,HDIF,XM(I),YM(I),ZM(I),ETA,X,Y,Z,N1,N2,N3,N4)
  150 CONTINUE
C
      II=3*(I-1)
      GW(2,1)=GW(1,2)
      GW(3,1)=GW(1,3)
      GW(3,2)=GW(2,3)
      DO 190 JN=1,3
      DO 190 IN=1,3
      G(II+IN,JJ+JN)=GW(IN,JN)
      IF(I-J) 160,170,160
  160 H(II+IN,JJ+JN)=HW(IN,JN)
      H(II+IN,II+JN)=H(II+IN,II+JN)-CMPLX(HS(IN,JN),0.)
      GOTO 190
  170 H(II+IN,II+JN)=H(II+IN,II+JN)+HDIF(IN,JN)
  190 CONTINUE
C
  200 CONTINUE
  220 CONTINUE
C
C  ADD ONE TO DIAGONAL TERMS FOR EXTERNAL PROBLEMS
C
      DO 225 I=1,3*N
      IF(REAL(H(I,I)).LT.0.) H(I,I)=H(I,I)+(1.,0.)
  225 CONTINUE
C
C  REORDER THE COLUMNS OF THE SYSTEM OF EQUATIONS IN ACCORDANCE WITH THE
C  BOUNDARY CONDITIONS AND FORM SYSTEM MATRIX A WHICH IS STORED IN G
C
      NN=3*N
      DO 250 J=1,NN
      IF(KODE(J)) 250,230,250
  230 DO 240 I=1,NN
      CH=G(I,J)*GE
      G(I,J)=-H(I,J)
  240 H(I,J)=-CH
  250 CONTINUE
C
C  FORM THE RIGHT HAND SIDE VECTOR F WHICH IS STORED IN FI
C
      DO 300 I=1,NN
      FI(I)=(0.,0.)
      DO 300 J=1,NN
      FI(I)=FI(I)+G(I,J)*DFI(J)
  300 CONTINUE
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE EXTIN7A(GW,HW,HDIF,HS,CO,XP,YP,ZP,ETA,ICODE)
C
      COMPLEX   GW(3,3),HW(3,3),U(3,3),T(3,3),HDIF(3,3),TDIF(3,3)
      DIMENSION ETA(3),CO(4,3),F(4),P(2,4),XJ(2,3)
      DIMENSION OME(6),GI(6),HS(3,3),TS(3,3)
      COMPLEX   CP,CS,GE,C1,C2,C3
      DIMENSION DELTA(3,3)
      COMMON    N,NP,L,CP,CS,GE,RO,XNU,FR,INP,IPR
      COMMON    C1,C2,C3,C4,C5,C6,DELTA
C
C  QUADRILATERAL ELEMENTS
C
C  THIS SUBROUTINE COMPUTES THE OFF-DIAGONAL COEFFICIENTS OF H AND G
C  MATRICES USING GAUSS QUADRATURE
C
      DATA OME/ .1713245, .3607616, .4679139,.4679139,.3607616,.1713245/
      DATA GI /-.9324695,-.6612094,-.2386192,.2386192,.6612094,.9324695/
C
C  ICODE .LE. 0   THE COLLOCATION POINT DOES NOT BELONG TO THE ELEMENT
C  ICODE .GT. 0   THE COLLOCATION POINT BELONGS TO THE ELEMENT
C
      DO 20 JN=1,3
      DO 20 IN=1,3
      GW(IN,JN)=(0.,0.)
      IF(ICODE) 10,10,15
   10 HW(IN,JN)=(0.,0.)
      HS(IN,JN)=0.
      GOTO 20
   15 HDIF(IN,JN)=(0.,0.)
   20 CONTINUE
C
      DO 400 JG=1,6
C
      G2=GI(JG)
      P2=OME(JG)
      SP=1.0+G2
      SM=1.0-G2
      P(1,1)=-0.25*SM
      P(1,2)= 0.25*SM
      P(1,3)= 0.25*SP
      P(1,4)=-0.25*SP
C
      DO 300 IG=1,6
C
      G1=GI(IG)
      P1=OME(IG)
      RP=1.0+G1
      RM=1.0-G1
      F(1)=0.25*RM*SM
      F(2)=0.25*RP*SM
      F(3)=0.25*RP*SP
      F(4)=0.25*RM*SP
      P(2,1)=-0.25*RM
      P(2,2)=-0.25*RP
      P(2,3)= 0.25*RP
      P(2,4)= 0.25*RM
C
C  COMPUTE THE RELATION BETWEEN CARTESIAN AND HOMOGENOUS COORDINATES
C
      DO 90 I=1,2
      DO 80 J=1,3
      TEMP=0.0
      DO 70 K=1,4
      TEMP=TEMP+P(I,K)*CO(K,J)
   70 CONTINUE
      XJ(I,J)=TEMP
   80 CONTINUE
   90 CONTINUE
C
C  COMPUTE THE JACOBIAN
C
      DET=SQRT( (XJ(1,2)*XJ(2,3)-XJ(2,2)*XJ(1,3))**2 +
     1          (XJ(2,1)*XJ(1,3)-XJ(1,1)*XJ(2,3))**2 +
     2          (XJ(1,1)*XJ(2,2)-XJ(2,1)*XJ(1,2))**2  )
      IF(DET-1.0E-5) 100,110,110
  100 WRITE(IPR,1000) DET
      WRITE(IPR,1100) ((CO(I,J),J=1,3),I=1,4)
 1000 FORMAT(///' EXTIN7A : ERROR, NULL OR NEGATIVE JACOBIAN =',G14.5)
 1100 FORMAT(/1X,'COORDINATES OF CORNER POINTS '/(3G14.5))
      STOP ' EXTIN7A : ERROR, NULL OR NEGATIVE JACOBIAN'
  110 CONTINUE
C
C  COMPUTE THE COORDINATES OF THE INTEGRATION POINTS
C
      XG=0.0
      YG=0.0
      ZG=0.0
      DO 130 I=1,4
      XG=XG+CO(I,1)*F(I)
      YG=YG+CO(I,2)*F(I)
      ZG=ZG+CO(I,3)*F(I)
  130 CONTINUE
C
      CALL FUNTHEH(U,T,TS,XP,YP,ZP,ETA,XG,YG,ZG,ICODE,TDIF)
C
      P12=P1*P2*DET
      DO 200 JN=1,3
      DO 200 IN=1,3
      GW(IN,JN)=GW(IN,JN)+U(IN,JN)*CMPLX(P12,0.)
      IF(ICODE) 150,150,160
  150 HW(IN,JN)=HW(IN,JN)+T(IN,JN)*CMPLX(P12,0.)
      HS(IN,JN)=HS(IN,JN)+TS(IN,JN)*P12
      GOTO 200
  160 HDIF(IN,JN)=HDIF(IN,JN)+TDIF(IN,JN)*CMPLX(P12,0.)
  200 CONTINUE
C
  300 CONTINUE
  400 CONTINUE
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE EXTIN7B(GW,HW,HS,CO,XP,YP,ZP,ETA)
C
C  TRIANGULAR ELEMENTS
C
C  THIS SUBROUTINE COMPUTES THE OFF DIAGONAL COEFFICIENTS OF H AND G
C  MATRICES USING GAUSS QUADRATURE
C
      COMPLEX   GW(3,3),HW(3,3),U(3,3),T(3,3),TDIF(3,3)
      DIMENSION ETA(3),CO(4,3),F(3),XJ(2,3)
      DIMENSION OMEI(4),OMEJ(4),GI(4),GJ(4),HS(3,3),TS(3,3)
      COMPLEX   CP,CS,GE,C1,C2,C3
      DIMENSION DELTA(3,3)
      COMMON    N,NP,L,CP,CS,GE,RO,XNU,FR,INP,IPR
      COMMON    C1,C2,C3,C4,C5,C6,DELTA
C
      DATA OMEI/.1739274226,.3260725774,.3260725774,.1739274226/
      DATA GI  /.0571041961,.2768430136,.5835904324,.8602401357/
      DATA OMEJ/.1355069134,.2034645680,.1298475476,.0311809709/
      DATA GJ  /.0694318422,.3300094782,.6699905218,.9305681558/
C
      DO 10 I=1,3
      XJ(1,I)=CO(1,I)-CO(3,I)
      XJ(2,I)=CO(2,I)-CO(3,I)
   10 CONTINUE
C
C  COMPUTE THE JACOBIAN
C
      DET=SQRT( (XJ(1,2)*XJ(2,3)-XJ(2,2)*XJ(1,3))**2 +
     &          (XJ(2,1)*XJ(1,3)-XJ(1,1)*XJ(2,3))**2 +
     &          (XJ(1,1)*XJ(2,2)-XJ(2,1)*XJ(1,2))**2  )
      IF(DET-1.0E-5) 20,30,30
   20 WRITE(IPR,1000) DET
      WRITE(IPR,1100) ((CO(I,J),J=1,3),I=1,3)
 1000 FORMAT(///' EXTIN7B : ERROR, NULL OR NEGATIVE JACOBIAN =',G14.5)
 1100 FORMAT(/1X,'COORDINATES OF EXTREME POINTS '/(3G14.5))
      STOP ' EXTIN7B : ERROR, NULL OR NEGATIVE JACOBIAN'
   30 CONTINUE
C
      DO 35 JN=1,3
      DO 35 IN=1,3
      GW(IN,JN)=(0.,0.)
      HW(IN,JN)=(0.,0.)
   35 HS(IN,JN)=0.
C
      DO 200 IG=1,4
      G1=GI(IG)
      GC1=1.-G1
      P1=OMEI(IG)
C
      DO 100 JG=1,4
      G2=GJ(JG)*GC1
      P2=OMEJ(JG)
      F(1)=G1
      F(2)=G2
      F(3)=1.-G1-G2
C
C  COMPUTE COORDINATES OF THE INTEGRATION POINTS
C
      XG=0.0
      YG=0.0
      ZG=0.0
      DO 40 I=1,3
      XG=XG+CO(I,1)*F(I)
      YG=YG+CO(I,2)*F(I)
      ZG=ZG+CO(I,3)*F(I)
   40 CONTINUE
C
      CALL FUNTHEH(U,T,TS,XP,YP,ZP,ETA,XG,YG,ZG,0,TDIF)
C
      P12=P1*P2*DET
      DO 90 JN=1,3
      DO 90 IN=1,3
      GW(IN,JN)=GW(IN,JN)+U(IN,JN)*CMPLX(P12,0.)
      HW(IN,JN)=HW(IN,JN)+T(IN,JN)*CMPLX(P12,0.)
   90 HS(IN,JN)=HS(IN,JN)+TS(IN,JN)*P12
C
  100 CONTINUE
  200 CONTINUE
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE FUNTHEH(U,T,TS,XP,YP,ZP,ETA,XG,YG,ZG,ICODE,TDIF)
C
C  THIS SUBROUTINE COMPUTES THE FUNDAMENTAL SOLUTION TENSORS
C  AT EACH GAUSS POINT
C  U()    : DISPLACEMENT
C  T()    : TRACTION
C  TS()   : STATIC TRACTION
C  TDIF() : T()-TS()
C
C  ICODE .LE. 0   THE COLLOCATION POINT DOES NOT BELONG TO THE ELEMENT
C  ICODE .GT. 0   THE COLLOCATION POINT BELONGS TO THE ELEMENT
C
      COMPLEX   U(3,3),T(3,3),TDIF(3,3)
      COMPLEX   C7,ZP2,ZS2,AA,BB,CC,TA,TB,TC
      COMPLEX   WI,ZKP,ZKS,ZZP,ZZS,EZP,EZS,FHI,CAPPA,FHIDR,CAPPADR
      DIMENSION RD(3),ETA(1),TS(3,3)
      COMPLEX   CP,CS,GE,C1,C2,C3
      DIMENSION DELTA(3,3)
      COMMON    N,NP,L,CP,CS,GE,RO,XNU,FR,INP,IPR
      COMMON    C1,C2,C3,C4,C5,C6,DELTA
C
      DATA PI/3.141592654/
      DATA TOL/1.E-15/
C
      C7=CS/CP
C
      R1=XG-XP
      R2=YG-YP
      R3=ZG-ZP
      R=SQRT(R1**2+R2**2+R3**2)
      RDN=(R1*ETA(1)+R2*ETA(2)+R3*ETA(3))/R
      RD(1)=R1/R
      RD(2)=R2/R
      RD(3)=R3/R
C
      WI=CMPLX(0.,FR)
      ZKP=-WI/CP
      ZKS=-WI/CS
      ZZP=ZKP*R
      ZZS=ZKS*R
      EZP=CEXP(ZZP)
      EZS=CEXP(ZZS)
      ZP2=ZZP*ZZP
      ZS2=ZZS*ZZS
C
      FHI=(1.+1./ZS2-1./ZZS)*EZS/R-
     @    C2*(1./ZP2-1./ZZP)*EZP/R
      CAPPA=   (1.+3./ZS2-3./ZZS)*EZS/R-
     @      C2*(1.+3./ZP2-3./ZZP)*EZP/R
      FHIDR=(-2.+ZZS+3./ZZS-3./ZS2)*EZS/R**2-
     @       C2*(-1.+3./ZZP-3./ZP2)*EZP/R**2
      CAPPADR=   (ZZS-4.+9./ZZS-9./ZS2)*EZS/R**2-
     @        C2*(ZZP-4.+9./ZZP-9./ZP2)*EZP/R**2
C
C   COMPUTE TENSORS U(),T() AND TS()
C
      AA=FHIDR-CAPPA/R
      BB=4.*CAPPA/R-2.*CAPPADR
      CC=(C1-2.)*(AA+0.5*BB-3.*CAPPA/R)-2.*CAPPA/R
C
      DO 10 II=1,3
      DO 10 JJ=1,3
      U(JJ,II)=DELTA(JJ,II)*FHI-CAPPA*RD(II)*RD(JJ)
      U(JJ,II)=U(JJ,II)*C3
      T(JJ,II)=AA*(RDN*DELTA(JJ,II)+RD(II)*ETA(JJ))+
     1RD(II)*RD(JJ)*RDN*BB+RD(JJ)*ETA(II)*CC
      T(JJ,II)=T(JJ,II)*C4
      TS(JJ,II)=RDN*(C6*DELTA(JJ,II)+3.*RD(II)*RD(JJ))+
     1C6*(ETA(JJ)*RD(II)-ETA(II)*RD(JJ))
      TS(JJ,II)=C5*TS(JJ,II)/R**2
   10 CONTINUE
C
C  WHEN ICODE .GT. 0 , COMPUTE TDIF()
C
C  TDIF() IS COMPUTED FROM THE SERIES EXPANSION OF THE DIFFERENCE
C  BETWEEN THE DYNAMIC AND STATIC TRACTION TENSORS.
C
      IF (ICODE) 700,700,30
   30 IF (CABS(FR*R/CP)-0.4) 50,50,40
C
C  THE VALUES OF THE TRACTION TENSOR ARE FAR FROM BEING SINGULAR
C
   40 DO 45 II=1,3
      DO 45 JJ=1,3
   45 TDIF(JJ,II)=T(JJ,II)-TS(JJ,II)
      RETURN
C
C  THE MAXIMUM VALUE OF THE FREQUENCY IS LIMITED FOR AN
C  ADEQUATE CONVERGENCE OF THE SERIES
C
   50 IF (CABS(FR*R/CS)-2.*PI) 55,55,56
   56 STOP 'FUNTHEH : FREQUENCY TOO HIGH'
   55 AA=(0.,0.)
      BB=(0.,0.)
      CC=(0.,0.)
      FACTORIAL=1.
      DO 70 NA=0,100
      IF (NA.NE.0) FACTORIAL=FACTORIAL*NA
      TA=(NA+2.*(1.+C7**(NA+4)))*ZZS**NA/
     1((NA+2.)*(NA+4.)*FACTORIAL)
      AA=AA+TA
      TTA=CABS(TA)
      ABA=CABS(AA)
      IF((TTA/ABA).LE.TOL) GO TO 100
   70 CONTINUE
C
  100 FACTORIAL=1.
      DO 80 NB=0,100
      IF (NB.NE.0) FACTORIAL=FACTORIAL*NB
      TB=2.*(NB-1.)*(1.-C7**(NB+4))*ZZS**NB/
     1((NB+2.)*(NB+4.)*FACTORIAL)
      BB=BB+TB
      TTB=CABS(TB)
      ABB=CABS(BB)
      IF(((TTB/ABB).LE.TOL).AND.(NB.NE.1)) GO TO 101
   80 CONTINUE
C
  101 FACTORIAL=1.
      DO 90 NC=0,100
      IF (NC.NE.0) FACTORIAL=FACTORIAL*NC
      TC=(-2.+(NC+4.)*C7**(NC+2)-2.*(NC+3.)*C7**(NC+4))*ZZS**NC/
     1((NC+2.)*(NC+4.)*FACTORIAL)
      CC=CC+TC
      TTC=CABS(TC)
      ABC=CABS(CC)
      IF((TTC/ABC).LE.TOL) GO TO 102
   90 CONTINUE
C
  102 AA=ZKS*ZKS*AA
      BB=-ZKS*ZKS*BB
      CC=ZKS*ZKS*CC
C
      DO 110 II=1,3
      DO 110 JJ=1,3
      TDIF(JJ,II)=AA*(RDN*DELTA(JJ,II)+RD(II)*ETA(JJ))+
     1RD(II)*RD(JJ)*RDN*BB+RD(JJ)*ETA(II)*CC
  110 TDIF(JJ,II)=TDIF(JJ,II)*C4
C
  700 RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE LOCIN7(GW,HW,HDIF,XM,YM,ZM,ETA,X,Y,Z,N1,N2,N3,N4)
C
C  THIS SUBROUTINE COMPUTES THE VALUE OF THE DIAGONAL COEFFICIENTS OF
C  THE G MATRIX BY NUMERICAL INTEGRATION.
C
      COMPLEX   GW(3,3),HW(3,3),GP(3,3),HP(3,3),HDIF(3,3),HDIFP(3,3)
      DIMENSION CO(4,3),X(1),Y(1),Z(1),HS(3,3)
      COMPLEX   CP,CS,GE,C1,C2,C3
      DIMENSION DELTA(3,3)
      COMMON    N,NP,L,CP,CS,GE,RO,XNU,FR,INP,IPR
      COMMON    C1,C2,C3,C4,C5,C6,DELTA
C
C  NUMERICAL INTEGRATION OVER INTERNAL TRIANGLES COLAPSING THE EDGE 3-4
C  OF A QUADRILATERAL
C
      CO(3,1)=XM
      CO(3,2)=YM
      CO(3,3)=ZM
      CO(4,1)=XM
      CO(4,2)=YM
      CO(4,3)=ZM
C
      DO 10 JN=1,3
      DO 10 IN=1,3
      GW(IN,JN)=(0.,0.)
      HW(IN,JN)=(0.,0.)
   10 HDIF(IN,JN)=(0.,0.)
C
      DO 500 IT=1,4
C
      GO TO (20,30,40,50),IT
   20 CO(1,1)=X(N1)
      CO(1,2)=Y(N1)
      CO(1,3)=Z(N1)
      CO(2,1)=X(N2)
      CO(2,2)=Y(N2)
      CO(2,3)=Z(N2)
      GO TO 60
   30 CO(1,1)=X(N2)
      CO(1,2)=Y(N2)
      CO(1,3)=Z(N2)
      CO(2,1)=X(N3)
      CO(2,2)=Y(N3)
      CO(2,3)=Z(N3)
      GO TO 60
   40 CO(1,1)=X(N3)
      CO(1,2)=Y(N3)
      CO(1,3)=Z(N3)
      IF(N3-N4) 42,44,42
   42 CO(2,1)=X(N4)
      CO(2,2)=Y(N4)
      CO(2,3)=Z(N4)
      GO TO 60
   44 CO(2,1)=X(N1)
      CO(2,2)=Y(N1)
      CO(2,3)=Z(N1)
      GO TO 60
   50 IF(N3-N4) 52,500,52
   52 CO(1,1)=X(N4)
      CO(1,2)=Y(N4)
      CO(1,3)=Z(N4)
      CO(2,1)=X(N1)
      CO(2,2)=Y(N1)
      CO(2,3)=Z(N1)
   60 CONTINUE
C
C  INTEGRATE OVER THE COLAPSED QUADRILATERAL
C
      CALL EXTIN7A(GP,HP,HDIFP,HS,CO,XM,YM,ZM,ETA,1)
C
      DO 70 JN=1,3
      DO 70 IN=1,3
      GW(IN,JN)=GW(IN,JN)+GP(IN,JN)
      HW(IN,JN)=HW(IN,JN)+HP(IN,JN)
   70 HDIF(IN,JN)=HDIF(IN,JN)+HDIFP(IN,JN)
C
  500 CONTINUE
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE INTER7(FI,DFI,KODE,CX,CY,CZ,X,Y,Z,DSOL,NE,CONE)
C
C  THIS SUBROUTINE COMPUTES THE VALUES OF THE DISPLACEMENT COMPONENTS
C  AT INTERNAL POINTS
C
      COMPLEX   FI(1),DFI(1),DSOL(1),CH,GW(3,3),HW(3,3),HDIF(3,3)
      DIMENSION KODE(1),X(1),Y(1),Z(1),CX(1),CY(1),CZ(1)
      DIMENSION CO(4,3),ETA(3),HS(3,3)
      INTEGER   CONE(NE,4)
      COMPLEX   CP,CS,GE,C1,C2,C3
      DIMENSION DELTA(3,3)
      COMMON    N,NP,L,CP,CS,GE,RO,XNU,FR,INP,IPR
      COMMON    C1,C2,C3,C4,C5,C6,DELTA
C
C  REARRANGE FI AND DFI ARRAYS TO STORE ALL THE VALUES OF THE
C  DISPLACEMENTS IN FI AND ALL THE VALUES OF THE TRACTIONS IN DFI
C
      NN=3*N
      DO 20 I=1,NN
      IF(KODE(I)) 20,10,20
   10 CH=FI(I)*GE
      FI(I)=DFI(I)
      DFI(I)=CH
   20 CONTINUE
C
C  COMPUTE THE VALUES OF THE DISPLACEMENTS AT INTERNAL POINTS
C
      IF(L.LE.0) RETURN
C
      DO 30 K=1,L*3
      DSOL(K)=(0.,0.)
   30 CONTINUE
C
      DO 120 J=1,N
C
      N1=CONE(J,1)
      N2=CONE(J,2)
      N3=CONE(J,3)
      N4=CONE(J,4)
      A=(Y(N2)-Y(N1))*(Z(N3)-Z(N1))-(Z(N2)-Z(N1))*(Y(N3)-Y(N1))
      B=(Z(N2)-Z(N1))*(X(N3)-X(N1))-(X(N2)-X(N1))*(Z(N3)-Z(N1))
      C=(X(N2)-X(N1))*(Y(N3)-Y(N1))-(Y(N2)-Y(N1))*(X(N3)-X(N1))
      R=SQRT(A*A+B*B+C*C)
      ETA(1)=A/R
      ETA(2)=B/R
      ETA(3)=C/R
      CO(1,1)=X(N1)
      CO(1,2)=Y(N1)
      CO(1,3)=Z(N1)
      CO(2,1)=X(N2)
      CO(2,2)=Y(N2)
      CO(2,3)=Z(N2)
      CO(3,1)=X(N3)
      CO(3,2)=Y(N3)
      CO(3,3)=Z(N3)
      CO(4,1)=X(N4)
      CO(4,2)=Y(N4)
      CO(4,3)=Z(N4)
      JJ=3*(J-1)
C
      DO 110 K=1,L
C
C
      IF(N3-N4) 60,70,60
C
C  QUADRILATERAL ELEMENTS
C
   60 CALL EXTIN7A(GW,HW,HDIF,HS,CO,CX(K),CY(K),CZ(K),ETA,0)
      GO TO 80
C
C  TRIANGULAR ELEMENTS
C
   70 CALL EXTIN7B(GW,HW,HS,CO,CX(K),CY(K),CZ(K),ETA)
   80 CONTINUE
C
      KK=3*(K-1)
      DO 100 IN=1,3
      DO 90 JN=1,3
      DSOL(KK+IN)=DSOL(KK+IN)+DFI(JJ+JN)*GW(IN,JN)-FI(JJ+JN)*HW(IN,JN)
   90 CONTINUE
  100 CONTINUE
C
  110 CONTINUE
  120 CONTINUE
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE OUTPUT7(FI,DFI,DSOL)
C
C  THIS SUBROUTINE PRINTS THE VALUES OF THE DISPLACEMENTS AND
C  TRACTIONS AT BOUNDARY NODES. IT ALSO PRINTS THE VALUES OF
C  THE DISPLACEMENTS AT INTERNAL POINTS
C
      COMPLEX   FI(1),DFI(1),DSOL(1)
      COMPLEX   CP,CS,GE,C1,C2,C3
      DIMENSION DELTA(3,3)
      COMMON    N,NP,L,CP,CS,GE,RO,XNU,FR,INP,IPR
      COMMON    C1,C2,C3,C4,C5,C6,DELTA
C
      WRITE(IPR,50)
   50 FORMAT('1',79('*')/)
C
      WRITE(IPR,700) FR
  700 FORMAT(30X,'FREQUENCY = ',G14.7/)
C
      WRITE(IPR,100)
  100 FORMAT(' ',79('*')//1X,'RESULTS'//2X,'BOUNDARY NODES'//' ELEM',
     16X,'DISPLACEMENT X',11X,'DISPLACEMENT Y',11X,'DISPLACEMENT Z'/)
      DO 10 I=1,N
      WRITE(IPR,200) I,(FI(3*(I-1)+J),J=1,3)
   10 CONTINUE
C
      WRITE(IPR,150)
  150 FORMAT(/' ELEM',8X,'TRACTION X',15X,'TRACTION Y',15X
     1,'TRACTION Z'/)
      DO 15 I=1,N
      WRITE(IPR,200) I,(DFI(3*(I-1)+J),J=1,3)
   15 CONTINUE
  200 FORMAT(I5,3(2X,G11.4,',',G11.4))
C
      IF(L.LE.0) GO TO 60
      WRITE(IPR,300)
  300 FORMAT(//2X,'INTERNAL POINTS'//' POINT',5X,
     1 'DISPLACEMENT X',11X,'DISPLACEMENT Y',11X,'DISPLACEMENT Z')
      DO 20 I=1,L
      WRITE(IPR,200) I,(DSOL(3*(I-1)+J),J=1,3)
   20 CONTINUE
C
   60 WRITE(IPR,500)
  500 FORMAT(' ',79('*'))
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE CSOLVER(A,B,N,NX)
C
      COMPLEX A(NX,NX),B(NX),X,CDOT
C
C     A=L*U  FACTORIZATION
C
      DO 30 K=1,N
      K1=K-1
      DO 10 I=K,N
      A(I,K)=A(I,K)-CDOT(K1,A(I,1),NX,A(1,K),1)
   10 CONTINUE
      X=A(K,K)
      IF(ABS(REAL(X))+ABS(AIMAG(X)).EQ.0.) THEN
      WRITE(6,*) ' SOLVER : STOP  SINGULARITY IN ROW ',K
      STOP ' SOLVER : STOP SINGULARITY '
      END IF
      X=(1.,0.)/X
      DO 20 J=K+1,N
      A(K,J)=X*(A(K,J)-CDOT(K1,A(K,1),NX,A(1,J),1))
   20 CONTINUE
   30 CONTINUE
C
C  FORWARD AND BACK SUBSTITUTION    L*U*X=B
C
C  SOLVE  L*Y=B
C
      DO 40 I=1,N
      B(I)=-(CDOT((I-1),A(I,1),NX,B,1)+B(I))/A(I,I)
   40 CONTINUE
C
C  SOLVE  U*X=Y
C
      DO 50 I=N,1,-1
      B(I)=-(CDOT((N-I),A(I,I+1),NX,B(I+1),1)+B(I))
   50 CONTINUE
C
      RETURN
      END
C-----------------------------------------------------------------------
      COMPLEX FUNCTION CDOT(N,U,IUX,V,IVX)
C
C THIS FUNCTION MAKES THE COMPLEX DOT PRODUCT OF TWO VECTORS
C
      COMPLEX U(1),V(1)
C
      CDOT=(0.,0.)
      IF(N.LE.0) RETURN
      IU=1
      IV=1
      IF(IUX.LT.0) IU=1-(N-1)*IUX
      IF(IVX.LT.0) IV=1-(N-1)*IVX
      DO 10 I=1,N
      CDOT=CDOT+U(IU)*V(IV)
      IU=IU+IUX
      IV=IV+IVX
10    CONTINUE
      RETURN
C
      END
