C-----------------------------------------------------------------------
      PROGRAM CONDTHSH
C
C  THIS IS A B.E. PROGRAM THAT USES (CON)STANT ELEMENTS FOR THE
C  SOLUTION OF (D)YNAMIC (TH)REE-DIMENSIONAL (S)CALAR WAVE TIME
C  (H)ARMONIC PROBLEMS
C
C
      PARAMETER(NX=160,NPIX=10,NCOX=200,NFRX=10)
      CHARACTER*12 FILEIN,FILEOUT
C
      COMPLEX     G(NX,NX),H(NX,NX),BC(NX),FI(NX),DFI(NX),DSOL(3*NPIX)
      COMMON/MATG/G
      COMMON/MATH/H
      DIMENSION   X(NCOX),Y(NCOX),Z(NCOX),XM(NX),YM(NX),ZM(NX)
      INTEGER     CONE(NX,4)
      DIMENSION   KODE(NX),CX(NPIX),CY(NPIX),CZ(NPIX),AFR(NFRX)
      COMMON      N,NP,L,CW,FR,INP,IPR
C
C  NX   = MAXIMUM DIMENSION OF THE SYSTEM OF EQUATIONS
C  NPIX = MAXIMUM NUMBER OF INTERNAL POINTS
C  NCOX = MAXIMUM NUMBER OF CORNER POINTS
C  NFRX = MAXIMUM NUMBER OF FREQUENCIES
C
C  ASSIGN NUMBERS FOR INPUT AND OUTPUT FILES
C
      INP=5
      IPR=6
C
C  READ NAMES AND OPEN FILES FOR INPUT AND OUTPUT
C
      WRITE(*,'(A,$)') ' NAME OF THE INPUT  FILE (MAX. 12 CHART.) : '
      READ(*,'(A)') FILEIN
      OPEN(INP,FILE=FILEIN,STATUS='OLD')
      WRITE(*,'(A,$)') ' NAME OF THE OUTPUT FILE (MAX. 12 CHART.) : '
      READ(*,'(A)') FILEOUT
      OPEN(IPR,FILE=FILEOUT,STATUS='NEW')
C
C  READ DATA
C
      CALL INPUT5(CX,CY,CZ,X,Y,Z,KODE,BC,NFR,AFR,NX,CONE,XM,YM,ZM)
c
      DO 60 I=1,NFR
c
      FR=AFR(I)
      DO 40 J=1,N
      DFI(J)=BC(J)
  40  CONTINUE
C
C  FORM SYSTEM OF EQUATIONS
C
      CALL GHMAT5(X,Y,Z,XM,YM,ZM,NX,G,H,FI,DFI,KODE,CONE)
C
C  SOLVE SYSTEM OF EQUATIONS
C
      CALL CSOLVER(H,FI,N,NX)
C
C  COMPUTE DISPLACEMENT OF INTERNAL POINTS
C
      CALL INTER5(FI,DFI,KODE,CX,CY,CZ,X,Y,Z,DSOL,NX,CONE)
C
C  PRINT RESULTS AT BOUNDARY NODES AND INTERNAL POINTS
C
      CALL OUTPUT5(FI,DFI,DSOL)
   60 CONTINUE
C
      CLOSE (INP)
      CLOSE (IPR)
C
      STOP 'END CONDTHSH'
      END
C-----------------------------------------------------------------------
      SUBROUTINE INPUT5(CX,CY,CZ,X,Y,Z,KODE,BC,NFR,AFR,NX,CONE,XM,YM,ZM)
C
      CHARACTER*80 TITLE
      DIMENSION    CX(1),CY(1),CZ(1),X(1),Y(1),Z(1),KODE(1),AFR(1)
      DIMENSION    XM(1),YM(1),ZM(1)
      INTEGER      CONE(NX,4)
      COMPLEX      BC(1)
      COMMON       N,NP,L,CW,FR,INP,IPR
C
C  N=   NUMBER OF BOUNDARY ELEMENTS
C  NP=  NUMBER OF ELEMENT CORNER POINTS
C  L=   NUMBER OF INTERNAL POINTS WHERE VALUES OF THE POTENTIAL
C       ARE CALCULATED
C  CW=  WAVE VELOCITY
C  NFR= NUMBER OF FREQUENCIES TO BE ANALYSED
C  AFR= ARRAY FOR STORAGE OF FREQUENCIES
C  FR=  FREQUENCY
C
      WRITE(IPR,100)
  100 FORMAT(/' ',79('*'))
C
C  READ JOB TITLE
C
      READ(INP,'(A)') TITLE
      WRITE(IPR,'(/A)') TITLE
      WRITE(IPR,100)
C
C  READ NUMBER OF ELEMENTS, CORNER POINTS, INTERNAL POINTS, FREQUENCIES
C  AND WAVE VELOCITY
C
      READ(INP,*) N,NP,L,NFR,CW
      WRITE(IPR,300) N,NP,L,NFR,CW
  300 FORMAT(//' DATA'//2X,
     1'NUMBER OF BOUNDARY ELEMENTS        =',I3/2X,
     2'NUMBER OF ELEMENT CORNER POINTS    =',I3/2X,
     3'NUMBER OF INTERNAL POINTS          =',I3/2X,
     4'NUMBER OF FREQUENCIES              =',I3/2X,
     5'WAVE VELOCITY                      =',G14.7/)
C
C  READ FREQUENCIES
C
      READ(INP,*) (AFR(I),I=1,NFR)
      WRITE(IPR,400) (AFR(I),I=1,NFR)
  400 FORMAT(/2X,'FREQUENCIES'/5(2X,G14.7))
C
C  READ COORDINATES OF THE CORNER POINTS OF THE BOUNDARY
C  ELEMENTS IN ARRAYS X, Y AND Z
C
      WRITE(IPR,500)
  500 FORMAT(//2X,'COORDINATES OF THE CORNER POINTS OF ',
     1'THE BOUNDARY ELEMENTS'//4X,'POINT',10X,'X',18X,'Y',17X,' Z')
      READ(INP,*) (X(I),Y(I),Z(I),I=1,NP)
      WRITE(IPR,700) (I,X(I),Y(I),Z(I),I=1,NP)
  700 FORMAT(5X,I3,3G19.7)
C
C  READ THE CONNECTIVITY OF EACH ELEMENT
C
C  TRIANGULAR ELEMENTS HAVE THE SAME CONNECTIVITY FOR THE LAST TWO CORNERS
C
      WRITE(IPR,740)
  740 FORMAT(//2X,'CONECTIVITY OF THE BOUNDARY ELEMENTS'//
     12X,'ELEMENT',1X,'CORNER I',1X,'CORNER J',1X,'CORNER K',1X,
     2'CORNER L',4X,'MID - X',5X,'MID - Y',5X,'MID - Z')
      DO 18 I=1,N
      READ(INP,*) (CONE(I,J),J=1,4)
C
C  COMPUTE NODAL COORDINATES AND STORE IN ARRAYS XM, YM AND ZM
C
      IF(CONE(I,4).EQ.0) CONE(I,4)=CONE(I,3)
      N1=CONE(I,1)
      N2=CONE(I,2)
      N3=CONE(I,3)
      N4=CONE(I,4)
      IF(N3-N4) 12,14,12
C
C  QUADRILATERAL ELEMENTS
C
   12 XM(I)=(X(N1)+X(N2)+X(N3)+X(N4))*0.25
      YM(I)=(Y(N1)+Y(N2)+Y(N3)+Y(N4))*0.25
      ZM(I)=(Z(N1)+Z(N2)+Z(N3)+Z(N4))*0.25
      GO TO 16
C
C  TRIANGULAR ELEMENTS
C
   14 XM(I)=(X(N1)+X(N2)+X(N3))*0.3333333
      YM(I)=(Y(N1)+Y(N2)+Y(N3))*0.3333333
      ZM(I)=(Z(N1)+Z(N2)+Z(N3))*0.3333333
   16 CONTINUE
      WRITE(IPR,780) I,(CONE(I,J),J=1,4),XM(I),YM(I),ZM(I)
  780 FORMAT(I7,4I9,1X,3G12.4)
   18 CONTINUE
C
C  READ BOUNDARY CONDITIONS IN BC(I) VECTOR; IF KODE(I)=0 THE
C  BC(I) VALUE IS A KNOWN PRESSURE, IF KODE(I)=1 THE BC(I)
C  VALUE IS A KNOWN PRESSURE DERIVATIVE
C
      WRITE(IPR,800)
  800 FORMAT(//2X,'BOUNDARY CONDITIONS'//9X,'ELEM',8X,
     1'PRESCRIBED  VALUE',3X,'CODE')
      READ(INP,*) (KODE(I),BC(I),I=1,N)
      WRITE(IPR,950) (I,BC(I),KODE(I),I=1,N)
  950 FORMAT(8X,I4,6X,G10.3,',',G10.3,I5)
C
C  READ COORDINATES OF THE INTERNAL POINTS
C
      IF(L) 40,40,30
   30 WRITE(IPR,1000)
 1000 FORMAT(//2X,'COORDINATES OF THE INTERNAL POINTS ',
     1//4X,'POINT',10X,'X',18X,'Y',17X,' Z')
      READ(INP,*) (CX(I),CY(I),CZ(I),I=1,L)
      WRITE(IPR,700) (I,CX(I),CY(I),CZ(I),I=1,L)
   40 CONTINUE
C
      WRITE(IPR,100)
c
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE GHMAT5(X,Y,Z,XM,YM,ZM,NX,G,H,FI,DFI,KODE,CONE)
C
C  THIS SUBROUTINE COMPUTES THE G AND H MATRICES AND
C  FORM THE SYSTEM OF EQUATIONS  A X = F
C
      COMPLEX   G(NX,NX),H(NX,NX),FI(1),DFI(1),CH
      DIMENSION X(1),Y(1),Z(1),XM(1),YM(1),ZM(1),KODE(1),ETA(3),CO(4,3)
      INTEGER   CONE(NX,4)
      COMMON    N,NP,L,CW,FR,INP,IPR
C
C  COMPUTE THE COEFICIENTS OF G AND H MATRICES
C
      DO 220 J=1,N
C
C  COMPUTE THE COMPONENTS OF THE NORMAL VECTOR TO A FLAT ELEMENT
C  BY USING THE VECTOR PRODUCT OF SIDES 1-2 AND 1-3
C
      N1=CONE(J,1)
      N2=CONE(J,2)
      N3=CONE(J,3)
      N4=CONE(J,4)
      A=(Y(N2)-Y(N1))*(Z(N3)-Z(N1))-(Z(N2)-Z(N1))*(Y(N3)-Y(N1))
      B=(Z(N2)-Z(N1))*(X(N3)-X(N1))-(X(N2)-X(N1))*(Z(N3)-Z(N1))
      C=(X(N2)-X(N1))*(Y(N3)-Y(N1))-(Y(N2)-Y(N1))*(X(N3)-X(N1))
      R=SQRT(A*A+B*B+C*C)
      ETA(1)=A/R
      ETA(2)=B/R
      ETA(3)=C/R
C
C  STORE COORDINATES OF THE CORNER POINTS IN ARRAY CO
C
      CO(1,1)=X(N1)
      CO(1,2)=Y(N1)
      CO(1,3)=Z(N1)
      CO(2,1)=X(N2)
      CO(2,2)=Y(N2)
      CO(2,3)=Z(N2)
      CO(3,1)=X(N3)
      CO(3,2)=Y(N3)
      CO(3,3)=Z(N3)
      CO(4,1)=X(N4)
      CO(4,2)=Y(N4)
      CO(4,3)=Z(N4)
C
      DO 200 I=1,N
C
      IF(I-J) 120,140,120
  120 IF(N3-N4) 125,130,125
C
C  QUADRILATERAL ELEMENTS
C
  125 CALL EXTIN5A(G(I,J),H(I,J),CO,XM(I),YM(I),ZM(I),ETA,0)
      GO TO 150
C
C  TRIANGULAR ELEMENTS
C
  130 CALL EXTIN5B(G(I,J),H(I,J),CO,XM(I),YM(I),ZM(I),ETA)
      GO TO 150
  140 CALL LOCIN5(G(I,J),H(I,J),XM(I),YM(I),ZM(I),ETA,X,Y,Z,
     &            N1,N2,N3,N4)
  150 CONTINUE
C
  200 CONTINUE
  220 CONTINUE
C
C  REORDER THE COLUMNS OF THE SYSTEM OF EQUATIONS IN ACCORDANCE WITH THE
C  BOUNDARY CONDITIONS AND FORM SYSTEM MATRIX A WHICH IS STORED IN G
C
      DO 250 J=1,N
      IF(KODE(J)) 250,230,250
  230 DO 240 I=1,N
      CH=G(I,J)
      G(I,J)=-H(I,J)
  240 H(I,J)=-CH
  250 CONTINUE
C
C  FORM THE RIGHT HAND SIDE VECTOR F WHICH IS STORED IN FI
C
      DO 300 I=1,N
      FI(I)=(0.,0.)
      DO 300 J=1,N
  300 FI(I)=FI(I)+G(I,J)*DFI(J)
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE EXTIN5A(G,H,CO,XP,YP,ZP,ETA,ICOD)
C
      COMPLEX   G,H,P12,U,Q
      DIMENSION ETA(3),CO(4,3),F(4),P(2,4),XJ(2,3),OME(6),GI(6)
      COMMON    N,NP,L,CW,FR,INP,IPR
C
C  QUADRILATERAL ELEMENTS
C
C  THIS SUBROUTINE COMPUTES THE COEFFICIENTS OF H AND G
C  MATRICES USING GAUSS QUADRATURE
C
      DATA OME/ .1713245, .3607616, .4679139,.4679139,.3607616,.1713245/
      DATA GI /-.9324695,-.6612094,-.2386192,.2386192,.6612094,.9324695/
C
C  ICOD .NE. 0   COLAPSED EDGE, G MATRIX ONLY
C  ICOD .EQ. 0   NORMAL QUADRILATERAL, G AND H MATRICES
C
      G=(0.,0.)
      H=(0.,0.)
C
      DO 400 JG=1,6
C
      G2=GI(JG)
      P2=OME(JG)
      SP=1.0+G2
      SM=1.0-G2
      P(1,1)=-0.25*SM
      P(1,2)= 0.25*SM
      P(1,3)= 0.25*SP
      P(1,4)=-0.25*SP
C
      DO 300 IG=1,6
C
      G1=GI(IG)
      P1=OME(IG)
      RP=1.0+G1
      RM=1.0-G1
      F(1)=0.25*RM*SM
      F(2)=0.25*RP*SM
      F(3)=0.25*RP*SP
      F(4)=0.25*RM*SP
      P(2,1)=-0.25*RM
      P(2,2)=-0.25*RP
      P(2,3)= 0.25*RP
      P(2,4)= 0.25*RM
C
C  COMPUTE THE RELATION BETWEEN CARTESIAN AND HOMOGENEOUS COORDINATES
C
      DO 90 I=1,2
      DO 80 J=1,3
      TEMP=0.0
      DO 70 K=1,4
      TEMP=TEMP+P(I,K)*CO(K,J)
   70 CONTINUE
      XJ(I,J)=TEMP
   80 CONTINUE
   90 CONTINUE
C
C  COMPUTE THE JACOBIAN
C
      DET=SQRT( (XJ(1,2)*XJ(2,3)-XJ(2,2)*XJ(1,3))**2 +
     1          (XJ(2,1)*XJ(1,3)-XJ(1,1)*XJ(2,3))**2 +
     2          (XJ(1,1)*XJ(2,2)-XJ(2,1)*XJ(1,2))**2  )
      IF(DET-1.0E-5) 100,110,110
  100 WRITE(IPR,1000) DET
      WRITE(IPR,1100) ((CO(I,J),J=1,3),I=1,4)
 1000 FORMAT(///' EXTIN5A : ERROR, NULL OR NEGATIVE JACOBIAN =',G14.5)
 1100 FORMAT(/1X,'COORDINATES OF CORNER POINTS '/(3G14.5))
      STOP ' EXTIN5A : ERROR, NULL OR NEGATIVE JACOBIAN'
  110 CONTINUE
C
C  COMPUTE THE COORDINATES OF THE INTEGRATION POINT
C
      XG=0.0
      YG=0.0
      ZG=0.0
      DO 130 I=1,4
      XG=XG+CO(I,1)*F(I)
      YG=YG+CO(I,2)*F(I)
      ZG=ZG+CO(I,3)*F(I)
  130 CONTINUE
C
      CALL FUNDA5(U,Q,XP,YP,ZP,ETA,ICOD,XG,YG,ZG)
C
      P12=CMPLX(P1*P2*DET,0.)
      G=G+U*P12
      IF(ICOD) 300,150,300
  150 H=H+Q*P12
C
  300 CONTINUE
  400 CONTINUE
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE EXTIN5B(G,H,CO,XP,YP,ZP,ETA)
C
C  TRIANGULAR ELEMENTS
C
C  THIS SUBROUTINE COMPUTES THE OFF DIAGONAL COEFFICIENTS OF H AND G
C  MATRICES USING GAUSS QUADRATURE
C
      COMPLEX   G,H,P12,U,Q
      DIMENSION ETA(3),CO(4,3),F(3),XJ(2,3),OMEI(4),OMEJ(4),GI(4),GJ(4)
      COMMON    N,NP,L,CW,FR,INP,IPR
C
      DATA OMEI/.1739274226,.3260725774,.3260725774,.1739274226/
      DATA GI  /.0571041961,.2768430136,.5835904324,.8602401357/
      DATA OMEJ/.1355069134,.2034645680,.1298475476,.0311809709/
      DATA GJ  /.0694318422,.3300094782,.6699905218,.9305681558/
C
      DO 10 I=1,3
      XJ(1,I)=CO(1,I)-CO(3,I)
      XJ(2,I)=CO(2,I)-CO(3,I)
   10 CONTINUE
C
C  COMPUTE THE JACOBIAN
C
      DET=SQRT( (XJ(1,2)*XJ(2,3)-XJ(2,2)*XJ(1,3))**2 +
     &          (XJ(2,1)*XJ(1,3)-XJ(1,1)*XJ(2,3))**2 +
     &          (XJ(1,1)*XJ(2,2)-XJ(2,1)*XJ(1,2))**2  )
      IF(DET-1.0E-5) 20,30,30
   20 WRITE(IPR,1000) DET
      WRITE(IPR,1100) ((CO(I,J),J=1,3),I=1,3)
 1000 FORMAT(///' EXTIN5B : ERROR, NULL OR NEGATIVE JACOBIAN =',G14.5)
 1100 FORMAT(/1X,'COORDINATES OF EXTREME POINTS '/(3G14.5))
      STOP ' EXTIN5B : ERROR, NULL OR NEGATIVE JACOBIAN'
   30 CONTINUE
C
      G=(0.,0.)
      H=(0.,0.)
C
      DO 200 IG=1,4
      G1=GI(IG)
      GC1=1.-G1
      P1=OMEI(IG)
C
      DO 100 JG=1,4
      G2=GJ(JG)*GC1
      P2=OMEJ(JG)
      F(1)=G1
      F(2)=G2
      F(3)=1.-G1-G2
C
C  COMPUTE THE COORDINATES OF THE INTEGRATION POINT
C
      XG=0.0
      YG=0.0
      ZG=0.0
      DO 40 I=1,3
      XG=XG+CO(I,1)*F(I)
      YG=YG+CO(I,2)*F(I)
      ZG=ZG+CO(I,3)*F(I)
   40 CONTINUE
C
      CALL FUNDA5(U,Q,XP,YP,ZP,ETA,0,XG,YG,ZG)
C
      P12=CMPLX(P1*P2*DET,0.)
      G=G+U*P12
      H=H+Q*P12
C
  100 CONTINUE
  200 CONTINUE
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE FUNDA5(U,Q,XP,YP,ZP,ETA,ICOD,XG,YG,ZG)
C
C  THIS SUBROUTINE COMPUTES THE FUNDAMENTAL SOLUTION AT A GAUSS POINT
C
C  IF ICOD.NE.0  POTENTIAL
C  IF ICOD.EQ.0  POTENTIAL AND FLUX
C
      COMPLEX   U,Q,ZW
      DIMENSION ETA(3)
      COMMON    N,NP,L,CW,FR,INP,IPR
C
      RD1=XG-XP
      RD2=YG-YP
      RD3=ZG-ZP
      R=SQRT(RD1*RD1+RD2*RD2+RD3*RD3)
C
      ZW=CMPLX(0.,-FR*R/CW)
      U=CEXP(ZW)/CMPLX(12.56637061*R,0.)
C
      IF(ICOD) 150,140,150
  140 RDN=(RD1*ETA(1)+RD2*ETA(2)+RD3*ETA(3))/R
      Q=(ZW-(1.,0.))*U*CMPLX(RDN/R,0.)
  150 CONTINUE
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE  LOCIN5(G,H,XM,YM,ZM,ETA,X,Y,Z,N1,N2,N3,N4)
C
C  THIS SUBROUTINE COMPUTES THE VALUE OF THE DIAGONAL COEFFICIENTS OF
C  THE G MATRIX BY NUMERICAL INTEGRATION.
C
      COMPLEX   G,H,GP
      DIMENSION CO(4,3),X(1),Y(1),Z(1),ETA(3)
      COMMON    N,NP,L,CW,FR,INP,IPR
C
C  NUMERICAL INTEGRATION OVER INTERNAL TRIANGLES COLAPSING THE EDGE 3-4
C  OF A QUADRILATERAL
C
      CO(3,1)=XM
      CO(3,2)=YM
      CO(3,3)=ZM
      CO(4,1)=XM
      CO(4,2)=YM
      CO(4,3)=ZM
C
      G=(0.,0.)
C
      DO 500 IT=1,4
C
      GO TO (20,30,40,50),IT
   20 CO(1,1)=X(N1)
      CO(1,2)=Y(N1)
      CO(1,3)=Z(N1)
      CO(2,1)=X(N2)
      CO(2,2)=Y(N2)
      CO(2,3)=Z(N2)
      GO TO 60
   30 CO(1,1)=X(N2)
      CO(1,2)=Y(N2)
      CO(1,3)=Z(N2)
      CO(2,1)=X(N3)
      CO(2,2)=Y(N3)
      CO(2,3)=Z(N3)
      GO TO 60
   40 CO(1,1)=X(N3)
      CO(1,2)=Y(N3)
      CO(1,3)=Z(N3)
      IF(N3-N4) 42,44,42
   42 CO(2,1)=X(N4)
      CO(2,2)=Y(N4)
      CO(2,3)=Z(N4)
      GO TO 60
   44 CO(2,1)=X(N1)
      CO(2,2)=Y(N1)
      CO(2,3)=Z(N1)
      GO TO 60
   50 IF(N3-N4) 52,500,52
   52 CO(1,1)=X(N4)
      CO(1,2)=Y(N4)
      CO(1,3)=Z(N4)
      CO(2,1)=X(N1)
      CO(2,2)=Y(N1)
      CO(2,3)=Z(N1)
   60 CONTINUE
C
C  INTEGRATE OVER THE COLAPSED QUADRILATERAL
C
      CALL EXTIN5A(GP,H,CO,XM,YM,ZM,ETA,1)
C
      G=G+GP
C
  500 CONTINUE
C
      H=(0.5,0.)
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE INTER5(FI,DFI,KODE,CX,CY,CZ,X,Y,Z,DSOL,NX,CONE)
C
C  THIS SUBROUTINE COMPUTES THE VALUES OF THE POTENTIAL
C  AT INTERNAL POINTS
C
      COMPLEX   FI(1),DFI(1),DSOL(1),CH,GW,HW
      DIMENSION KODE(1),X(1),Y(1),Z(1),CX(1),CY(1),CZ(1),CO(4,3),ETA(3)
      INTEGER   CONE(NX,4)
      COMMON    N,NP,L,CW,FR,INP,IPR
C
C  REARRANGE FI AND DFI ARRAYS TO STORE ALL THE VALUES OF THE
C  POTENTIAL IN FI AND ALL THE VALUES OF THE FLUX IN DFI
C
      DO 20 I=1,N
      IF(KODE(I)) 20,10,20
   10 CH=FI(I)
      FI(I)=DFI(I)
      DFI(I)=CH
   20 CONTINUE
C
C  COMPUTE THE VALUES OF THE POTENTIAL AT INTERNAL POINTS
C
      IF(L.LE.0) RETURN
C
      DO 30 K=1,L
      DSOL(K)=(0.,0.)
   30 CONTINUE
C
      DO 120 J=1,N
C
      N1=CONE(J,1)
      N2=CONE(J,2)
      N3=CONE(J,3)
      N4=CONE(J,4)
      A=(Y(N2)-Y(N1))*(Z(N3)-Z(N1))-(Z(N2)-Z(N1))*(Y(N3)-Y(N1))
      B=(Z(N2)-Z(N1))*(X(N3)-X(N1))-(X(N2)-X(N1))*(Z(N3)-Z(N1))
      C=(X(N2)-X(N1))*(Y(N3)-Y(N1))-(Y(N2)-Y(N1))*(X(N3)-X(N1))
      R=SQRT(A*A+B*B+C*C)
      ETA(1)=A/R
      ETA(2)=B/R
      ETA(3)=C/R
      CO(1,1)=X(N1)
      CO(1,2)=Y(N1)
      CO(1,3)=Z(N1)
      CO(2,1)=X(N2)
      CO(2,2)=Y(N2)
      CO(2,3)=Z(N2)
      CO(3,1)=X(N3)
      CO(3,2)=Y(N3)
      CO(3,3)=Z(N3)
      CO(4,1)=X(N4)
      CO(4,2)=Y(N4)
      CO(4,3)=Z(N4)
C
      DO 110 K=1,L
C
      IF(N3-N4) 60,70,60
C
C  QUADRILATERAL ELEMENTS
C
   60 CALL EXTIN5A(GW,HW,CO,CX(K),CY(K),CZ(K),ETA,0)
      GO TO 80
C
C  TRIANGULAR ELEMENTS
C
   70 CALL EXTIN5B(GW,HW,CO,CX(K),CY(K),CZ(K),ETA)
   80 CONTINUE
C
      DSOL(K)=DSOL(K)+DFI(J)*GW-FI(J)*HW
C
  110 CONTINUE
  120 CONTINUE
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE OUTPUT5(FI,DFI,DSOL)
C
C  THIS SUBROUTINE PRINTS THE VALUES OF THE POTENTIAL AND
C  THE FLUX AT BOUNDARY NODES. IT ALSO PRINTS THE VALUES OF
C  THE POTENTIAL AT INTERNAL POINTS
C
      COMPLEX   FI(1),DFI(1),DSOL(1)
      COMMON    N,NP,L,CW,FR,INP,IPR
C
      WRITE(IPR,50)
   50 FORMAT('1',79('*')/)
C
      WRITE(IPR,700) FR
  700 FORMAT(30X,'FREQUENCY = ',G14.7/)
C
      WRITE(IPR,100)
  100 FORMAT(' ',79('*')//1X,'RESULTS'//2X,'BOUNDARY ELEMENTS'//
     18X,' ELEM',12X,'   POTENTIAL  ',11X,'        FLUX       ')
      DO 10 I=1,N
      WRITE(IPR,200) I,FI(I),DFI(I)
   10 CONTINUE
  200 FORMAT(7X,I5,5X,2(4X,G11.4,',',G11.4))
C
      IF(L.LE.0) GO TO 60
      WRITE(IPR,300)
  300 FORMAT(//2X,'INTERNAL POINTS'//7X,' POINT',12X,'   POTENTIAL  ')
      DO 20 I=1,L
      WRITE(IPR,200) I,DSOL(I)
   20 CONTINUE
C
   60 WRITE(IPR,500)
  500 FORMAT(/' ',79('*'))
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE CSOLVER(A,B,N,NX)
C
      COMPLEX A(NX,NX),B(NX),X,CDOT
C
C     A=L*U  FACTORIZATION
C
      DO 30 K=1,N
      K1=K-1
      DO 10 I=K,N
      A(I,K)=A(I,K)-CDOT(K1,A(I,1),NX,A(1,K),1)
   10 CONTINUE
      X=A(K,K)
      IF(ABS(REAL(X))+ABS(AIMAG(X)).EQ.0.) THEN
      WRITE(6,*) ' SOLVER : STOP  SINGULARITY IN ROW ',K
      STOP ' SOLVER : STOP SINGULARITY '
      END IF
      X=(1.,0.)/X
      DO 20 J=K+1,N
      A(K,J)=X*(A(K,J)-CDOT(K1,A(K,1),NX,A(1,J),1))
   20 CONTINUE
   30 CONTINUE
C
C  FORWARD AND BACK SUBSTITUTION    L*U*X=B
C
C  SOLVE  L*Y=B
C
      DO 40 I=1,N
      B(I)=-(CDOT((I-1),A(I,1),NX,B,1)+B(I))/A(I,I)
   40 CONTINUE
C
C  SOLVE  U*X=Y
C
      DO 50 I=N,1,-1
      B(I)=-(CDOT((N-I),A(I,I+1),NX,B(I+1),1)+B(I))
   50 CONTINUE
C
      RETURN
      END
C-----------------------------------------------------------------------
      COMPLEX FUNCTION CDOT(N,U,IUX,V,IVX)
C
C THIS FUNCTION MAKES THE COMPLEX DOT PRODUCT OF TWO VECTORS
C
      COMPLEX U(1),V(1)
C
      CDOT=(0.,0.)
      IF(N.LE.0) RETURN
      IU=1
      IV=1
      IF(IUX.LT.0) IU=1-(N-1)*IUX
      IF(IVX.LT.0) IV=1-(N-1)*IVX
      DO 10 I=1,N
      CDOT=CDOT+U(IU)*V(IV)
      IU=IU+IUX
      IV=IV+IVX
10    CONTINUE
      RETURN
C
      END
