C ----------------------------------------------------------------------
      PROGRAM QUADAPEH
C
C  THIS IS A B. E. PROGRAM THAT USES (QUA)DRATIC ELEMENTS FOR THE
C  SOLUTION OF (D)YNAMIC (A)NTI(P)LANE (E)LASTIC OR VISCOELASTIC
C  TIME (H)ARMONIC PROBLEMS
C  
C  PROGRAM N0. 2
C
C
C  SET MAXIMUM DIMENSIONS
C  NX= MAXIMUM NUMBER OF BOUNDARY NODES = 2*MAXIMUM NUMBER OF ELEMENTS
C  NX1= 3*MAXIMUM NUMBER OF BOUNDARY ELEMENTS
C  NPM= MAXIMUM NUMBER OF INTERNAL POINTS
C  NFM= MAXIMUM NUMBER OF FREQUENCIES
C
      PARAMETER(NX=50,NX1=75,NPM=20,NFM=10)
      CHARACTER*12 FILEIN,FILEOUT
      COMPLEX G(NX,NX1),H(NX,NX),BC(NX1),FI(NX),DFI(NX1),SOL(NPM),CW,GE 
      COMMON/MATG/G
      COMMON/MATH/H
      COMMON N,L,CW,GE,FR,INP,IPR
      DIMENSION X(NX+1),Y(NX+1),KODE(NX1),CX(NPM),CY(NPM),AFR(NFM)
C
C  ASSIGN NUMBERS FOR INPUT AND OUTPUT FILES
C
      INP=5
      IPR=6
C
C  READ NAMES AND OPEN  FILES FOR INPUT AND OUTPUT
C
      WRITE(*,' (A) ') ' NAME OF INPUT FILE (MAX. 12 CHART.)'
      READ(*,'  (A) ')FILEIN
      OPEN(INP,FILE=FILEIN,STATUS='OLD')
      WRITE(*,' (A) ') ' NAME OF OUTPUT FILE (MAX. 12 CHART.)'
      READ(*,' (A) ')FILEOUT
      OPEN(IPR,FILE=FILEOUT,STATUS='NEW')
C
C  READ DATA
C
      CALL INPUT2(CX,CY,X,Y,KODE,BC,NFR,AFR)
      NE=N/2
      DO 60 I=1,NFR
      FR=AFR(I)
      DO 40 J=1,3*NE
   40 DFI(J)=BC(J)
C
C  FORM SYSTEM OF EQUATIONS
C
      CALL GHMAT2(X,Y,G,H,FI,DFI,KODE,NX,NX1)
C
C  SOLVE SYSTEM OF EQUATIONS
C
      CALL CSOLVER(H,FI,N,NX)
C
C  COMPUTE PRESSURE VALUES AT INTERNAL POINTS.
C
      CALL INTER2(FI,DFI,KODE,CX,CY,X,Y,SOL)
C
C  PRINT RESULTS AT BOUNDARY NODES AND INTERNAL POINTS
C
      CALL OUTPUT2(X,Y,FI,DFI,CX,CY,SOL)
   60 CONTINUE
C
C   CLOSE INPUT AND OUTPUT  FILES
C
      CLOSE (INP)
      CLOSE (IPR)
      STOP
      END

C-----------------------------------------------------------------------
      SUBROUTINE INPUT2(CX,CY,X,Y,KODE,BC,NFR,AFR)
C
C  NE  = NUMBER OF BOUNDARY ELEMENTS
C  N   = NUMBER OF BOUNDARY NODES = 2*NE
C  L   = NUMBER OF INTERNAL POINTS
C  CW  = SHEAR WAVE VELOCITY
C  GE  = SHEAR MODULUS
C  RO  = DENSITY
C  NFR = NUMBER OF FREQUENCIES TO BE ANALYSED
C  AFR = ARRAY FOR STORAGE OF FREQUENCIES
C
      CHARACTER*80 TITLE
      DIMENSION KODE(1),X(1),Y(1),CX(1),CY(1),AFR(1)
      COMPLEX BC(1),CW,GE
      COMMON N,L,CW,GE,FR,INP,IPR
      WRITE(IPR,100)
  100 FORMAT(' ',79('*'))
C
C  READ JOB TITLE
C
      READ(INP,'(A)') TITLE
      WRITE(IPR,'(A)') TITLE
C
C  READ NUMBER OF BOUNDARY ELEMENTS, INTERNAL POINTS AND FREQUENCIES
C  READ MATERIAL PROPERTIES
C
      READ(INP,*)NE,L,NFR,RGE,DAM,RO
      GE=CMPLX(RGE,RGE*2.*DAM)
      CW=CSQRT(GE/RO)
      WRITE(IPR,210)NE,L,NFR,CW,GE,DAM,RO
  210 FORMAT(//2X,'DATA'/2X,'NUMBER OF BOUNDARY ELEMENTS=',
     1I3/2X,'NUMBER OF INTERNAL POINTS=',I3/2X,'NUMBER OF FREQUENCIES ='
     2,I3/2X'SHEAR WAVE VELOCITY =',G14.7,',',G14.7/2X,
     3'SHEAR MODULUS=',G14.7,',',G14.7/2X,
     4'DAMPING RATIO=',G14.7/2X,'DENSITY=',G14.7)
      N=2*NE
C
C  READ FREQUENCIES
C
      READ(INP,*) (AFR(I),I=1,NFR)
      WRITE(IPR,400) (AFR(I),I=1,NFR)
  400 FORMAT(/2X,'FREQUENCIES'/5(2X,G14.7))
C
C  READ BOUNDARY NODES COORDINATES IN ARRAYS X AND Y
C
      WRITE(IPR,500)
      READ(INP,*) (X(I),Y(I),I=1,N)
      DO 10 I=1,N
   10 WRITE(IPR,240) I,X(I),Y(I)
  500 FORMAT(//2X,'BOUNDARY NODES COORDINATES'///4X,
     1'NODE',10X,'X',18X,'Y'/)
  240 FORMAT(5X,I3,2(5X,E14.7))
C
C  READ BOUNDARY CONDITIONS IN BC VECTOR, IF KODE(I) =0 THE
C  BC(I) VALUE IS A KNOWN DISPLACEMENT; IF KODE(I)=1 THE BC(I)
C  VALUE IS A KNOWN TRACTION.
C
C  THREE BOUNDARY CONDITIONS ARE READ PER ELEMENT.
C
C  NODES BETWEEN TWO ELEMENTS MAY HAVE TWO DIFFERENT VALUES
C  OF THE TRACTION BUT ONLY ONE VALUE OF THE DISPLACEMENT.
C
      WRITE(IPR,800)
  800 FORMAT(//2X,'BOUNDARY CONDITIONS'//11X,
     1'-----FIRST NODE------',3X,'-----SECOND NODE-----',3X,
     2'-----THIRD NODE------'/13X,'PRESCRIBED',14X,
     3'PRESCRIBED',14X,'PRESCRIBED'/1X,'ELEMENT',8X,'VALUE',
     47X,'CODE',8X,'VALUE',7X,'CODE',8X,'VALUE',7X,'CODE')
      DO 20 I=1,NE
      READ(INP,*) (KODE(3*I-3+J),BC(3*I-3+J),J=1,3)
   20 WRITE(IPR,950)I,(BC(3*I-3+J),KODE(3*I-3+J),J=1,3)
  950 FORMAT(3X,I3,1X,3(G10.3,',',G10.3,2X,I1))
C
C  READ COORDINATES OF THE INTERNAL POINTS
C
      IF(L.EQ.0) GO TO 30
      READ(INP,*) (CX(I),CY(I),I=1,L)
   30 RETURN
      END
C------------------------------------------------------------------------
      SUBROUTINE GHMAT2(X,Y,G,H,FI,DFI,KODE,NX,NX1)
C
C  THIS SUBROUTINE COMPUTES THE G AND H MATRICES AND FORMS
C  THE SYSTEM OF EQUATIONS   A X = F
C  H IS A SQUARE MATRIX (2*NE,2*NE); G IS RECTANGULAR (2*NE,3*NE)
C
      DIMENSION KODE(1),X(1),Y(1),HS(3)
      COMPLEX FI(1),DFI(1),G(NX,NX1),H(NX,NX),HW(3),GW(3),CH,CW,GE
      COMMON N,L,CW,GE,FR,INP,IPR
      NE=N/2
      DO 20 I=1,N
      DO 11 J=1,N
   11 H(I,J)=(0.,0.)
      DO 12 J=1,3*NE
   12 G(I,J)=(0.,0.)
   20 CONTINUE
      X(N+1)=X(1)
      Y(N+1)=Y(1)

C
C  COMPUTE THE GW, HW AND HS MATRICES FOR EACH COLLOCATION
C  POINT AND EACH BOUNDARY ELEMENT
C
      DO 60 LL=1,N
      DO 50 I=1,N-1,2
      IF((LL-I)*(LL-I-1)*(LL-I-2)*(LL-I+N-2)) 22,21,22
   21 NODO=LL-I+1
      IF((LL.EQ.1).AND.(I.EQ.N-1)) NODO=NODO+N
      CALL EXTIN2(X(LL),Y(LL),X(I),Y(I),X(I+1),Y(I+1),X(I+2),Y(I+2),
     *HW,GW,HS)
      CALL LOCIN2(X(I),Y(I),X(I+1),Y(I+1),X(I+2),Y(I+2),GW,NODO)
      GO TO 34
   22 CALL EXTIN2(X(LL),Y(LL),X(I),Y(I),X(I+1),Y(I+1),X(I+2),Y(I+2),
     *HW,GW,HS)
C
C  PLUG THE GW, HW AND HS MATRICES INTO THE GENERAL G AND H MATRICES.
C
   34 DO 48 J=1,3
C
C  PLUG THE GW MATRICES
C
      K=3*(I-1)/2
      G(LL,K+J)=G(LL,K+J)+GW(J)
C
C  PLUG THE HW AND HS MATRICES
C
      II=I
      IF(I.EQ.N-1.AND.J.EQ.3) II=-1
      IF(LL-II+1-J) 45,46,45
C
C  THE ELEMENT DOES NOT BELONG TO THE DIAGONAL
C
   45 H(LL,II-1+J)=H(LL,II-1+J)+HW(J)+HS(J)
      H(LL,LL)=H(LL,LL)-HS(J)
      GO TO 48
C
C  THE ELEMENT BELONGS TO THE DIAGONAL
C
   46 H(LL,LL)=H(LL,LL)+HW(J)
   48 CONTINUE
   50 CONTINUE
C
C  ADD ONE TO THE DIAGONAL COEFFICIENTS FOR EXTERNAL PROBLEMS
C
      IF(REAL(H(LL,LL)).LT.0) H(LL,LL)=H(LL,LL)+1

   60 CONTINUE
C
C  ARRANGE SYSTEM OF EQUATIONS IN ACCORDANCE WITH THE BOUNDARY
C  CONDITIONS AND FORM SYSTEM MATRIX A WHICH IS STORED IN H
C
      DO 180 I=1,NE
      DO 170 J=1,3
      IF(KODE(3*I-3+J)) 110,110,170
  110 IF((I-NE).NE.0 .OR. J.NE.3) GO TO 125
C
C  LAST NODE OF THE LAST ELEMENT
C
      IF(KODE(1)) 115,115,113
  113 DO 114 K=1,N
      CH=H(K,1)
      H(K,1)=-G(K,3*I)*GE
  114 G(K,3*I)=-CH
      GO TO 170
  115 DO 116 K=1,N
      H(K,1)=H(K,1)-G(K,3*I)*GE
  116 G(K,3*I)=0.
      GO TO 170
  125 IF(I.EQ.1 .OR. J.GT.1 .OR. KODE(3*I-3).EQ.1) GO TO 130
C
C  FIRST NODE OF AN ELEMENT OTHER THAN THE FIRST WHEN THE CHANGE
C  WAS ALREADY DONE FOR THE LAST NODE OF THE PREVIOUS ELEMENT
C
      DO 129 K=1,N
      H(K,2*I-1)=H(K,2*I-1)-G(K,3*I-2)*GE
  129 G(K,3*I-2)=(0.,0.)
      GO TO 170
C
C  FIRST ELEMENT, OR SECOND OR THIRD NODE OF ANOTHER ELEMENT,
C  OR FIRST NODE OF AN ELEMENT OTHER THAN THE FIRST WHEN THE
C  CHANGE WAS NOT DONE FOR THE LAST NODE OF THE PREVIOUS
C  ELEMENT
C
  130 DO 132 K=1,N
      CH=H(K,2*I-2+J)
      H(K,2*I-2+J)=-G(K,3*I-3+J)*GE
  132 G(K,3*I-3+J)=-CH
  170 CONTINUE
  180 CONTINUE
C
C  FORM THE RIGHT HAND SIDE VECTOR F WHICH IS STORED IN FI
C
      DO 200 II=1,N
      FI(II)=(0.,0.)
      DO 200 J=1,3*NE
      FI(II)=FI(II)+G(II,J)*DFI(J)
  200 CONTINUE
      RETURN
      END

C-----------------------------------------------------------------------
      SUBROUTINE EXTIN2(XP,YP,X1,Y1,X2,Y2,X3,Y3,HW,GW,HS)
C
C  THIS SUBRUOTINE COMPUTES THE HW, HS AND GW
C  MATRICES THAT RELATE A NODE (XP,YP) WITH A BOUNDARY
C  ELEMENT USING GAUSS QUADRATURE
C
C  RA           = RADIUS
C  RD1,RD2,RDN  = RADIUS DERIVATIVES
C  ETA1,ETA2    = COMPONENTS OF THE UNIT NORMAL TO THE ELEMENT
C  XCO,YCO      = INTEGRATION POINT ALONG THE ELEMENT
C  XJA          = JACOBIAN
C
      COMMON N,L,CW,GE,FR,INP,IPR
      COMPLEX GW(3),HW(3),Z,F0C,F1C,F2C,F0CM,F1CM,F2CM,CW,GE
      DIMENSION GI(10),OME(10),HS(3)
      DATA GI/0.9739065285,-0.9739065285,0.8650633666,-0.8650633666
     @       ,0.6794095683,-0.6794095682,0.4333953941,-0.4333953941,
     @        0.1488743389,-0.1488743389/
      DATA OME/0.0666713443,0.0666713443,0.1494513491,0.1494513491
     @        ,0.2190863625,0.2190863625,0.2692667193,0.2692667193,
     @         0.2955242247,0.2955242247/

      DO 20 J=1,3
      HS(J)=0.
      HW(J)=(0.,0.)
   20 GW(J)=(0.,0.)

      A=X3-2*X2+X1
      B=(X3-X1)/2
      C=Y3-2*Y2+Y1
      D=(Y3-Y1)/2

      DO 40 I=1,10
C
C  COMPUTE THE VALUES OF THE SHAPE FUNCTIONS AT THE INTEGRATION POINT
C
      F1=GI(I)*(GI(I)-1)*0.5
      F2=1.-GI(I)**2
      F3=GI(I)*(GI(I)+1)*0.5
C
C  COMPUTE GEOMETRICAL PARAMETERS AT THE INTEGRATION POINT
C
      XCO=X1*F1+X2*F2+X3*F3
      YCO=Y1*F1+Y2*F2+Y3*F3
      XJA=SQRT((GI(I)*A+B)**2+(GI(I)*C+D)**2)
      ETA1=(GI(I)*C+D)/XJA
      ETA2=-(GI(I)*A+B)/XJA
      RA=SQRT((XP-XCO)**2+(YP-YCO)**2)
      RD1=(XCO-XP)/RA
      RD2=(YCO-YP)/RA
      RDN=RD1*ETA1+RD2*ETA2
C
C  COMPUTE GW, HW, AND HS MATRICES
C
      ZR=FR*RA/CW
      Z=CMPLX(0.,ZR)
      CALL BESSEL(Z,F0C,F1C,F2C,F0CM,F1CM,F2CM)
      GW(1)=GW(1)+F0C*OME(I)*XJA*F1/(GE*6.2831852)
      GW(2)=GW(2)+F0C*OME(I)*XJA*F2/(GE*6.2831852)
      GW(3)=GW(3)+F0C*OME(I)*XJA*F3/(GE*6.2831852)
      HW(1)=HW(1)-Z/RA*RDN*(F1CM+Z*(CLOG(Z/2.)+0.07721566)/2.)
     &*OME(I)*XJA*F1/6.2831852
      HW(2)=HW(2)-Z/RA*RDN*(F1CM+Z*(CLOG(Z/2.)+0.07721566)/2.)
     &*OME(I)*XJA*F2/6.2631852
      HW(3)=HW(3)-Z/RA*RDN*(F1CM+Z*(CLOG(Z/2.)+0.07721566)/2.)
     &*OME(I)*XJA*F3/6.2831852
      HS(1)=HS(1)-RDN/RA*OME(I)*XJA*F1/6.2831852
      HS(2)=HS(2)-RDN/RA*OME(I)*XJA*F2/6.2631852
40    HS(3)=HS(3)-RDN/RA*OME(I)*XJA*F3/6.2831852
      RETURN
      END

C-----------------------------------------------------------------------
      SUBROUTINE LOCIN2(XG1,YG1,XG2,YG2,XG3,YG3,GW,NODO)
C
C  THIS SUBROUTINE COMPUTES THE GW MATRIX WHEN THE COLLOCATION
C  POINT ONE OF THE NODES OF THE INTEGRATION ELEMENT.
C  THE COEFFICIENTS ARE COMPUTED BY NUMERICAL INTEGRATION:
C  THE NON-SINGULAR PART IS COMPUTED USING STANDARD GAUSS QUADRATURE,
C  THE LOGARITHMIC PART IS COMPUTED USING A SPECIAL QUADRATURE FORMULA.
C
      COMMON N,L,CW,GE,FR,INP,IPR
      DIMENSION GI(10),OME(10),GIL(10),OMEL(10)
      COMPLEX GW(3),Z,F0C,F1C,F2C,F0CM,F1CM,F2CM,TERNS,S1,S2,S3,CW,GE
C
C  DATA FOR THE GAUSS QUADRATURE
C
      DATA GI/0.9739065285,-0.9739065285,0.8650633666,-0.8650633666
     @       ,0.6794095682,-0.6794095682,0.4333953941,-0.4333953941,
     @        0.1488743389,-0.1488743389/
      DATA OME/0.0666713443,0.0666713443,0.1494513491,0.1494513491
     @        ,0.2190863625,0.2190863625,0.2692667193,0.2692667193,
     @         0.2955242247,0.2955242247/
C
C  DATA FOR THE SPECIAL QUADRATURE
C
      DATA GIL/0.0090426309,0.0539712662,0.1353118246,0.2470524162
     @        ,0.3802125396,0.5237923179,0.6657752055,0.7941904160,
     @         0.8981610912,0.9688479887/
      DATA OMEL/0.1209551319,0.1863635425,0.1956608732,0.1735771421
     @         ,0.1356956729,0.0936467585,0.0557877273,0.0271598109,
     @          0.0095151826,0.0016381576/
C
C  SET LOCAL COORDINATES SYSTEM
C
      GOTO(1,2,3),NODO
    1 X3=XG3-XG1
      Y3=YG3-YG1
      X2=XG2-XG1
      Y2=YG2-YG1
      A1=(X3-2*X2)*0.5
      B1=X2
      A2=(Y3-2*Y2)*0.5
      B2=Y2
      GO TO 4
    2 X3=XG3-XG2
      Y3=YG3-YG2
      X1=XG1-XG2
      Y1=YG1-YG2
      A1=X1+X3
      B1=X3-X1
      A2=Y1+Y3
      B2=Y3-Y1
      GO TO 4
    3 X2=XG2-XG3
      Y2=YG2-YG3
      X1=XG1-XG3
      Y1=YG1-YG3
      A1=(X1-2*X2)*0.5
      B1=-X2
      A2=(Y1-2*Y2)*0.5
      B2=-Y2
    4 CONTINUE

      DO 10 J=1,3
   10 GW(J)=(0.,0.)
C
      DO 250 I=1,10
C
C  COMPUTE SHAPE FUNCTIONS FOR NUMERICAL INTEGRATION
C
      F3=0.5*GI(I)*(GI(I)+1.)
      F2=1.-GI(I)**2
      F1=0.5*GI(I)*(GI(I)-1.)
      FL3=GIL(I)*(2.*GIL(I)-1.)
      FL2=4.*GIL(I)*(1.-GIL(I))
      FL1=(GIL(I)-1.)*(2.*GIL(I)-1.)
      FLN3=0.5*GIL(I)*(GIL(I)+1.)
      FLN2=1.-GIL(I)**2
      FLN1=0.5*GIL(I)*(GIL(I)-1.)
C
C  COMPUTE THE GW MATRIX
C
      RANS=SQRT((GI(I)*A1+B1)**2+(GI(I)*A2+B2)**2)
      GO TO(50,60,70) NODO
C
   50 XJA1=SQRT((4*A1*GIL(I)-2*A1+0.5*X3)**2+(4*A2*GIL(I)-2*A2+0.5*Y3)**
     @  2)*2
      XJA2=SQRT((A1*GI(I)*2+0.5*X3)**2+(A2*GI(I)*2+0.5*Y3)**2)
      RA=RANS*ABS(1.+GI(I))
      ZR=FR*RA/CW
      Z=CMPLX(0.,ZR)
      CALL BESSEL(Z,F0C,F1C,F2C,F0CM,F1CM,F2CM)
      TERNS=F0C+ALOG(ABS(1.+GI(I))/2.)
      S3=FL3*XJA1*OMEL(I)+F3*XJA2*TERNS*OME(I)
      S2=FL2*XJA1*OMEL(I)+F2*XJA2*TERNS*OME(I)
      S1=FL1*XJA1*OMEL(I)+F1*XJA2*TERNS*OME(I)
      GO TO 200

   60 XJA1=SQRT((0.5*B1-A1*GIL(I))**2+(0.5*B2-A2*GIL(I))**2)
      XJA11=SQRT((0.5*B1+A1*GIL(I))**2+(0.5*B2+A2*GIL(I))**2)
      XJA2=SQRT((0.5*B1+A1*GI(I))**2+(0.5*B2+A2*GI(I))**2)
      RA=RANS*ABS(GI(I))/2.
      ZR=FR*RA/CW
      Z=CMPLX(0.,ZR)
      CALL BESSEL(Z,F0C,F1C,F2C,F0CM,F1CM,F2CM)
      TERNS=F0C+ALOG(ABS(GI(I)))
      S3=(FLN1*XJA1+FLN3*XJA11)*OMEL(I)+F3*XJA2*TERNS*OME(I)
      S2=FLN2*(XJA1+XJA11)*OMEL(I)+F2*XJA2*TERNS*OME(I)
      S1=(FLN3*XJA1+FLN1*XJA11)*OMEL(I)+F1*XJA2*TERNS*OME(I)
      GO TO 200

   70 XJA1=SQRT((2*A1-4*A1*GIL(I)-0.5*X1)**2+(2*A2-4*A2*GIL(I)-0.5*Y1)**
     @2)*2
      XJA2=SQRT((2*A1*GI(I)-0.5*X1)**2+(2*A2*GI(I)-0.5*Y1)**2)
      RA=RANS*ABS(GI(I)-1.)
      ZR=FR*RA/CW
      Z=CMPLX(0.,ZR)
      CALL BESSEL(Z,F0C,F1C,F2C,F0CM,F1CM,F2CM)
      TERNS=F0C+ALOG(ABS(GI(I)-1.)/2.)
      S3=FL1*XJA1*OMEL(I)+F3*XJA2*TERNS*OME(I)
      S2=FL2*XJA1*OMEL(I)+F2*XJA2*TERNS*OME(I)
      S1=FL3*XJA1*OMEL(I)+F1*XJA2*TERNS*OME(I)

  200 GW(3)= GW(3)+S3/(6.2831852*GE)
      GW(2)= GW(2)+S2/(6.2831852*GE)
      GW(1)= GW(1)+S1/(6.2831852*GE)
C
  250 CONTINUE
C
      RETURN
      END
C------------------------------------------------------------------------
      SUBROUTINE INTER2(FI,DFI,KODE,CX,CY,X,Y,SOL)
C
C  THIS SUBROUTINE REORDERS THE FI AND DFI ARRAYS TO PUT ALL THE VALUES
C  OF THE DISPLACEMENT IN FI AND ALL THE VALUES OF THE TRACTION IN DFI
C  THEN THE DISPLACEMENT IS COMPUTED AT INTERNAL POINTS.
C
      COMMON N,L,CW,GE,FR,INP,IPR
      DIMENSION KODE(1),CX(1),CY(1),X(1),Y(1),HS(3)
      COMPLEX HW(3),GW(3),FI(1),DFI(1),SOL(1),CH,CW,GE
C
C  REORDER THE FI AND DFI ARRAYS
C
      NE=N/2
      DO 180 I=1,NE
      DO 170 J=1,3
      IF(KODE(3*I-3+J)) 110,110,170
  110 IF((I-NE).NE.0 .OR. J.NE.3) GO TO 125
      IF(KODE(1)) 114,114,113
  113 CH=FI(1)*GE
      FI(1)=DFI(3*I)
      DFI(3*I)=CH
      GO TO 170
  114 DFI(3*I)=DFI(1)
      GO TO 170
  125 IF(I.EQ.1 .OR. J.GT.1 .OR. KODE(3*I-3).EQ.1) GO TO 130
      DFI(3*I-2)=DFI(3*I-3)
      GO TO 170
  130 CH=FI(2*I-2+J)*GE
      FI(2*I-2+J)=DFI(3*I-3+J)
      DFI(3*I-3+J)=CH
  170 CONTINUE
  180 CONTINUE
C
C  COMPUTE DISPLACEMENT VALUES AT INTERNAL POINTS
C
      IF(L.EQ.0)  RETURN
      DO 240 K=1,L
      SOL(K)=(0.,0.)
      DO 230 I=1,NE
      CALL EXTIN2(CX(K),CY(K),X(2*I-1),Y(2*I-1),X(2*I),Y(2*I),X(2*I+1),
     1Y(2*I+1),HW,GW,HS)
      DO 220 J=1,3
      IJ2=2*I-2+J
      IF(IJ2.GT.(2*NE)) IJ2=J-2
  220 SOL(K)=SOL(K)+GW(J)*DFI(3*I-3+J)-(HW(J)+HS(J))*FI(IJ2)
  230 CONTINUE
  240 CONTINUE
      RETURN
      END
C------------------------------------------------------------------------
      SUBROUTINE OUTPUT2(X,Y,FI,DFI,CX,CY,SOL)
C
C  THIS SUBROUTINE PRINTS THE VALUES OF THE DISPLACEMENT AND THE
C  TRACTION AT BOUNDARY NODES. IT ALSO PRINTS THE VALUES OF THE
C  DISPLACEMENT AT INTERNAL POINTS.
C
      DIMENSION X(1),Y(1),CX(1),CY(1)
      COMPLEX FI(1),DFI(1),SOL(1),CW,GE
      COMMON N,L,CW,GE,FR,INP,IPR
C
      NE=N/2
      WRITE(IPR,50)
   50 FORMAT(' ',79('*')/)
      WRITE(IPR,700) FR
  700 FORMAT(30X,'FREQUENCY =',G14.7/)
      WRITE(IPR,100)
  100 FORMAT(' ',79('*')//2X,'RESULTS'//2X,'BOUNDARY NODES'//
     153X,'TRACTION'/3X,'X',6X,'Y',8X,'DISPLACEMENT',9X,
     2'BEFORE NODE',11X,'AFTER NODE'/)
      WRITE(IPR,200) X(1),Y(1),FI(1),DFI(3*NE),DFI(1)
      WRITE(IPR,200) X(2),Y(2),FI(2),DFI(2),DFI(2)
      DO 10 I=2,NE
      WRITE(IPR,200) X(2*I-1),Y(2*I-1),FI(2*I-1),DFI(3*I-3),DFI(3*I-2)
   10 WRITE(IPR,200) X(2*I),Y(2*I),FI(2*I),DFI(3*I-1),DFI(3*I-1)
  200 FORMAT(2(1X,F6.2),3(1X,G10.3,',',G10.3))
      WRITE(IPR,300)
  300 FORMAT(//,2X,'INTERNAL POINTS',//5X,'X',8X,'Y',9X,
     1'DISPLACEMENT',/)
      DO 20 K=1,L
   20 WRITE(IPR,400)CX(K),CY(K),SOL(K)
  400 FORMAT(2(2X,F7.3),2X,G10.3,',',G10.3)
      WRITE(IPR,500)
  500 FORMAT(' ',79('*'))
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE CSOLVER(A,B,N,NX)
C
      COMPLEX A(NX,NX),B(NX),X,CDOT
C
C     A=L*U  FACTORIZATION
C
      DO 30 K=1,N
      K1=K-1
      DO 10 I=K,N
      A(I,K)=A(I,K)-CDOT(K1,A(I,1),NX,A(1,K),1)
   10 CONTINUE
      X=A(K,K)
      IF(ABS(REAL(X))+ABS(AIMAG(X)).EQ.0.) THEN
      WRITE(6,*) ' SOLVER : STOP  SINGULARITY IN ROW ',K
      STOP ' SOLVER : STOP SINGULARITY '
      END IF
      X=(1.,0.)/X
      DO 20 J=K+1,N
      A(K,J)=X*(A(K,J)-CDOT(K1,A(K,1),NX,A(1,J),1))
   20 CONTINUE
   30 CONTINUE
C
C  FORWARD AND BACK SUBSTITUTION    L*U*X=B
C
C  SOLVE  L*Y=B
C
      DO 40 I=1,N
      B(I)=-(CDOT((I-1),A(I,1),NX,B,1)+B(I))/A(I,I)
   40 CONTINUE
C
C  SOLVE  U*X=Y
C
      DO 50 I=N,1,-1
      B(I)=-(CDOT((N-I),A(I,I+1),NX,B(I+1),1)+B(I))
   50 CONTINUE
C
      RETURN
      END
C-----------------------------------------------------------------------
      COMPLEX FUNCTION CDOT(N,U,IUX,V,IVX)
C
C THIS FUNCTION MAKES THE COMPLEX DOT PRODUCT OF TWO VECTORS
C
      COMPLEX U(1),V(1)
C
      CDOT=(0.,0.)
      IF(N.LE.0) RETURN
      IU=1
      IV=1
      IF(IUX.LT.0) IU=1-(N-1)*IUX
      IF(IVX.LT.0) IV=1-(N-1)*IVX
      DO 10 I=1,N
      CDOT=CDOT+U(IU)*V(IV)
      IU=IU+IUX
      IV=IV+IVX
10    CONTINUE
      RETURN
C
      END

C-----------------------------------------------------------------------
      SUBROUTINE BESSEL(Z,FK0,FK1,FK2,FK0M,FK1M,FK2M)
C
C
C  THIS SUBROUTINE COMPUTES THE MODIFIED BESSEL FUNCTIONS OF THE SECOND
C  KIND AND ORDERS ZERO, ONE AND TWO (FK0,FK1,FK2) AND COMPLEX ARGUMENT
C  Z USING A POLINOMIUM
C
C  THE COEFFICIENTS OF THE POLINOMIUM HAVE BEEN OBTAINED FROM THE
C  TERMS OF THE SERIES EXPANSION
C
C  TWO DIFFEERENT SERIES HAVE BEEN USED. ONE FOR MODULUS(Z).LT.5
C  AND THE OTHER FOR MODULUS(Z).GT.5
C
C  FOR SMALL ARGUMENTS THE PARTS OF THE MODIFIED BESSEL FUNCTIONS
C  TENDING TO ZERO WITH THE ARGUMENT ARE COMPUTED SEPARATEDLY FROM
C  THE BESSEL FUNCTIONS AND STORED IN FK0M,FK1M AND FK2M. THESE
C  PARTS ARE ONLY USED BY THE QUADRATIC ELEMENT CODES IN ORDER
C  TO COMPUTE THE DIFFERENCE BETWEEN THE DYNAMIC AND THE STATIC
C  FUNDAMENTAL SOLUTION TRACTIONS

C
      COMPLEX  Z,FK0,FK1,FK2,ZZ,SER,FK0M,FK1M,FK2M,FI0M,FI1M
      REAL     CI0P(15),CI1P(15),CK0P(15),CK0G(12),CK1P(15),CK1G(12)
C
      DATA CI0P/ 1.00000000E+00, 0.62500000E+01, 0.97656250E+01,
     &           0.67816840E+01, 0.26490953E+01, 0.66227383E+00,
     &           0.11497810E+00, 0.14665573E-01, 0.14321849E-02,
     &           0.11050809E-03, 0.69067559E-05, 0.35675392E-06,
     &           0.15484111E-07, 0.57263725E-09, 0.18260116E-10/
      DATA CI1P/ 0.50000000E+00, 0.15625000E+01, 0.16276042E+01,
     &           0.84771050E+00, 0.26490953E+00, 0.55189486E-01,
     &           0.82127211E-02, 0.91659834E-03, 0.79565828E-04,
     &           0.55254047E-05, 0.31394345E-06, 0.14864747E-07,
     &           0.59554274E-09, 0.20451330E-10, 0.60867054E-12/
      DATA CK0P/-0.57721566E+00, 0.26424021E+01, 0.90115658E+01,
     &           0.85185931E+01, 0.39898493E+01, 0.11299171E+01,
     &           0.21532918E+00, 0.29560538E-01, 0.30657944E-02,
     &           0.24883689E-03, 0.16242981E-04, 0.87142913E-06,
     &           0.39112788E-07, 0.14905278E-08, 0.48833881E-10/
      DATA CK1P/ 1.00000000E+00, 0.96519581E+00,-0.26280638E+02,
     &          -0.44329875E+02,-0.29269699E+02,-0.10636897E+02,
     &          -0.24689720E+01,-0.39918196E+00,-0.47620526E-01,
     &          -0.43685559E-02,-0.31795287E-03,-0.18814687E-04,
     &          -0.92322279E-06,-0.38181087E-07,-0.13490886E-08/
      DATA CK0G/ 0.12533141E+01,-0.31332853E-01, 0.35249460E-02,
     &          -0.73436375E-03, 0.22489890E-03,-0.91084054E-04,
     &           0.45921544E-04,-0.27716932E-04, 0.19488468E-04,
     &          -0.15644909E-04, 0.14119530E-04,-0.14151620E-04/
      DATA CK1G/ 0.12533141E+01, 0.93998560E-01,-0.58749100E-02,
     &           0.10281093E-02,-0.28915573E-03, 0.11132496E-03,
     &          -0.54270916E-04, 0.31981075E-04,-0.22086930E-04,
     &           0.17485486E-04,-0.15605797E-04, 0.15499393E-04/
C
      R=CABS(Z)
      RR=5.0
C
      IF(R-RR) 10,20,20
C
   10 ZZ=Z/CMPLX(RR,0.)
      ZZ=ZZ*ZZ
C
C  COMPUTE THE NUMBER OF TERMS TO BE USED IN THE SERIES
C  FOR SMALL ARGUMENTS
C
      N=MAX0(2,MIN1(15.,24.177/ALOG(RR/R)))
C
C  COMPUTE  I0(Z)-1.
C
      FI0M=SER(CI0P,ZZ,2,N)
C
C  COMPUTE I1(Z)-Z/2
C
      FI1M=Z*SER(CI1P,ZZ,2,N)
C
C  COMPUTE K0M(Z) AND K0(Z)
C
      FK0M=-FI0M*CLOG(Z/(2.,0.))+SER(CK0P,ZZ,2,N)
      FK0=FK0M-CLOG(Z/(2.,0.))+CMPLX(CK0P(1),0.)
C
C  COMPUTE K1M(Z) AND K1(Z)
C
      FK1M=(FI1M*Z*CLOG(Z/(2.,0.))+SER(CK1P,ZZ,2,N))/Z
     &+CMPLX(CK0P(1)+0.5,0.)*Z/(2.,0.)
      FK1=FK1M+(1.,0.)/Z+Z*(CLOG(Z/(2.,0.))-CMPLX(CK0P(1)+0.5,0.))/2
C
C  COMPUTE K2M(Z) AND K2(Z)
C
      FK2M=FK0M+FK1M*(2.,0.)/Z
      FK2=FK0+FK1*(2.,0.)/Z
      GO TO 30

   20 ZZ=CMPLX(RR,0.)/Z
C
C  COMPUTE THE NUMBER OF TERMS TO BE USED IN THE SERIES
C  FOR LARGE ARGUMENTS
C
C  IN THIS CASE FK0M,FK1M AND FK2M ARE COMPUTED IN ORDER
C  TO BE CONSISTENT IN THE NUMERICAL INTEGRATION PROCESS
C
      N=MAX0(2,MIN1(12.,12.088/ALOG(R/RR)))
C
C  COMPUTE K0(Z) AND K0M(Z)
C
      FK0=(CMPLX(CK0G(1),0.)+SER(CK0G,ZZ,2,N))*CEXP(-Z)/CSQRT(Z)
      FK0M=FK0+CLOG(Z/(2.,0.))-CMPLX(CK0P(1),0.)
C
C  COMPUTE K1(Z) AND K1M(Z)
C
      FK1=(CMPLX(CK1G(1),0.)+SER(CK1G,ZZ,2,N))*CEXP(-Z)/CSQRT(Z)
      FK1M=FK1-(1.,0.)/Z-Z*(CLOG(Z/(2.,0.))-CMPLX(CK0P(1)+0.5,0.))/2
C
C  COMPUTE K2(Z) AND K2M(Z)
C
      FK2=FK0+FK1*(2.,0.)/Z
      FK2M=FK0M+FK1M*(2.,0.)/Z
C
   30 RETURN
      END
C-----------------------------------------------------------------------
      COMPLEX FUNCTION SER(S,ZZ,N1,N2)
C
C  THIS FUNCTION COMPUTES THE VALUE OF A SERIES OF N2-N1+1
C  TERMS CONSISTING OF INCREASING POWERS OF ZZ TIMES THE
C  COEFFICIENTS IN ARRAY S
C
      COMPLEX  ZZ,ZZZ
      REAL     S(N2)
C
      SER=(0.,0.)
      ZZZ=(1.,0.)
      DO 10 I=N1,N2
      ZZZ=ZZZ*ZZ
      SER=SER+CMPLX(S(I),0.)*ZZZ
   10 CONTINUE
C
      RETURN
      END
