C------------------------------------------------------------------
      PROGRAM QUADPLED
C
C  THIS IS A B. E. PROGRAM THAT USES (QUA)DRATIC ELEMENTS
C  FOR THE SOLUTION OF (D)YNAMIC (PL)ANE (E)LASTIC PROBLEMS
C  USING THE (D)UAL RECIPROCITY APPROACH
C
      CHARACTER*12 FILEIN,FILEOUT
C
C  SET MAXIMUM NUMBER OF ELEMENTS, NODES AND INTERNAL POINTS
C
      PARAMETER(NELMAX=25,NNOMAX=2*NELMAX,NIPMAX=20)
C
C  SET MAXIMUM NUMBER OF TIME FUNCTIONS AND MAXIMUM NUMBER OF POINTS
C  THAT DEFINE ONE OF THESE TIME FUNCTIONS.
C
      PARAMETER(NFMAX=10,NPFMAX=15)
C
C  SET MAXIMUM DIMENSION OF THE SYSTEM OF EQUATIONS (NX)
C  NX=   2*(MAXIMUM NUMBER OF NODES + MAXIMUM NUMBER OF INTERNAL POINTS)
C  NX1=  6*MAXIMUM NUMBER OF ELEMENTS = 3*MAXIMUM NUMBER OF NODES
C
      PARAMETER(NX=2*(NNOMAX+NIPMAX),NX1=6*NELMAX)
C
      COMMON/MATE/ E(NX,NX)
      COMMON/MATM/ XM(NX,NX)
      COMMON/MATG/ G(NX,NX1)
      COMMON/MATH/ H(NX,NX)
      COMMON N,NI,RO,GE,XNU,INP,IPR,NSTEP,XT,C
      DIMENSION X(NNOMAX+1),Y(NNOMAX+1)
      DIMENSION T(NX1),U(NX,3),KODE(NX1)
      DIMENSION CX(NIPMAX),CY(NIPMAX)
      DIMENSION FSCALE(NFMAX),FUNC(NFMAX,NPFMAX,2),BC(NX1),IFUNC(NX1)
      DIMENSION ETA(NX1,2),PSI(NX,2)
      DIMENSION AUX(NX),AUX2(NX),IAUX(NX)
C
C  ASSIGN NUMBERS FOR INPUT AND OUTPUT FILES
C
      INP=5
      IPR=6
C
C  READ NAMES AND OPEN  FILES FOR INPUT AND OUTPUT
C
      WRITE(*,' (A) ') ' NAME OF INPUT FILE (MAX. 12 CHART.)'
      READ(*,'  (A) ')FILEIN
      OPEN(INP,FILE=FILEIN,STATUS='OLD')
      WRITE(*,' (A) ') ' NAME OF OUTPUT FILE (MAX. 12 CHART.)'
      READ(*,' (A) ')FILEOUT
      OPEN(IPR,FILE=FILEOUT,STATUS='NEW')
C
C  READ DATA
C
      CALL INPUT13(CX,CY,X,Y,KODE,BC,IFUNC,FSCALE,FUNC,
     1NFMAX,NPFMAX,INTIMP)
C
C  COMPUTE G AND H MATRICES
C
      CALL GHMAT13(X,Y,CX,CY,G,H,NX,NX1)
C
C  COMPUTE E MATRIX
C
      CALL EMAT13(X,Y,CX,CY,E,AUX,IAUX,NX)
C
C  COMPUTE MASS MATRIX XM
C
      CALL XMMAT13(X,Y,CX,CY,XM,G,H,E,AUX,ETA,PSI,NX,NX1)
C
C  PREPARE SYSTEM OF EQUATIONS FOR THE STEP BY STEP PROCESS
C
      CALL PSOLVE13(U,G,H,XM,KODE,AUX,IAUX,NX,NX1)
C
C  SOLVE SYSTEM OF EQUATIONS (STEP BY STEP PROCESS)
C
      DO 10 ISTEP=1,NSTEP
        CALL SOLVER13(U,T,G,H,XM,KODE,BC,IFUNC,FSCALE,FUNC,
     1AUX,AUX2,NX,NX1,ISTEP,NFMAX,NPFMAX)
C
C  PRINT RESULTS AT BOUNDARY NODES FOR THE PRESENT TIME STEP (ISTEP)
C
      IF(((ISTEP/INTIMP)*INTIMP).NE.ISTEP.AND.ISTEP.NE.1) GO TO 10
      CALL OUTPUT13(X,Y,U,T,ISTEP,NX)
C
10    CONTINUE
C
C   CLOSE INPUT AND OUTPUT FILES
C
      CLOSE (INP)
      CLOSE (IPR)
      STOP
      END
C-----------------------------------------------------------------------
      SUBROUTINE INPUT13(CX,CY,X,Y,KODE,BC,IFUNC,FSCALE,FUNC,
     1NFMAX,NPFMAX,INTIMP)
C
C  NE= NUMBER OF BOUNDARY ELEMENTS
C  NI= NUMBER OF INTERNAL POINTS
C  GE= SHEAR MODULUS
C  XNU= POISSON RATIO
C
      CHARACTER*80 TITLE
      DIMENSION KODE(1),X(1),Y(1),CX(1),CY(1)
      DIMENSION BC(1),IFUNC(1),FSCALE(1),FUNC(NFMAX,NPFMAX,2)
      COMMON N,NI,RO,GE,XNU,INP,IPR,NSTEP,XT,C
C
      WRITE(IPR,100)
  100 FORMAT(' ',79('*'))
C
C  READ JOB TITLE
C
      READ(INP,'(A)') TITLE
      WRITE(IPR,'(A)') TITLE
C
C  READ NUMBER OF BOUNDARY ELEMENTS AND INTERNAL POINTS;
C  READ MATERIAL PROPERTIES
C
      READ(INP,*)NE,NI,GE,XNU,RO
      WRITE(IPR,210)NE,NI,GE,XNU,RO
  210 FORMAT(//2X,'DATA'//2X,
     1       'NUMBER OF BOUNDARY ELEMENTS= ',I3/2X,
     2       'NUMBER OF INTERNAL POINTS  = ',I3/2X,
     3       'SHEAR MODULUS              = ',E14.7/2X,
     4       'POISSON RATIO              = ',E14.7/2X,
     5       'DENSITY                    = ',E14.7)
C
C  READ NUMBER AND SIZE OF TIME INTERVALS, READ PRINTING INTERVAL
C
      READ(INP,*) NSTEP,XT,INTIMP
      WRITE(IPR,220) NSTEP,XT,INTIMP
  220 FORMAT(/2X,
     1       'NUMBER OF TIME STEPS       = ',I3/2X,
     2       'SIZE OF TIME INTERVALS     = ',E14.7/2X,
     3       'INTERVAL OF PRINTING       = ',I3)
C
C  READ THE "C" CONSTANT
C
      READ(INP,*) C
      WRITE(IPR,230) C
  230 FORMAT(/2X,
     1       '"C" CONSTANT               = ',E14.7)
C
C  READ BOUNDARY NODES COORDINATES
C
      N=2*NE
      WRITE(IPR,500)
      READ(INP,*) (X(I),Y(I),I=1,N)
      DO 10 I=1,N
10    WRITE(IPR,240) I,X(I),Y(I)
  500 FORMAT(//2X,'BOUNDARY NODES COORDINATES'///4X,
     1       'NODE',10X,'X',18X,'Y'/)
  240 FORMAT(5X,I3,2(5X,E14.7))
C
C  READ INTERNAL POINTS COORDINATES
C
      IF(NI.EQ.0) GO TO 20
      WRITE(IPR,600)
      READ(INP,*) (CX(I),CY(I),I=1,NI)
      DO 15 I=1,NI
15    WRITE(IPR,240) I,CX(I),CY(I)
  600 FORMAT(//2X,'INTERNAL POINTS COORDINATES'///4X,
     1       'POINT',10X,'X',18X,'Y'/)
20    CONTINUE
C
C  READ BOUNDARY CONDITIONS CODES:
C  IF KODE(I)=0 THE B.C. IS A KNOWN DISPLACEMENT;
C  IF KODE(I)=1 THE B.C. IS A KNOWN TRACTION.
C  SIX BOUNDARY CONDITION CODES ARE READ PER ELEMENT.
C  NODES BETWEEN TWO ELEMENTS MAY HAVE TWO DIFFERENT VALUES
C  OF THE TRACTION BUT ONLY ONE VALUE OF THE DISPLACEMENT.
C
      DO 30 I=1,NE
      READ(INP,*) (KODE(6*I-6+J),J=1,6)
30    CONTINUE
C
C  READ BOUNDARY CONDITIONS:
C
C  1.- DEFINITION OF THE TIME DEPENDENT FUNCTIONS
C
C  READ NUMBER OF FUNCTIONS
C
      READ(INP,*) NFUN
      WRITE(IPR,700) NFUN
  700 FORMAT(//2X,'TIME FUNCTIONS'//2X,
     1       'NUMBER OF TIME FUNCTIONS   = ',I3)
      DO 40 I=1,NFUN
C
C  READ NUMBER OF THE FUNCTION, NUMBER OF POINTS THAT DEFINE THE
C  FUNCTION AND SCALE ASSOCIATED TO THE FUNCTION
C
      READ(INP,*) NF,NPT,EX
      WRITE(IPR,725) NF,NPT,EX
  725 FORMAT(/2X,'FUNCTION NUMBER  = ',I3/2X,
     1       'NUMBER OF POINTS = ',I3/2X,
     2       'FUNCTION SCALE   = ',E14.7//4X
     3       'POINT',8X,'TIME',11X,'VALUE')
      FSCALE(I)=EX
C
C  FOR ALL THE POINTS THAT DEFINE THE FUNCTION, READ POINT AND
C  VALUE OF THE FUNCTION IN THAT POINT
C
      DO 40 J=1,NPT
      READ(INP,*) FUNC(I,J,1),FUNC(I,J,2)
      WRITE(IPR,750) J,FUNC(I,J,1),FUNC(I,J,2)
40    CONTINUE
  750 FORMAT(5X,I3,4X,E14.7,2X,E14.7)
C
C  2.- READ, FOR EACH ELEMENT, THE TIME-FUNCTION THAT DEFINES
C      THE B.C. AT ITS NODES AND THE SCALE ASSOCIATED TO EACH
C      NODE FOR DIRECTIONS XI AND YI.
C
      WRITE(IPR,775)
  775 FORMAT(//2X,'BOUNDARY CONDITIONS'//6X,
     1       '------FIRST NODE------',2X,'-----SECOND NODE------',2X,
     2       '------THIRD NODE------'/1X,'ELE',
     3       3(2X,'X DIR. C F  Y DIR. C F'))
      DO 50 I=1,NE
      READ(INP,*) (IFUNC(6*(I-1)+J),BC(6*(I-1)+J),J=1,6)
      WRITE(IPR,825) I,(BC(6*(I-1)+J),KODE(6*(I-1)+J),
     1IFUNC(6*(I-1)+J),J=1,6)
50    CONTINUE
  825 FORMAT(1X,I3,3(F8.2,1X,I1,I2,F8.2,1X,I1,I2))
C
      RETURN
      END
C-----------------------------------------------------------------------
      FUNCTION FUNCVAL(NF,TT,SCALE,NPOIN,FUNC,NFM,NPM)
      DIMENSION FUNC(NFM,NPM,2)
      FUNCVAL=FUNC(NF,1,2)*SCALE
      IF(SCALE.EQ.0.0) RETURN
      DO 10 I=1,NPOIN-1
      IF(TT.LT.FUNC(NF,I,1).OR.TT.GE.FUNC(NF,I+1,1)) GO TO 10
      FUNCVAL=SCALE*(FUNC(NF,I,2)+(FUNC(NF,I+1,2)-FUNC(NF,I,2))/
     &        (FUNC(NF,I+1,1)-FUNC(NF,I,1))*(TT-FUNC(NF,I,1)))
      RETURN
   10 CONTINUE
      RETURN
      END
C------------------------------------------------------------------------
      SUBROUTINE GHMAT13(X,Y,CX,CY,G,H,NX,NX1)
C
C  THIS ROUTINE COMPUTES THE G AND H MATRICES
C  H IS A SQUARE MATRIX (2*(N+NI),2*(N+NI))
C  G IS RECTANGULAR (2*(N+NI),6*NE)
C
      DIMENSION X(1),Y(1),CX(1),CY(1)
      DIMENSION G(NX,NX1),H(NX,NX),HW(2,6),GW(2,6)
      COMMON N,NI,RO,GE,XNU,INP,IPR,NSTEP,XT,C
      NN=2*N
      NE=N/2
      DO 20 I=1,NN
      DO 11 J=1,NN
   11 H(I,J)=0.
      DO 12 J=1,3*N
   12 G(I,J)=0.
   20 CONTINUE
      X(N+1)=X(1)
      Y(N+1)=Y(1)
C
C  COMPUTE THE GW AND HW MATRICES FOR EACH COLLOCATION
C  POINT AND EACH BOUNDARY ELEMENT
C
      DO 40 LL=1,N
      DO 40 I=1,N-1,2
      IF((LL-I)*(LL-I-1)*(LL-I-2)*(LL-I+N-2)) 22,21,22
   21 NODO=LL-I+1
      IF((LL.EQ.1).AND.(I.EQ.N-1)) NODO=NODO+N
      CALL EXTIN13(X(LL),Y(LL),X(I),Y(I),X(I+1),Y(I+1),X(I+2),Y(I+2),
     *HW,GW)
      CALL LOCIN13(X(I),Y(I),X(I+1),Y(I+1),X(I+2),Y(I+2),GW,NODO)
      GO TO 34
   22 CALL EXTIN13(X(LL),Y(LL),X(I),Y(I),X(I+1),Y(I+1),X(I+2),Y(I+2),
     *HW,GW)
C
C  PLUG THE GW AND HW MATRICES INTO THE GENERAL G AND H MATRICES.
C
   34 DO 39 K=1,2
      DO 38 J=1,6
      G(2*LL-2+K,3*I-3+J)=G(2*LL-2+K,3*I-3+J)+GW(K,J)
      IF(I-N+1) 37,35,37
   35 IF(J-5) 37,36,36
   36 H(2*LL-2+K,J-4)=H(2*LL-2+K,J-4)+HW(K,J)
      GO TO 38
   37 H(2*LL-2+K,2*I-2+J)=H(2*LL-2+K,2*I-2+J)+HW(K,J)
   38 CONTINUE
   39 CONTINUE
   40 CONTINUE
C
C  COMPUTE THE DIAGONAL COEFFICIENTS OF THE H MATRIX
C
      DO 70 I=1,N
      H(2*I-1,2*I-1)=0.
      H(2*I,2*I-1)=0.
      H(2*I-1,2*I)=0.
      H(2*I,2*I)=0.
      DO 60 J=1,N
      IF(I.EQ.J) GO TO 60
      H(2*I-1,2*I-1)=H(2*I-1,2*I-1)-H(2*I-1,2*J-1)
      H(2*I,2*I-1)=H(2*I,2*I-1)-H(2*I,2*J-1)
      H(2*I-1,2*I)=H(2*I-1,2*I)-H(2*I-1,2*J)
      H(2*I,2*I)=H(2*I,2*I)-H(2*I,2*J)
   60 CONTINUE
   70 CONTINUE
C
C  GENERATION OF THE INTERNAL POINTS EQUATIONS
C
      IF (NI.EQ.0) RETURN
      DO 80 I=1,2*N
      DO 80 J=2*N+1,2*(N+NI)
      H(I,J)=0.
   80 CONTINUE
      DO 90 I=2*N+1,2*(N+NI)
      DO 100 J=1,3*N
      G(I,J)=0.
  100 CONTINUE
      DO 110 J=1,2*(N+NI)
      H(I,J)=0.
  110 CONTINUE
   90 CONTINUE
C
C  COMPUTE THE GW AND HW MATRICES WHEN THE VIRTUAL FORCE IS
C  APPLIED IN THE INTERNAL POINTS
C
      DO 120 LL=1,NI
      DO 120 I=1,N-1,2
      CALL EXTIN13(CX(LL),CY(LL),X(I),Y(I),X(I+1),Y(I+1),X(I+2),Y(I+2),
     *HW,GW)
C
C  PLUG THE GW AND HW MATRICES INTO THE GENERAL G AND H MATRICES.
C
      DO 119 K=1,2
      DO 118 J=1,6
      G(2*(LL+N)-2+K,3*I-3+J)=G(2*(LL+N)-2+K,3*I-3+J)+GW(K,J)
      IF(I-N+1) 117,115,117
  115 IF(J-5) 117,116,116
  116 H(2*(LL+N)-2+K,J-4)=H(2*(LL+N)-2+K,J-4)+HW(K,J)
      GO TO 118
  117 H(2*(LL+N)-2+K,2*I-2+J)=H(2*(LL+N)-2+K,2*I-2+J)+HW(K,J)
  118 CONTINUE
  119 CONTINUE
  120 CONTINUE
C
C  PUT ONE IN THE DIAGONAL OF THE INTERNAL POINT EQUATIONS (I MATRIX)
C
      DO 130 I=2*N+1,2*(N+NI)
      H(I,I)=1.
  130 CONTINUE
C
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE EXTIN13(XP,YP,X1,Y1,X2,Y2,X3,Y3,HW,GW)
C
C  THIS SUBRUOTINE COMPUTES THE HW AND GW MATRICES
C  THAT RELATE A NODE (XP,YP) WITH A BOUNDARY
C  ELEMENT USING GAUSS QUADRATURE.
C
C  RA           = RADIUS
C  RD1,RD2,RDN  = RADIUS DERIVATIVES
C  ETA1,ETA2    = COMPONENTS OF THE UNIT NORMAL TO THE ELEMENT
C  XCO,YCO      = INTEGRATION POINT ALONG THE ELEMENT
C  XJA          = JACOBIAN
C
      COMMON N,NI,RO,GE,XNU,INP,IPR,NSTEP,XT
      DIMENSION GW(2,6),HW(2,6)
      DIMENSION GI(10),OME(10)
      DATA GI/0.9739065285,-0.9739065285,0.8650633666,-0.8650633666
     @,0.6794095683,-0.6794095682,0.4333953941,-0.4333953941,
     @0.1488743389,-0.1488743389/
      DATA OME/0.0666713443,0.0666713443,0.1494513491,0.1494513491
     @,0.2190863625,0.2190863625,0.2692667193,0.2692667193,
     @0.2955242247,0.2955242247/
C
      DO 30 I=1,2
      DO 20 J=1,6
      HW(I,J)=0.
   20 GW(I,J)=0.
   30 CONTINUE
      A=X3-2*X2+X1
      B=(X3-X1)/2
      C=Y3-2*Y2+Y1
      D=(Y3-Y1)/2
      DE=4*3.141592*(1-XNU)
      DO 40 I=1,10
C
C  COMPUTE THE VALUES OF THE SHAPE FUNCTIONS AT THE
C  INTEGRATION POINTS
C
      F1=GI(I)*(GI(I)-1)*0.5
      F2=1.-GI(I)**2
      F3=GI(I)*(GI(I)+1)*0.5
C
C  COMPUTE GEOMETRICAL PROPERTIES AT THE INTEGRATION POINTS
C
      XCO=X1*F1+X2*F2+X3*F3
      YCO=Y1*F1+Y2*F2+Y3*F3
      XJA=SQRT((GI(I)*A+B)**2+(GI(I)*C+D)**2)
      ETA1=(GI(I)*C+D)/XJA
      ETA2=-(GI(I)*A+B)/XJA
      RA=SQRT((XP-XCO)**2+(YP-YCO)**2)
      RD1=(XCO-XP)/RA
      RD2=(YCO-YP)/RA
      RDN=RD1*ETA1+RD2*ETA2
C
C  COMPUTE GW AND HW MATRICES
C
      GW(1,1)=GW(1,1)+((3-4*XNU)*ALOG(1./RA)+RD1**2)*OME(I)*XJA*F1/
     1(2*DE*GE)
      GW(1,2)=GW(1,2)+RD1*RD2*OME(I)*XJA*F1/(2*DE*GE)
      GW(2,1)=GW(1,2)
      GW(2,2)=GW(2,2)+((3-4*XNU)*ALOG(1./RA)+RD2**2)*OME(I)*XJA*F1/
     1(2*DE*GE)
      HW(1,1)=HW(1,1)-RDN*((1-2*XNU)+2*RD1**2)/(RA*DE)*OME(I)*XJA*F1
      HW(1,2)=HW(1,2)-(RDN*2*RD1*RD2+(1-2*XNU)*(ETA1*RD2-ETA2*RD1))*
     1OME(I)*XJA*F1/(RA*DE)
      HW(2,1)=HW(2,1)-(RDN*2*RD1*RD2+(1-2*XNU)*(ETA2*RD1-ETA1*RD2))*
     1OME(I)*XJA*F1/(RA*DE)
      HW(2,2)=HW(2,2)-RDN*((1-2*XNU)+2*RD2**2)*OME(I)*XJA*F1/(RA*DE)
      GW(1,3)=GW(1,3)+((3-4*XNU)*ALOG(1./RA)+RD1**2)*OME(I)*XJA*F2/
     1(2*DE*GE)
      GW(1,4)=GW(1,4)+RD1*RD2*OME(I)*XJA*F2/(2*DE*GE)
      GW(2,3)=GW(1,4)
      GW(2,4)=GW(2,4)+((3-4*XNU)*ALOG(1./RA)+RD2**2)*OME(I)*XJA*F2/
     1(2*DE*GE)
      HW(1,3)=HW(1,3)-RDN*((1-2*XNU)+2*RD1**2)/(RA*DE)*OME(I)*XJA*F2
      HW(1,4)=HW(1,4)-(RDN*2*RD1*RD2+(1-2*XNU)*(ETA1*RD2-ETA2*RD1))*
     1OME(I)*XJA*F2/(RA*DE)
      HW(2,3)=HW(2,3)-(RDN*2*RD1*RD2+(1-2*XNU)*(ETA2*RD1-ETA1*RD2))*
     1OME(I)*XJA*F2/(RA*DE)
      HW(2,4)=HW(2,4)-RDN*((1-2*XNU)+2*RD2**2)*OME(I)*XJA*F2/(RA*DE)
      GW(1,5)=GW(1,5)+((3-4*XNU)*ALOG(1./RA)+RD1**2)*OME(I)*XJA*F3/
     1(2*DE*GE)
      GW(1,6)=GW(1,6)+RD1*RD2*OME(I)*XJA*F3/(2*DE*GE)
      GW(2,5)=GW(1,6)
      GW(2,6)=GW(2,6)+((3-4*XNU)*ALOG(1./RA)+RD2**2)*OME(I)*XJA*F3/
     1(2*DE*GE)
      HW(1,5)=HW(1,5)-RDN*((1-2*XNU)+2*RD1**2)/(RA*DE)*OME(I)*XJA*F3
      HW(1,6)=HW(1,6)-(RDN*2*RD1*RD2+(1-2*XNU)*(ETA1*RD2-ETA2*RD1))*
     1OME(I)*XJA*F3/(RA*DE)
      HW(2,5)=HW(2,5)-(RDN*2*RD1*RD2+(1-2*XNU)*(ETA2*RD1-ETA1*RD2))*
     1OME(I)*XJA*F3/(RA*DE)
   40 HW(2,6)=HW(2,6)-RDN*((1-2*XNU)+2*RD2**2)*OME(I)*XJA*F3/(RA*DE)
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE LOCIN13(XG1,YG1,XG2,YG2,XG3,YG3,GW,NODO)
C
C  THIS SUBROUTINE COMPUTES THE GW MATRIX WHEN THE COLLOCATION
C  POINT IS ONE OF THE NODES OF THE INTEGRATION ELEMENT.
C  THE COEFFICIENTS ARE COMPUTED BY NUMERICAL INTEGRATION:
C  THE NON SINGULAR PART IS COMPUTED USING STANDARD GAUSS QUADRATURE,
C  THE LOGARITHMIC PART IS COMPUTED USING A SPECIAL QUADRATURE FORMULA.
C
      COMMON N,NI,RO,GE,XNU,INP,IPR,NSTEP,XT
      DIMENSION GI(10),OME(10),GIL(10),OMEL(10),GW(2,6)
C
C  DATA FOR THE GAUSS QUADRATURE
C
      DATA GI/0.9739065285,-0.9739065285,0.8650633666,-0.8650633666
     @,0.6794095682,-0.6794095682,0.4333953941,-0.4333953941,
     @0.1488743389,-0.1488743389/
      DATA OME/0.0666713443,0.0666713443,0.1494513491,0.1494513491
     @,0.2190863625,0.2190863625,0.2692667193,0.2692667193,
     @0.2955242247,0.2955242247/
C
C  DATA FOR THE SPECIAL QUADRATURE
C
      DATA GIL/0.0090426309,0.0539712662,0.1353118246,0.2470524162
     @,0.3802125396,0.5237923179,0.6657752055,0.7941904160,
     @0.8981610912,0.9688479887/
      DATA OMEL/0.1209551319,0.1863635425,0.1956608732,0.1735771421
     @,0.1356956729,0.0936467585,0.0557877273,0.0271598109,
     @0.0095151826,0.0016381576/
C
C  SET LOCAL COORDINATES SYSTEM
C
      GOTO(1,2,3),NODO
    1 X3=XG3-XG1
      Y3=YG3-YG1
      X2=XG2-XG1
      Y2=YG2-YG1
      A1=(X3-2*X2)*0.5
      B1=X2
      A2=(Y3-2*Y2)*0.5
      B2=Y2
      GO TO 4
    2 X3=XG3-XG2
      Y3=YG3-YG2
      X1=XG1-XG2
      Y1=YG1-YG2
      A1=X1+X3
      B1=X3-X1
      A2=Y1+Y3
      B2=Y3-Y1
      GO TO 4
    3 X2=XG2-XG3
      Y2=YG2-YG3
      X1=XG1-XG3
      Y1=YG1-YG3
      A1=(X1-2*X2)*0.5
      B1=-X2
      A2=(Y1-2*Y2)*0.5
      B2=-Y2
    4 CONTINUE
C
      DO 10 I=1,2
      DO 10 J=1,6
   10 GW(I,J)=0.
      A=A1**2 + A2**2
      B=2*(A1*B1 + A2*B2)
      C=B1**2 + B2**2
      CONT1=(3-4*XNU)/(8.*3.1415926*GE*(1-XNU))
      CONT2=CONT1/(3-4*XNU)
      DO 250 I=1,10
      T1=((A1*GI(I)+B1)**2)/((A2*GI(I)+B2)**2 + (A1*GI(I)+B1)**2)
      T2=((A2*GI(I)+B2)**2)/((A2*GI(I)+B2)**2 + (A1*GI(I)+B1)**2)
      T3=(A1*GI(I)+B1)*(A2*GI(I)+B2)/((A1*GI(I)+B1)**2+(A2*GI(I)+B2)**2)
C
C  COMPUTE SHAPE FUNTIONS FOR NUMERICAL INTEGRATION
C
      F3=0.5*GI(I)*(GI(I)+1.)
      F2=1.-GI(I)**2
      F1=0.5*GI(I)*(GI(I)-1.)
      FL3=GIL(I)*(2.*GIL(I)-1.)
      FL2=4.*GIL(I)*(1.-GIL(I))
      FL1=(GIL(I)-1.)*(2.*GIL(I)-1.)
      FLN3=0.5*GIL(I)*(GIL(I)+1.)
      FLN2=1.-GIL(I)**2
      FLN1=0.5*GIL(I)*(GIL(I)-1.)
C
C  COMPUTE GW COEFFICIENTS
C
      GO TO(50,60,70) NODO
   50 XJA1=SQRT((4*A1*GIL(I)-2*A1+0.5*X3)**2+(4*A2*GIL(I)-2*A2+0.5*Y3)**
     *2)*2
      XJA2=SQRT((A1*GI(I)*2+0.5*X3)**2+(A2*GI(I)*2+0.5*Y3)**2)
      XLO=-ALOG(2*SQRT((GI(I)*A1+B1)**2+(GI(I)*A2+B2)**2))
      S3=CONT1*(FL3*XJA1*OMEL(I)+F3*XJA2*XLO*OME(I))
      S2=CONT1*(FL2*XJA1*OMEL(I)+F2*XJA2*XLO*OME(I))
      S1=CONT1*(FL1*XJA1*OMEL(I)+F1*XJA2*XLO*OME(I))
      GO TO 200
   60 XJA1=SQRT((0.5*B1-A1*GIL(I))**2+(0.5*B2-A2*GIL(I))**2)
      XJA11=SQRT((0.5*B1+A1*GIL(I))**2+(0.5*B2+A2*GIL(I))**2)
      XJA2=SQRT((0.5*B1+A1*GI(I))**2+(0.5*B2+A2*GI(I))**2)
      XLO=-0.5*ALOG((GI(I)*A1*0.5+B1*0.5)**2+(GI(I)*A2*0.5+B2*0.5)**2)
      S3=CONT1*((FLN1*XJA1+FLN3*XJA11)*OMEL(I)+F3*XJA2*XLO*OME(I))
      S2=CONT1*(FLN2*(XJA1+XJA11)*OMEL(I)+F2*XJA2*XLO*OME(I))
      S1=CONT1*((FLN3*XJA1+FLN1*XJA11)*OMEL(I)+F1*XJA2*XLO*OME(I))
      GO TO 200
   70 XJA1=SQRT((2*A1-4*A1*GIL(I)-0.5*X1)**2+(2*A2-4*A2*GIL(I)-0.5*Y1)**
     *2)*2
      XJA2=SQRT((2*A1*GI(I)-0.5*X1)**2+(2*A2*GI(I)-0.5*Y1)**2)
      XLO=-ALOG(2*SQRT((A1*GI(I)+B1)**2+(A2*GI(I)+B2)**2))
      S3=CONT1*(FL1*XJA1*OMEL(I)+F3*XJA2*XLO*OME(I))
      S2=CONT1*(FL2*XJA1*OMEL(I)+F2*XJA2*XLO*OME(I))
      S1=CONT1*(FL3*XJA1*OMEL(I)+F1*XJA2*XLO*OME(I))
  200 GW(1,5)= GW(1,5)+S3+CONT2*F3*T1*XJA2*OME(I)
      GW(1,3)= GW(1,3)+S2+CONT2*F2*T1*XJA2*OME(I)
      GW(1,1)= GW(1,1)+S1+CONT2*F1*T1*XJA2*OME(I)
      GW(1,6)= GW(1,6)+ CONT2*T3*XJA2*F3*OME(I)
      GW(1,4)= GW(1,4)+ CONT2*T3*XJA2*F2*OME(I)
      GW(1,2)= GW(1,2)+ CONT2*T3*XJA2*F1*OME(I)
      GW(2,5)= GW(1,6)
      GW(2,3)= GW(1,4)
      GW(2,1)=GW(1,2)
      GW(2,6)= GW(2,6)+ S3+CONT2*T2*XJA2*F3*OME(I)
      GW(2,4)=GW(2,4)+S2+CONT2*T2*XJA2*F2*OME(I)
      GW(2,2)= GW(2,2)+ S1+CONT2*T2*XJA2*F1*OME(I)
C
  250 CONTINUE
C
      RETURN
      END
C----------------------------------------------------
      SUBROUTINE EMAT13(X,Y,CX,CY,E,AUX,IAUX,NX)
C
C  THIS ROUTINE COMPUTES AND INVERTS THE F MATRIX
C  AND INVERTS IT TO OBTAIN THE E MATRIX.
C  F IS A SQUARE SYMMETRIC MATRIX (2*(N+NI),2*(N+NI))
C
      DIMENSION X(1),Y(1),CX(1),CY(1)
      DIMENSION AUX(1),IAUX(1)
      DIMENSION E(NX,NX)
      COMMON N,NI,RO,GE,XNU,INP,IPR,NSTEP,XT,C
C
      DO 10 I=1,N+NI
      DO 10 J=1,N+NI
      IF(I.LE.N) THEN
      XI=X(I)
      YI=Y(I)
      ELSE
      XI=CX(I-N)
      YI=CY(I-N)
      ENDIF
      IF(J.LE.N) THEN
      XJ=X(J)
      YJ=Y(J)
      ELSE
      XJ=CX(J-N)
      YJ=CY(J-N)
      ENDIF
      RA=SQRT((XJ-XI)**2+(YJ-YI)**2)
      E(2*I-1,2*J-1)=C-RA
      E(2*I,2*J)=E(2*I-1,2*J-1)
      E(2*I-1,2*J)=0.
      E(2*I,2*J-1)=0.
   10 CONTINUE
C
C  INVERT F
C
      NGL=2*(N+NI)
      CALL INVERT(E,IAUX,AUX,NGL,NX)
C
      RETURN
      END
C------------------------------------------------------------
      SUBROUTINE XMMAT13(X,Y,CX,CY,XM,G,H,E,V,ETA,PSI,NX,NX1)
C
C  THIS ROUTINE COMPUTES THE MASS MATRIX XM
C  XM IS A SQUARE MATRIX (2*(N+NI),2*(N+NI))
C
      DIMENSION X(1),Y(1),CX(1),CY(1),G(NX,NX1),H(NX,NX),
     *E(NX,NX),XM(NX,NX)
      DIMENSION PSI(NX,2),ETA(NX1,2),V(1)
      COMMON N,NI,RO,GE,XNU,INP,IPR,NSTEP,XT,C
C
      DO 10 I=1,2*(N+NI)
      DO 10 J=1,2*(N+NI)
      XM(I,J)=0.
10    CONTINUE
C
C  COMPUTE THE PRODUCT G*ETA-H*PSI
C
      DO 70 M=1,N+NI
      CALL MATPSI13(X,Y,CX,CY,PSI,M,NX)
      CALL MATETA13(X,Y,CX,CY,ETA,M,NX1)
      DO 60 I=1,2*(N+NI)
      DO 40 J=1,2*(N+NI)
      XM(I,2*M-1)=XM(I,2*M-1)-H(I,J)*PSI(J,1)
      XM(I,2*M)=XM(I,2*M)-H(I,J)*PSI(J,2)
40    CONTINUE
      DO 50 J=1,3*N
      XM(I,2*M-1)=XM(I,2*M-1)+G(I,J)*ETA(J,1)
      XM(I,2*M)=XM(I,2*M)+G(I,J)*ETA(J,2)
50    CONTINUE
60    CONTINUE
70    CONTINUE
C
C  MULTIPLY BY THE E MATRIX AND THE MASS DENSITY RO
C
      DO 100 I=1,2*(N+NI)
      DO 110 L=1,2*(N+NI)
110   V(L)=0.
      DO 90 J=1,2*(N+NI)
      DO 80 K=1,2*(N+NI)
      V(J)=V(J)+XM(I,K)*E(K,J)
80    CONTINUE
90    CONTINUE
      DO 95 J=1,2*(N+NI)
      XM(I,J)=V(J)*RO
95    CONTINUE
100   CONTINUE
C
      RETURN
      END
C-----------------------------------------------------------
      SUBROUTINE MATETA13(X,Y,CX,CY,ETA,M,NX1)
C
C  THIS SUBROUTINE COMPUTES TWO COLUMNS OF THE ETA MATRIX
C  CORRESPONDING TO THE M DOMAIN INTERPOLATION FUNCTION
C
      DIMENSION X(1),Y(1),CX(1),CY(1),ETA(NX1,2)
      DIMENSION XNOR(3),YNOR(3)
      COMMON N,NI,RO,GE,XNU,INP,IPR,NSTEP,XT,C
C
C  COMPUTE BASIC CONSTANTS
C
      A=((1-2*XNU)/(5-4*XNU))*C
      B=1/(30*(1-XNU))
      D=(10*XNU-9)/(90*(1-XNU))
      FZ=(2*XNU)/(1-2*XNU)
C
C  COMPUTE FOR EACH ELEMENT
C
      DO 10 L=1,N-1,2
C
C  NORMAL COMPONENTS FOR THE THREE NODES OF THE ELEMENT
C
      IR=L+2
      IF(IR.EQ.(N+1)) IR=1
      X1=X(L)
      Y1=Y(L)
      X2=X(L+1)
      Y2=Y(L+1)
      X3=X(IR)
      Y3=Y(IR)
      AN=X3+X1-2*X2
      BN=(X3-X1)/2.
      CN=Y3+Y1-2*Y2
      DN=(Y3-Y1)/2.
      DO 5 I=1,3
      GI=I-2
      XJA=SQRT((GI*AN+BN)**2+(GI*CN+DN)**2)
      XNOR(I)=(GI*CN+DN)/XJA
      YNOR(I)=-(GI*AN+BN)/XJA
5     CONTINUE
C
C  ETA ELEMENTS CORRESPONDING TO THE THREE NODES OF THE ELEMENT L
C
      DO 20 I=1,3
      LI=L+I-1
      IF(LI.EQ.N+1) LI=1
      IF(M.LE.N) THEN
      XM=X(M)
      YM=Y(M)
      ELSE
      XM=CX(M-N)
      YM=CY(M-N)
      ENDIF
      R=SQRT((X(LI)-XM)**2+(Y(LI)-YM)**2)
      IF(M.EQ.LI) GO TO 30
      RD1=(X(LI)-XM)/R
      RD2=(Y(LI)-YM)/R
      C1=3*A*R+4*B*R*R+3*D*R*R
      C2=A*R+B*R*R
      C3=A*R+B*R*R+3*D*R*R
      S111=FZ*C1*RD1+2*C2*RD1+2*C3*RD1+2*B*R*R*RD1*RD1*RD1
      S112=C3*RD2+2*B*R*R*RD1*RD1*RD2
      S211=S112
      S212=FZ*C1*RD1+2*C2*RD1+2*B*R*R*RD1*RD2*RD2
      S121=FZ*C1*RD2+2*C2*RD2+2*B*R*R*RD1*RD1*RD2
      S122=C3*RD1+2*B*R*R*RD1*RD2*RD2
      S221=S122
      S222=FZ*C1*RD2+2*C2*RD2+2*C3*RD2+2*B*R*R*RD2*RD2*RD2
C
C  COMPUTE ETA ELEMENTS
C
      T11=S111*XNOR(I)+S112*YNOR(I)
      T21=S211*XNOR(I)+S212*YNOR(I)
      T12=S121*XNOR(I)+S122*YNOR(I)
      T22=S221*XNOR(I)+S222*YNOR(I)
      GO TO 40
30    T11=0.
      T21=0.
      T12=0.
      T22=0.
40    ETA(3*L+2*I-4,1)=T11
      ETA(3*L+2*I-3,1)=T21
      ETA(3*L+2*I-4,2)=T12
      ETA(3*L+2*I-3,2)=T22
20    CONTINUE
10    CONTINUE
C
      RETURN
      END
C--------------------------------------------------------------
      SUBROUTINE MATPSI13(X,Y,CX,CY,PSI,M,NX)
C
C  THIS SUBROUTINE COMPUTES TWO COLUMNS OF THE PSI MATRIX
C  CORRESPONDING TO THE M DOMAIN INTERPOLATION FUNCTION
C
      DIMENSION X(1),Y(1),CX(1),CY(1),PSI(NX,2)
      COMMON N,NI,RO,GE,XNU,INP,IPR,NSTEP,XT,C
C
C  COMPUTE BASIC CONSTANTS
C
      A=((1-2*XNU)/(5-4*XNU))*C/GE
      B=1/(30*(1-XNU)*GE)
      D=(10*XNU-9)/(90*(1-XNU))/GE
C
      IF(M.LE.N) THEN
      XM=X(M)
      YM=Y(M)
      ELSE
      XM=CX(M-N)
      YM=CY(M-N)
      ENDIF
C
C  COMPUTE PSI ELEMENTS CORRESPONDING TO THE N NODES AND
C  THE NI INTERNAL POINTS
C
      DO 10 L=1,N+NI
      IF(L.LE.N) THEN
      XL=X(L)
      YL=Y(L)
      ELSE
      XL=CX(L-N)
      YL=CY(L-N)
      ENDIF
      RA=SQRT((XL-XM)**2+(YL-YM)**2)
      IF(M.EQ.L) GO TO 20
      RD1=(XL-XM)/RA
      RD2=(YL-YM)/RA
      U11=((A+B*RA)*RD1**2+D*RA)*RA**2
      U22=((A+B*RA)*RD2**2+D*RA)*RA**2
      U12=(A+B*RA)*RD1*RD2*RA**2
      GO TO 30
20    U11=0.
      U22=0.
      U12=0.
30    PSI(2*L-1,1)=U11
      PSI(2*L,2)=U22
      PSI(2*L-1,2)=U12
      PSI(2*L,1)=U12
10    CONTINUE
C
      RETURN
      END
C------------------------------------------------------------------------
      SUBROUTINE PSOLVE13(U,G,H,XM,KODE,AUX,IAUX,NX,NX1)
C
C  THIS ROUTINE PREPARES THE SYSTEM OF EQUATIONS FOR THE TIME
C  MARCHING PROCESS
C
      DIMENSION G(NX,NX1),H(NX,NX),XM(NX,NX),KODE(1)
      DIMENSION U(NX,3),AUX(1),IAUX(1)
      COMMON  N,NI,RO,GE,XNU,INP,IPR,NSTEP,XT,C
C
C  COMPUTE INTEGRATION CONSTANTS
C
      A0=2./(XT**2)
C
C  SET INITIAL CONDITIONS TO ZERO
C
      NN=2*(N+NI)
      DO 1 I=1,NN
      U(I,1)=0.
      U(I,2)=0.
      U(I,3)=0.
1     CONTINUE
C
C  COMPUTE THE MODIFIED H MATRIX
C
      DO 5 I=1,2*(N+NI)
      DO 10 J=1,2*(N+NI)
      H(I,J)=A0*XM(I,J)+H(I,J)
10    CONTINUE
5     CONTINUE
C
C  REORDER THE COLUMNS OF THE SYSTEM OF EQUATIONS IN ACCORDANCE
C  WITH THE BOUNDARY CONDITIONS AND FORM THE SYSTEM MATRIX A WHICH
C  IS STORED IN H
C
      NE=N/2
      DO 180 I=1,NE
      DO 170 J=1,6
      IF(KODE(6*I-6+J)) 110,110,170
  110 IF((I-NE).NE.0 .OR. J.LT.5) GO TO 125
      IF(KODE(J-4)) 115,115,113
  113 DO 114 K=1,NN
      CH=H(K,J-4)
      H(K,J-4)=-G(K,6*I-6+J)*GE
  114 G(K,6*I-6+J)=-CH
      GO TO 170
  115 DO 116 K=1,NN
      H(K,J-4)=H(K,J-4)-G(K,6*I-6+J)*GE
  116 G(K,6*I-6+J)=0.
      GO TO 170
  125 IF(I.EQ.1 .OR. J.GT.2 .OR. KODE(6*I-8+J).EQ.1) GO TO 130
      DO 129 K=1,NN
      H(K,4*I-4+J)=H(K,4*I-4+J)-G(K,6*I-6+J)*GE
  129 G(K,6*I-6+J)=0.
      GO TO 170
  130 DO 132 K=1,NN
      CH=H(K,4*I-4+J)
      H(K,4*I-4+J)=-G(K,6*I-6+J)*GE
  132 G(K,6*I-6+J)=-CH
  170 CONTINUE
  180 CONTINUE
C
C  INVERT THE SYSTEM MATRIX A
C
      CALL INVERT(H,IAUX,AUX,NN,NX)
C
      RETURN
      END
C------------------------------------------------------------------
      SUBROUTINE SOLVER13(U,T,G,H,XM,KODE,BC,IFUNC,FSCALE,FUNC,
     1SOL,TI,NX,NX1,ISTEP,NFMAX,NPFMAX)
C
C  THIS ROUTINE COMPUTES THE VECTOR OF UNKNOWNS FOR EACH NEW STEP
C
      DIMENSION G(NX,NX1),H(NX,NX),XM(NX,NX),KODE(1)
      DIMENSION BC(1),IFUNC(1),FSCALE(1),FUNC(NFMAX,NPFMAX,2)
      DIMENSION SOL(1),U(NX,3),T(1),TI(1)
      COMMON  N,NI,RO,GE,XNU,INP,IPR,NSTEP,XT,C
C
      NN=2*(N+NI)
      NE=N/2
C
C  COMPUTE CONSTANTS
C
      A0=2./(XT**2)
      A2=5./(XT**2)
      A4=-2.*A0
      A6=A0/2.
C
C  SET SOLUTION VECTOR TO ZERO
C
      DO 10 I=1,NN
      SOL(I)=0.
10    CONTINUE
C
C  COMPUTE THE RIGHT HAND SIDE VECTOR
C
      DO 400 I=1,NN
      TI(I)=0.
      DO 500 J=1,6*NE
      NF=IFUNC(J)
      TI(I)=TI(I)+G(I,J)*FUNCVAL(NF,ISTEP*XT,BC(J)*FSCALE(NF),NPFMAX,
     1FUNC,NFMAX,NPFMAX)
500   CONTINUE
400   CONTINUE
C
      IF(ISTEP.EQ.1) GO TO 120
C
      DO 100 I=1,NN
      DO 110 J=1,NN
      TI(I)=TI(I)+XM(I,J)*(A2*U(J,3)+A4*U(J,2)+A6*U(J,1))
110   CONTINUE
100   CONTINUE
      DO 200 I=1,NN
      U(I,1)=U(I,2)
      U(I,2)=U(I,3)
200   CONTINUE
C
C  COMPUTE THE VECTOR OF UNKNOWNS
C
120    DO 125 I=1,NN
       DO 125 J=1,NN
       SOL(I)=SOL(I)+H(I,J)*TI(J)
125    CONTINUE
C
C  SET PRESENT BOUNDARY DISPLACEMENTS AND TRACTIONS IN
C  U(IUR,3) AND T(ITR) RESPECTIVELY
C
      DO 180 I=1,NE
      DO 170 J=1,6
      IUR=4*(I-1)+J
      ITR=6*(I-1)+J
      NF=IFUNC(ITR)
      IF((I.EQ.NE).AND.(J.GT.4)) IUR=J-4
      IF(KODE(ITR)) 130,130,140
130   U(IUR,3)=FUNCVAL(NF,ISTEP*XT,BC(ITR)*FSCALE(NF),NPFMAX,
     1FUNC,NFMAX,NPFMAX)
      T(ITR)=GE*SOL(IUR)
      GO TO 170
140   T(ITR)=FUNCVAL(NF,ISTEP*XT,BC(ITR)*FSCALE(NF),NPFMAX,
     1FUNC,NFMAX,NPFMAX)
      IF(J.GT.4) GO TO 170
      IF((J-1)/2) 150,150,160
150   IF(I.EQ.1) ITR=6*NE+J
      IF(KODE(ITR-2).NE.0) U(IUR,3)=SOL(IUR)
      GO TO 170
160   U(IUR,3)=SOL(IUR)
170   CONTINUE
180   CONTINUE
C
C  SET PRESENT INTERNAL POINTS DISPLACEMENTS
C
      DO I=2*N+1,NN
        U(I,3)=SOL(I)
      ENDDO
C
      RETURN
      END
C------------------------------------------------------------------
      SUBROUTINE INVERT(A,MM,B,NN,NX)
C
C  THIS SUBROUTINE INVERTS A REAL MATRIX USING THE GAUSS-JORDAN METHOD.
C  'A' IS A MATRIX (NN*NN)  (NN.LE.NX)
C  'B' AND 'MM' ARE AUXILIAR VECTORS
C
      DIMENSION MM(1)
      DIMENSION A(NX,NX),B(1)
C
      IF(NN-1) 300,350,100
100   DO 110 I=1,NN
110   MM(I)=-I
      DO 200 I=1,NN
C
C  LOCATE THE BIGGEST ELEMENT
C
      X=0.D0
      DO 130 L=1,NN
      IF(MM(L).GT.0) GO TO 130
      DO 120 K=1,NN
      IF(MM(K).GT.0) GO TO 120
      D=A(L,K)
      Y=ABS(D)
      IF(X.GT.Y) GO TO 120
      LD=L
      KD=K
      X=Y
120   CONTINUE
130   CONTINUE
      D=A(LD,KD)
C
C  ROWS EXCHANGE
C
      L=-MM(LD)
      MM(LD)=MM(KD)
      MM(KD)=L
      DO 140 J=1,NN
      B(J)=A(LD,J)
      A(LD,J)=A(KD,J)
140   A(KD,J)=B(J)
C
C  DIVIDE COLUMN BY THE BIGGEST ELEMENT
C
      DO 150 K=1,NN
150   A(K,KD)=A(K,KD)/D
C
C  REDUCE ROWS AND COLUMNS REMAINING
C
      DO 170 J=1,NN
      IF(J.EQ.KD) GO TO 170
      DO 160 K=1,NN
160   A(K,J)=A(K,J)-B(J)*A(K,KD)
170   CONTINUE
C
C  REDUCE ROW
C
      B(KD)=-1.D0
      DO 180 K=1,NN
180   A(KD,K)=-B(K)/D
200   CONTINUE
C
C  COLUMNS EXCHANGE
C
      DO 240 I=1,NN
      L=0
220   L=L+1
      IF(MM(L).NE.I) GO TO 220
      MM(L)=MM(I)
      MM(I)=I
      DO 240 K=1,NN
      TEMP=A(K,L)
      A(K,L)=A(K,I)
240   A(K,I)=TEMP
300   RETURN
350   A(1,1)=1.D0/A(1,1)
      GO TO 300
      END
C-----------------------------------------------------------------------------
      SUBROUTINE OUTPUT13(X,Y,U,T,ISTEP,NX)
C
C  THIS SUBROUTINE PRINTS THE VALUES OF THE DISPLACEMENTS
C  AND TRACTIONS AT BOUNDARY NODES.
C
      DIMENSION X(1),Y(1),U(NX,3),T(1)
      COMMON N,NI,RO,GE,XNU,INP,IPR,NSTEP,XT,C
C
      NN=2*N
      NE=N/2
      TIME=ISTEP*XT
      WRITE(IPR,100) TIME
      DO 60 I=1,N
   60 WRITE(IPR,200) X(I),Y(I),U(2*I-1,3),U(2*I,3)
      WRITE(IPR,300)
      WRITE(IPR,400) X(1),Y(1),T(6*NE-1),T(6*NE),
     1               T(1),T(2)
      WRITE(IPR,400) X(2),Y(2),T(3),T(4),
     1               T(3),T(4)
      DO 70 I=2,NE
      WRITE(IPR,400) X(2*I-1),Y(2*I-1),T(6*I-7),T(6*I-6),
     1               T(6*I-5),T(6*I-4)
   70 WRITE(IPR,400) X(2*I),  Y(2*I),  T(6*I-3),T(6*I-2),
     1               T(6*I-3),T(6*I-2)
C
  100 FORMAT(' ',79('*')//' RESULTS AT TIME = ',E12.5//
     1       '  BOUNDARY NODES'//10X,'X',17X,'Y',11X,'DISPLACEMENT X',
     2       4X,'DISPLACEMENT Y'/)
  200 FORMAT(4(4X,E14.7))
  300 FORMAT(///28X,'TRACTION X',3X,'TRACTION Y',3X,'TRACTION X',3X,
     1       'TRACTION Y'/7X,'X',12X,'Y',7X,'BEFORE NODE',2X,
     2       'BEFORE NODE',2X,'AFTER NODE',3X,'AFTER NODE'/)
  400 FORMAT(6(1X,E12.5))
      RETURN
      END
