C------------------------------------------------------------------
C
      PROGRAM QUADTHEH
C
C  THIS IS A B.E. PROGRAM THAT USES (QUA)DRATIC ELEMENTS FOR THE
C  SOLUTION OF (D)YNAMIC (TH)REE DIMENSIONAL (E)LASTIC OR
C  VISCOELASTIC TIME (H)ARMONIC PROBLEMS
C
C  PROGRAM NO. 8
C
C  SET MAXIMUM DIMENSIONS
C  NX=MAXIMUM DIMENSION OF THE SYSTEM OF EQUATIONS
C  NX/3= MAXIMUM NUMBER OF NODES
C  NNE=MAXIMUM NUMBER OF ELEMENTS
C  NPM=MAXIMUM NUMBER OF INTERNAL POINTS
C  NFM=MAXIMUM NUMBER OF FREQUENCIES
C
      PARAMETER(NX=100,NNE=25,NPM=20,NFM=10)
      CHARACTER*12 FILEIN,FILEOUT
C
      COMPLEX  A(NX,NX),FI(NX),BC(27,NNE),DFI(27,NNE),DSOL(3*NPM)
      DIMENSION X(NX/3),Y(NX/3),Z(NX/3),CX(NPM),CY(NPM),CZ(NPM)
      DIMENSION KODE(3*NNE),AFR(NFM),NESIM(NX/3),KE(5,NX/3)
      DIMENSION NCONEC(NNE),KCONEC(9,NNE),DELTA(3,3)
      COMPLEX CP,CS,GE,C1,C2,C3
      COMMON/MATA/ A
      COMMON/VECF/ FI
      COMMON N,NE,L,CP,CS,GE,RO,XNU,FR,INP,IPR
      COMMON C1,C2,C3,C4,C5,C6,DELTA
C
C  ASSIGN NUMBERS FOR INPUT AND OUTPUT FILES
C
      INP=5
      IPR=6
C
C  READ NAMES AND OPEN  FILES FOR INPUT AND OUTPUT
C
      WRITE(*,' (A) ') ' NAME OF INPUT FILE (MAX. 12 CHART.)'
      READ(*,'  (A) ')FILEIN
      OPEN(INP,FILE=FILEIN,STATUS='OLD')
      WRITE(*,' (A) ') ' NAME OF OUTPUT FILE (MAX. 12 CHART.)'
      READ(*,' (A) ')FILEOUT
      OPEN(IPR,FILE=FILEOUT,STATUS='NEW')
C
C  READ DATA
C
      CALL INPUT8(CX,CY,CZ,X,Y,Z,KODE,BC,NFR,AFR,NESIM,KE,NCONEC,
     1KCONEC)
      DO 60 I=1,NFR
      FR=AFR(I)
C
C  FORM SYSTEM OF EQUATIONS
C
      CALL GHMAT8(X,Y,Z,A,FI,BC,KODE,NCONEC,KCONEC,NESIM,KE,NX)
C
C  SOLVE SYSTEM OF EQUATIONS
C
      NN=3*N
      CALL CSOLVER(A,FI,NN,NX)
C
C  COMPUTE DISPLACEMENT VALUES AT INTERNAL POINTS.
C
      CALL INTER8(FI,DFI,KODE,BC,CX,CY,CZ,X,Y,Z,DSOL,NCONEC,KCONEC,
     1NESIM,KE)
C
C  PRINT RESULTS AT BOUNDARY NODES AND INTERNAL POINTS
C
      CALL OUTPUT8(FI,DFI,DSOL,NCONEC,KCONEC)
   60 CONTINUE
C
C   CLOSE INPUT AND OUTPUT FILES
C
      CLOSE (INP)
      CLOSE (IPR)
      STOP
      END
C-----------------------------------------------------------------------
      SUBROUTINE INPUT8(CX,CY,CZ,X,Y,Z,KODE,BC,NFR,AFR,NESIM,KE,
     1NCONEC,KCONEC)
C
C  NE= NUMBER OF BOUNDARY ELEMENTS
C  L = NUMBER OF INTERNAL POINTS
C  CP= P-WAVE VELOCITY
C  CS= S-WAVE VELOCITY
C  RO= DENSITY
C  NFR= NUMBER OF FREQUENCIES TO BE ANALYSED
C  AFR= ARRAY FOR STORAGE OF FREQUENCIES
C  FR= FREQUENCY
C  GE= SHEAR MODULUS
C  XNU= POISSON RATIO
C  NCONEC= ARRAY FOR STORAGE OF NUMBER OF NODES OF THE ELEMENT (6 OR 9)
C  KCONEC= MATRIX FOR STORAGE OF ELEMENT CONECTIVITIES
C
      CHARACTER*80 TITLE
      COMPLEX BC(27,1),CP,CS,GE,C1,C2,C3
      COMMON N,NE,L,CP,CS,GE,RO,XNU,FR,INP,IPR
      COMMON C1,C2,C3,C4,C5,C6,DELTA
      DIMENSION KODE(1),X(1),Y(1),Z(1),CX(1),CY(1),CZ(1),AFR(1)
      DIMENSION NESIM(1),KE(5,1),NCONEC(1),KCONEC(9,1),DELTA(3,3)
      WRITE(IPR,100)
  100 FORMAT(' ',79('*'))
C
C  READ JOB TITLE
C
      READ(INP,'(A)') TITLE
      WRITE(IPR,'(A)') TITLE
C
C  READ NUMBER OF  BOUNDARY NODES, ELEMENTS, INTERNAL POINTS AND
C  FREQUENCIES; READ MATERIAL PROPERTIES
C
      READ(INP,*) N,NE,L,NFR,RGE,DAM,RO,XNU
      GE=CMPLX(RGE,RGE*2.*DAM)
      CS=CSQRT(GE/RO)
      CP=CS*SQRT((2.-2.*XNU)/(1.-2.*XNU))
      WRITE(IPR,300)N,NE,L,NFR,CP,CS,GE,DAM,RO,XNU
  300 FORMAT(//' DATA'//2X,'NUMBER OF BOUNDARY NODES    =',I3/2X,
     1'NUMBER OF BOUNDARY ELEMENTS =',I3/2X,'NUMBER OF INTERNAL POINTS
     2   =',I3/2X,'NUMBER OF FREQUENCIES       =',I3/2X,
     3'P-WAVE VELOCITY             =',G14.7,',',G14.7/2X,
     5'S-WAVE VELOCITY             =',G14.7,',',G14.7/2X,
     6'SHEAR MODULUS               =',G14.7,',',G14.7/2X,
     7'DAMPING RATIO               =',G14.7/2X,'DENSITY
     8      =',G14.7/2X,'POISSON RATIO               =',G14.7)
C
C  READ FREQUENCIES
C
      READ(INP,*) (AFR(I),I=1,NFR)
      WRITE(IPR,400) (AFR(I),I=1,NFR)
  400 FORMAT(/2X,'FREQUENCIES'/5(2X,G14.7))
C
C  READ BOUNDARY NODES COORDINATES
C
      WRITE(IPR,500)
      READ(INP,*) (IN,X(IN),Y(IN),Z(IN),I=1,N)
      DO 10 I=1,N
   10 WRITE(IPR,240) I,X(I),Y(I),Z(I)
  500 FORMAT(//2X,'BOUNDARY NODES COORDINATES'///4X,
     1'NODE',12X,'X',18X,'Y',18X,'Z'/)
  240 FORMAT(5X,I3,3(5X,E14.7))
C
C  READ NUMBER OF NODES AND CONNECTIVITIES FOR EACH ELEMENT
C
      WRITE(IPR,600)
      DO 15 I=1,NE
      READ(INP,*) IE,NCONEC(IE),(KCONEC(J,IE),J=1,NCONEC(IE))
   15 WRITE(IPR,340) IE,NCONEC(IE),(KCONEC(J,IE),J=1,NCONEC(IE))
  600 FORMAT(//2X,'ELEMENT CONECTIVITIES'///2X,'ELEMENT',1X,
     1'NO.CON.',24X,'NODES'/)
  340 FORMAT(4X,I3,4X,I2,2X,9(3X,I3))
C
C  READ BOUNDARY CONDITIONS IN BC() MATRIX;
C  IF KODE()=0 THE BC() VALUES ARE KNOWN DISPLACEMENTS;
C  IF KODE()=1 THE BC() VALUES ARE KNOWN TRACTIONS.
C  27 OR 18 BOUNDARY CONDITIONS ARE READ PER ELEMENT.
C  NODES BETWEEN TWO OR MORE ELEMENTS MAY HAVE TWO OR MORE DIFFERENT
C  VALUES OF THE TRACTION BUT ONLY ONE VALUE OF THE DISPLACEMENT.
C
      WRITE(IPR,800)
  800 FORMAT(//2X,'BOUNDARY CONDITIONS'//1X,
     1'ELEMEN',1X,'CODE',22X,'PRESCRIBED VALUES'/)
      DO 30 I=1,NE
      DO 35 K=1,3
   35 READ(INP,*) IE,KODE(3*IE-3+K),(BC(3*J-3+K,IE),J=1,NCONEC(IE))
      WRITE(IPR,950) IE,KODE(3*IE-2),(BC(3*J-2,IE),J=1,NCONEC(IE))
      WRITE(IPR,960) KODE(3*IE-1),(BC(3*J-1,IE),J=1,NCONEC(IE))
   30 WRITE(IPR,970) KODE(3*IE),(BC(3*J,IE),J=1,NCONEC(IE))
  950 FORMAT(1X,I3,4X,I2,1X,'X',3(1X,G10.3,',',G10.3)/
     1(12X,1X,G10.3,',',G10.3,1X,G10.3,',',G10.3,1X,G10.3,',',G10.3))
  960 FORMAT(8X,I2,1X,'Y',3(1X,G10.3,',',G10.3)/
     1(12X,1X,G10.3,',',G10.3,1X,G10.3,',',G10.3,1X,G10.3,',',G10.3))
  970 FORMAT(8X,I2,1X,'Z',3(1X,G10.3,',',G10.3)/
     1(12X,1X,G10.3,',',G10.3,1X,G10.3,',',G10.3,1X,G10.3,',',G10.3))
C
C  READ INTERNAL POINTS COORDINATES
C
      IF(L.EQ.0) GO TO 150
      READ(INP,*) (CX(I),CY(I),CZ(I),I=1,L)
      WRITE(IPR,1000)
 1000 FORMAT(//2X,'COORDINATES OF INTERNAL POINTS',
     1//4X,'POINT',11X,'X',18X,'Y',17X,' Z',/)
      DO 40 I=1,L
   40 WRITE(IPR,240) I,CX(I),CY(I),CZ(I)
C
C  DETERMINE THE NUMBER OF ELEMENTS TO WHICH EACH NODE BELONGS
C  AND DETERMINE THE ORDER NUMBERS OF THOSE ELEMENTS
C
  150 DO 45 I=1,N
   45 NESIM(I)=0
      DO 50 IE=1,NE
      DO 50 J=1,NCONEC(IE)
      IN=KCONEC(J,IE)
      NESIM(IN)=NESIM(IN)+1
   50 KE(NESIM(IN),IN)=IE
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE GHMAT8(X,Y,Z,A,FI,BC,KODE,NCONEC,KCONEC,NESIM,KE,NX)
C
C  THIS SUBROUTINE COMPUTES THE A MATRIX AND THE RIGHT HAND SIDE VECTOR
C  FI AND FORMS THE SYSTEM A X = F READY TO BE SOLVED
C
      COMPLEX A(NX,1),FI(1),BC(27,1),HW(3,27),GW(3,27),HDIF(3,3)
      COMPLEX D2(3,3),CP,CS,GE,VP,C1,C2,C3
      DIMENSION X(1),Y(1),Z(1),KODE(1),NESIM(1),KE(5,1),NCONEC(1)
      DIMENSION KCONEC(9,1),HS(3,27),XJ1(9),XJ2(9),XJ3(9),XI(3)
      DIMENSION ETA1(36),ETA2(36),ETA3(36),XJA(36),D1(3,3)
      DIMENSION XG1(36),XG2(36),XG3(36),FF(9,36),DELTA(3,3)
      COMMON N,NE,L,CP,CS,GE,RO,XNU,FR,INP,IPR
      COMMON C1,C2,C3,C4,C5,C6,DELTA
      DATA PI/3.141592654/
C
      DO 20 I=1,3*N
      FI(I)=(0.,0.)
      DO 11 J=1,3*N
   11 A(J,I)=(0.,0.)
   20 CONTINUE
      HS11=0.
C
C  COMPUTE CONSTANS AND DELTA FUNCTION
C
      C1=(CP/CS)**2
      C2=(CS/CP)**2
      C3=1./(4.*PI*RO*CS**2)
      C4=1./(4.*PI)
      C5=-1./(8.*PI*(1.-XNU))
      C6=1.-2.*XNU
C
      DO 25 II=1,3
      DO 25 JJ=1,3
      DELTA(II,JJ)=0.
      IF(II.EQ.JJ) DELTA(II,JJ)=1.
   25 CONTINUE
C
C  COMPUTE THE GW AND HW MATRICES FOR EACH BOUNDARY ELEMENT AND EACH
C  COLLOCATION POINT
C
      DO 500 IEJ=1,NE
C
C  STORE THE COORDINATES OF THE SIX OR NINE NODES OF THE IEJ ELEMENT
C  IN XJ1,XJ2,XJ3
C
      DO 30 M=1,NCONEC(IEJ)
      K=KCONEC(M,IEJ)
      XJ1(M)=X(K)
      XJ2(M)=Y(K)
   30 XJ3(M)=Z(K)
C
C  COMPUTE  THE ARRAYS OF CARTESIAN COORDINATES XG1,XG2,XG3, SHAPE
C  FUNCTIONS FF, JACOBIANS XJA AND COMPONENTS OF THE OUTWARD NORMAL
C  ETA CORRESPONDING TO THE GAUSS POINTS FOR EACH ELEMENT
C
      CALL ETAXJA(XJ1,XJ2,XJ3,ETA1,ETA2,ETA3,XJA,XG1,XG2,XG3,FF,
     1NCONEC(IEJ))
      DO 400 IN=1,N
      XI(1)=X(IN)
      XI(2)=Y(IN)
      XI(3)=Z(IN)
      NODO=0
      DO 60 I=1,NCONEC(IEJ)
      IF (IN.EQ.KCONEC(I,IEJ)) NODO=I
   60 CONTINUE
      IF(NODO) 55,55,65
   65 CALL LOCIN8(XJ1,XJ2,XJ3,HW,GW,HS,HDIF,NODO,NCONEC(IEJ))
      GO TO 120
   55 CALL EXTIN8(ETA1,ETA2,ETA3,XJA,XG1,XG2,XG3,FF,XI,HW,GW,HS,
     1NCONEC(IEJ))
C
C  COMPUTE CONTRIBUTION TO THE DIAGONAL COEFFICIENTS OF
C  MATRIX H (D1 OR D2)
C
  120 DO 70 J=1,3
      DO 70 I=1,3
      D1(I,J)=0.
   70 D2(I,J)=(0.,0.)
      IF(NODO) 75,75,76
C
C  THE COLLOCATION NODE DOES NOT BELONG TO THE INTEGRATION ELEMENT
C
   75 DO 95 I=1,3
      DO 95 J=1,3
      DO 90 K=1,NCONEC(IEJ)
      IND=3*K-3+J
   90 D1(I,J)=D1(I,J)-HS(I,IND)
   95 CONTINUE
      IF (IN.EQ.1) HS11=HS11+D1(1,1)
      GO TO 130
C
C  THE COLLOCATION NODE BELONGS TO THE INTEGRATION ELEMENT
C
   76 DO 100 I=1,3
      DO 100 J=1,3
      DO 105 K=1,NCONEC(IEJ)
      IF(K-NODO) 77,78,77
   77 IND=3*K-3+J
      D2(I,J)=D2(I,J)-HS(I,IND)
      GO TO 105
   78 D2(I,J)=HDIF(I,J)+D2(I,J)
  105 CONTINUE
  100 CONTINUE
      IF(IN-1) 130,135,130
  135 DO 140 K=1,NCONEC(IEJ)
      IF(K-NODO) 145,140,145
  145 HS11=HS11-HS(1,3*K-2)
  140 CONTINUE
C
C  FORM SYSTEM OF EQUATIONS USING THE KODE() OF EACH COMPONENT
C
  130 DO 150 LD=1,3
      IF (KODE(3*IEJ-3+LD)) 200,210,220
  200 WRITE(IPR,1000) IEJ
 1000 FORMAT(72X,'NEGATIVE KODE OF THE ELEMENT ',I3)
      STOP
C
C  DISPLACEMENT IS PRESCRIBED ON COMPONENT LD OF ELEMENT IEJ.
C
  210 DO 250 K=1,NCONEC(IEJ)
      IK=KCONEC(K,IEJ)
      A(3*IN-2,3*IK-3+LD)=A(3*IN-2,3*IK-3+LD)-GW(1,3*K-3+LD)*GE
      A(3*IN-1,3*IK-3+LD)=A(3*IN-1,3*IK-3+LD)-GW(2,3*K-3+LD)*GE
      A(3*IN  ,3*IK-3+LD)=A(3*IN  ,3*IK-3+LD)-GW(3,3*K-3+LD)*GE
      IF (IN-IK) 260,265,260
  260 FI(3*IN-2)=FI(3*IN-2)-HW(1,3*K-3+LD)*BC(3*K-3+LD,IEJ)
      FI(3*IN-1)=FI(3*IN-1)-HW(2,3*K-3+LD)*BC(3*K-3+LD,IEJ)
      FI(3*IN)  =FI(3*IN)  -HW(3,3*K-3+LD)*BC(3*K-3+LD,IEJ)
      GO TO 250
  265 FI(3*IN-2)=FI(3*IN-2)-D2(1,LD)*BC(3*K-3+LD,IEJ)
      FI(3*IN-1)=FI(3*IN-1)-D2(2,LD)*BC(3*K-3+LD,IEJ)
      FI(3*IN)  =FI(3*IN)  -D2(3,LD)*BC(3*K-3+LD,IEJ)
  250 CONTINUE
      GO TO 150
C
C  TRACTION IS PRESCRIBED ON COMPONENT LD OF ELEMENT IEJ
C
  220 DO 270 K=1,NCONEC(IEJ)
      IK=KCONEC(K,IEJ)
      KODE0=0
C
C  CHECK FOR ALL THE ELEMENTS TO WHICH NODE IK BELONGS
C
      DO 280 I=1,NESIM(IK)
      IF (KODE(3*KE(I,IK)-3+LD)) 280,295,280
  295 DO 300 M=1,NCONEC(KE(I,IK))
C
C  STORE IN VP THE PRESCRIBED DISPLACEMENT FOR THE NODE IK AS PART
C  OF ANOTHER ELEMENT TO BE USED ALSO IN ELEMENT IEJ
C
      IF(IK-KCONEC(M,KE(I,IK))) 300,297,300
  297 VP=BC(3*M-3+LD,KE(I,IK))
      KODE0=1
      GO TO 22
  300 CONTINUE
  280 CONTINUE
   22 IF (KODE0) 310,315,310
C
C  ALL ELEMENTS TO WHICH IK BELONGS HAVE PRESCRIBED TRACTION IN THE
C  LD DIRECTION. THE DISPLACEMENT IS THE UNKNOWN
C
  315 FI(3*IN-2)=FI(3*IN-2)+GW(1,3*K-3+LD)*BC(3*K-3+LD,IEJ)
      FI(3*IN-1)=FI(3*IN-1)+GW(2,3*K-3+LD)*BC(3*K-3+LD,IEJ)
      FI(3*IN)  =FI(3*IN)  +GW(3,3*K-3+LD)*BC(3*K-3+LD,IEJ)
      IF (IN-IK) 320,325,320
  320 A(3*IN-2,3*IK-3+LD)=A(3*IN-2,3*IK-3+LD)+HW(1,3*K-3+LD)
      A(3*IN-1,3*IK-3+LD)=A(3*IN-1,3*IK-3+LD)+HW(2,3*K-3+LD)
      A(3*IN  ,3*IK-3+LD)=A(3*IN  ,3*IK-3+LD)+HW(3,3*K-3+LD)
      GO TO 270
  325 A(3*IN-2,3*IN-3+LD)=A(3*IN-2,3*IN-3+LD)+D2(1,LD)
      A(3*IN-1,3*IN-3+LD)=A(3*IN-1,3*IN-3+LD)+D2(2,LD)
      A(3*IN  ,3*IN-3+LD)=A(3*IN  ,3*IN-3+LD)+D2(3,LD)
      GO TO 270
C
C  THE TRACTION IS ALSO PRESCRIBED FOR NODE IK IN THE LD DIRECTION.
C  THE UNKNOWN IS ANOTHER TRACTION
C
  310 IF (IN-IK) 330,335,330
  330 FI(3*IN-2)=FI(3*IN-2)+GW(1,3*K-3+LD)*BC(3*K-3+LD,IEJ)-
     1HW(1,3*K-3+LD)*VP
      FI(3*IN-1)=FI(3*IN-1)+GW(2,3*K-3+LD)*BC(3*K-3+LD,IEJ)-
     1HW(2,3*K-3+LD)*VP
      FI(3*IN)  =FI(3*IN)  +GW(3,3*K-3+LD)*BC(3*K-3+LD,IEJ)-
     1HW(3,3*K-3+LD)*VP
      GO TO 270
  335 FI(3*IN-2)=FI(3*IN-2)+GW(1,3*K-3+LD)*BC(3*K-3+LD,IEJ)-
     1D2(1,LD)*VP
      FI(3*IN-1)=FI(3*IN-1)+GW(2,3*K-3+LD)*BC(3*K-3+LD,IEJ)-
     1D2(2,LD)*VP
      FI(3*IN)  =FI(3*IN)  +GW(3,3*K-3+LD)*BC(3*K-3+LD,IEJ)-
     1D2(3,LD)*VP
  270 CONTINUE
  150 CONTINUE
C
C  IF THE COLLOCATION POINT IN DOES NOT BELONG TO THE INTEGRATION
C  ELEMENT ADD D1 TO THE DIAGONAL COEFFICIENTS OF MATRIX H OR
C  -D1*BC TO THE RIGHT HAND SIDE VECTOR
C
      IF (NODO) 400,340,400
  340 DO 160 LD=1,3
      KODE0=0
      DO 380 I=1,NESIM(IN)
      IF (KODE(3*KE(I,IN)-3+LD)) 380,385,380
  385 DO 390 M=1,NCONEC(KE(I,IN))
      IF(IN-KCONEC(M,KE(I,IN))) 390,395,390
  395 VP=BC(3*M-3+LD,KE(I,IN))
      KODE0=1
      GO TO 33
  390 CONTINUE
  380 CONTINUE
   33 IF (KODE0) 396,399,396
  399 A(3*IN-2,3*IN-3+LD)=A(3*IN-2,3*IN-3+LD)+D1(1,LD)
      A(3*IN-1,3*IN-3+LD)=A(3*IN-1,3*IN-3+LD)+D1(2,LD)
      A(3*IN  ,3*IN-3+LD)=A(3*IN  ,3*IN-3+LD)+D1(3,LD)
      GO TO 160
  396 FI(3*IN-2)=FI(3*IN-2)-D1(1,LD)*VP
      FI(3*IN-1)=FI(3*IN-1)-D1(2,LD)*VP
      FI(3*IN  )=FI(3*IN  )-D1(3,LD)*VP
  160 CONTINUE
  400 CONTINUE
  500 CONTINUE
C
C  ADD ONE TO THE DIAGONAL TERMS OF THE H MATRIX FOR EXTERNAL PROBLEMS
C
      IF(HS11) 510,520,520
  510 DO 600 IN=1,N
      DO 760 LD=1,3
      KODE0=0
      DO 780 I=1,NESIM(IN)
      IF (KODE(3*KE(I,IN)-3+LD)) 780,785,780
  785 DO 790 M=1,NCONEC(KE(I,IN))
      IF(IN-KCONEC(M,KE(I,IN))) 790,795,790
  795 VP=BC(3*M-3+LD,KE(I,IN))
      KODE0=1
      GO TO 535
  790 CONTINUE
  780 CONTINUE
  535 IF (KODE0) 796,799,796
  799 A(3*IN-3+LD,3*IN-3+LD)=A(3*IN-3+LD,3*IN-3+LD)+1.
      GO TO 760
  796 FI(3*IN-3+LD)=FI(3*IN-3+LD)-VP
  760 CONTINUE
  600 CONTINUE
  520 CONTINUE
      RETURN
      END
C---------------------------------------------------------------------
      SUBROUTINE ETAXJA(XJ1,XJ2,XJ3,ETA1,ETA2,ETA3,XJA,XG1,XG2,XG3,
     1FF,NCONE)
C
C  THIS SUBROUTINE COMPUTES THE COMPONENTS OF THE NORMAL VECTORS,
C  COORDINATES,JACOBIANS AND SHAPE FUNCTIONS FOR THE GAUSS POINTS
C
      DIMENSION ETA1(1),ETA2(1),ETA3(1),XJA(1)
      DIMENSION XG1(1),XG2(1),XG3(1),FF(9,1)
      DIMENSION XJ1(1),XJ2(1),XJ3(1)
      DIMENSION FD1(9),FD2(9),DC(9,9),DCT(9,9)
      DIMENSION AUX(9),BUX(9),CUX(9)
      DIMENSION GI(6),RR(4),SS(4)
C
C  GI()      : GAUSS POINTS FOR QUADRILATERAL ELEMENTS
C  RR(),SS() : GAUSS POINTS FOR TRIANGULAR ELEMENTS
C
      DATA GI/0.238619186083197,-0.238619186083197,0.661209386466265,
     1        -0.661209386466265,0.932469514203152,-0.932469514203152/
      DATA RR/ .0694318422 , .3300094782 , .6699905218 , .9305681558 /
      DATA SS/ .0571041961 , .2768430136 , .5835904324 , .8602401357 /
C
      IF (9-NCONE) 1,1,2
    1 NPG=6
      GO TO 3
    2 NPG=4
    3 DO 10 I=1,NPG*NPG
      ETA1(I)=0.
      ETA2(I)=0.
      ETA3(I)=0.
   10 XJA(I)=0.
C
      DO  200 I=1,NPG
      DO  200 J=1,NPG
C
C  COMPUTE THE ORDER NUMBER OF THE GAUSS POINT
C
      K=NPG*I-NPG+J
C
      IF (9-NCONE) 100,100,110
C
C  COMPUTE VALUES OF THE SHAPE FUNCTIONS FOR QUADRILATERAL ELEMENTS
C
  100 G1=GI(I)
      G2=GI(J)
      FF(1,K)=0.25*G1*G2*(G1-1.)*(G2-1.)
      FF(2,K)=0.5*(1.-G1**2)*G2*(G2-1.)
      FF(3,K)=0.25*G1*G2*(G1+1.)*(G2-1.)
      FF(4,K)=0.5*G1*(G1+1.)*(1.-G2**2)
      FF(5,K)=0.25*G1*G2*(G1+1.)*(G2+1.)
      FF(6,K)=0.5*(1.-G1**2)*G2*(G2+1.)
      FF(7,K)=0.25*G1*G2*(G1-1.)*(G2+1.)
      FF(8,K)=0.5*G1*(G1-1.)*(1.-G2**2)
      FF(9,K)=(1.-G1**2)*(1.-G2**2)
C
C  COMPUTE SHAPE FUNCTIONS DERIVATIVES FOR QUADRILATERAL ELEMENTS
C
      FD1(1)=0.25*G2*(G2-1.)*(2.*G1-1.)
      FD1(2)=-G1*G2*(G2-1.)
      FD1(3)=0.25*G2*(G2-1.)*(2.*G1+1.)
      FD1(4)=0.5*(1.-G2**2)*(2.*G1+1.)
      FD1(5)=0.25*G2*(G2+1.)*(2.*G1+1.)
      FD1(6)=-G1*G2*(G2+1.)
      FD1(7)=0.25*G2*(G2+1.)*(2.*G1-1.)
      FD1(8)=0.5*(1.-G2**2)*(2.*G1-1.)
      FD1(9)=-2.*G1*(1.-G2**2)
C
      FD2(1)=0.25*G1*(G1-1.)*(2.*G2-1.)
      FD2(2)=0.5*(1.-G1**2)*(2.*G2-1.)
      FD2(3)=0.25*G1*(G1+1.)*(2.*G2-1.)
      FD2(4)=-G1*G2*(G1+1.)
      FD2(5)=0.25*G1*(G1+1.)*(2.*G2+1.)
      FD2(6)=0.5*(1.-G1**2)*(2.*G2+1.)
      FD2(7)=0.25*G1*(G1-1.)*(2.*G2+1.)
      FD2(8)=-G1*G2*(G1-1.)
      FD2(9)=-2.*G2*(1.-G1**2)
      GO TO 120
C
C  COMPUTE VALUES OF THE SHAPE FUNCTIONS FOR TRIANGULAR ELEMENTS
C
  110 G1=SS(J)
      G2=RR(I)*(1.-SS(J))
      G3=1.-G1-G2
      FF(1,K)=G1*(2.*G1-1.)
      FF(2,K)=G2*(2.*G2-1.)
      FF(3,K)=G3*(2.*G3-1.)
      FF(4,K)=4.*G1*G2
      FF(5,K)=4.*G2*G3
      FF(6,K)=4.*G3*G1
C
C  COMPUTE SHAPE FUNCTIONS DERIVATIVES FOR TRIANGULAR ELEMENTS
C
      FD1(1)=4.*G1-1.
      FD1(2)=0.
      FD1(3)=1.-4.*G3
      FD1(4)=4.*G2
      FD1(5)=-4.*G2
      FD1(6)=4.*(G3-G1)
C
      FD2(1)=0.
      FD2(2)=4.*G2-1.
      FD2(3)=1.-4.*G3
      FD2(4)=4.*G1
      FD2(5)=4.*(G3-G2)
      FD2(6)=-4.*G1
C
C
C  CARTESIAN CO-ORDINATES OF THE GAUSS POINT
C
  120 XG1(K)=0.
      XG2(K)=0.
      XG3(K)=0.
      DO 50 IK=1,NCONE
      XG1(K)=XG1(K)+FF(IK,K)*XJ1(IK)
      XG2(K)=XG2(K)+FF(IK,K)*XJ2(IK)
   50 XG3(K)=XG3(K)+FF(IK,K)*XJ3(IK)
C
C  JACOBIAN AND NORMAL VECTOR FOR THE GAUSS POINT
C
      DO 60 IN=1,NCONE
      DO 60 JN=1,NCONE
      DC(IN,JN)=FD1(IN)*FD2(JN)
   60 DCT(IN,JN)=FD2(IN)*FD1(JN)
      DO 70 M=1,NCONE
      AUX(M)=0.
      BUX(M)=0.
   70 CUX(M)=0.
      DO 80 JK=1,NCONE
      DO 80 IK=1,NCONE
      AUX(JK)=AUX(JK)+XJ2(IK)*(DC(IK,JK)-DCT(IK,JK))
      BUX(JK)=BUX(JK)+XJ3(IK)*(DC(IK,JK)-DCT(IK,JK))
   80 CUX(JK)=CUX(JK)+XJ1(IK)*(DC(IK,JK)-DCT(IK,JK))
      S1=0.
      S2=0.
      S3=0.
      DO 90 M=1,NCONE
      S1=S1+AUX(M)*XJ3(M)
      S2=S2+BUX(M)*XJ1(M)
   90 S3=S3+CUX(M)*XJ2(M)
      XJA(K)=SQRT(S1**2+S2**2+S3**2)
      ETA1(K)=S1/XJA(K)
      ETA2(K)=S2/XJA(K)
      ETA3(K)=S3/XJA(K)
C
  200 CONTINUE
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE EXTIN8(ETA1,ETA2,ETA3,XJA,XG1,XG2,XG3,FF,XI,HW,GW,HS,
     1NCONE)
C
C  THIS SUBROUTINE COMPUTES THE CONTRIBUTIONS TO THE H AND G MATRICES
C  OF AN ELEMENT WHICH DOES NOT CONTAIN THE COLLOCATION POINT
C
      COMPLEX HW(3,1),GW(3,1),CP,CS,GE,C1,C2,C3
      COMPLEX T(3,3),U(3,3),TDIF(3,3)
      DIMENSION ETA1(1),ETA2(1),ETA3(1),XJA(1),TS(3,3)
      DIMENSION XG1(1),XG2(1),XG3(1),FF(9,1)
      DIMENSION XI(3),HS(3,1)
      DIMENSION ETA(3),DELTA(3,3)
      DIMENSION OME(6),OMEI(4),OMEJ(4),P12(9)
      COMMON N,NE,L,CP,CS,GE,RO,XNU,FR,INP,IPR
      COMMON C1,C2,C3,C4,C5,C6,DELTA
C
C  OME()         : GAUSS WEIGHTS FOR QUADRILATERAL ELEMENTS
C  OMEI(),OMEJ() : GAUSS WEIGHTS FOR TRIANGULAR ELEMENTS
C
      DATA OME/0.467913934572691,0.467913934572691,0.360761573048139,
     1         0.360761573048139,0.171324492379170,0.171324492379170/
      DATA OMEI/.1739274226 , .3260725774 , .3260725774 , .1739274226/
      DATA OMEJ/.1355069134 , .2034645680 , .1298475476 , .0311809709/
C
      DO 5 IK=1,3*NCONE
      DO 5 JK=1,3
      HW(JK,IK)=(0.,0.)
      GW(JK,IK)=(0.,0.)
    5 HS(JK,IK)=0.
C
C  COMPUTE GW, HW AND HS
C
      IF (9-NCONE) 1,1,2
    1 NPG=6
      GO TO 3
    2 NPG=4
    3 DO 200 I=1,NPG
      DO 200 J=1,NPG
      K=NPG*I-NPG+J
      ETA(1)=ETA1(K)
      ETA(2)=ETA2(K)
      ETA(3)=ETA3(K)
C
      DO 30 IK=1,NCONE
      IF (9-NCONE) 40,40,50
   50 P12(IK)=FF(IK,K)*XJA(K)*OMEI(I)*OMEJ(J)
      GO TO 30
   40 P12(IK)=FF(IK,K)*XJA(K)*OME(I)*OME(J)
   30 CONTINUE
C
C  COMPUTE FUNDAMENTAL SOLUTION TENSORS
C
      CALL FUNTHEH(U,T,TS,XI(1),XI(2),XI(3),ETA,XG1(K),XG2(K),
     1XG3(K),0,TDIF)
C
C  SUMMATION OVER THE GAUSS POINTS TO COMPUTE GW,HW AND HS
C
      DO 60 KK=1,NCONE
      DO 60 II=1,3
      DO 60 JJ=1,3
      IND=3*KK+II-3
      GW(JJ,IND)=GW(JJ,IND)+U(JJ,II)*P12(KK)
      HW(JJ,IND)=HW(JJ,IND)+T(JJ,II)*P12(KK)
      HS(JJ,IND)=HS(JJ,IND)+TS(JJ,II)*P12(KK)
   60 CONTINUE
C
  200 CONTINUE
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE FUNTHEH(U,T,TS,XP,YP,ZP,ETA,XG,YG,ZG,ICODE,TDIF)
C
C  THIS SUBROUTINE COMPUTES THE FUNDAMENTAL SOLUTION TENSORS
C  AT EACH GAUSS POINT
C  U()    : DISPLACEMENT
C  T()    : TRACTION
C  TS()   : STATIC TRACTION
C  TDIF() : T()-TS()
C
C  ICODE .LE. 0   THE COLLOCATION POINT DOES NOT BELONG TO THE ELEMENT
C  ICODE .GT. 0   THE COLLOCATION POINT BELONGS TO THE ELEMENT
C
      COMPLEX   U(3,3),T(3,3),TDIF(3,3)
      COMPLEX   C7,ZP2,ZS2,AA,BB,CC,TA,TB,TC
      COMPLEX   WI,ZKP,ZKS,ZZP,ZZS,EZP,EZS,FHI,CAPPA,FHIDR,CAPPADR
      DIMENSION RD(3),ETA(1),TS(3,3)
      COMPLEX   CP,CS,GE,C1,C2,C3
      DIMENSION DELTA(3,3)
      COMMON    N,NP,L,CP,CS,GE,RO,XNU,FR,INP,IPR
      COMMON    C1,C2,C3,C4,C5,C6,DELTA
C
      DATA PI/3.141592654/
      DATA TOL/1.E-15/
C
      C7=CS/CP
C
      R1=XG-XP
      R2=YG-YP
      R3=ZG-ZP
      R=SQRT(R1**2+R2**2+R3**2)
      RDN=(R1*ETA(1)+R2*ETA(2)+R3*ETA(3))/R
      RD(1)=R1/R
      RD(2)=R2/R
      RD(3)=R3/R
C
      WI=CMPLX(0.,FR)
      ZKP=-WI/CP
      ZKS=-WI/CS
      ZZP=ZKP*R
      ZZS=ZKS*R
      EZP=CEXP(ZZP)
      EZS=CEXP(ZZS)
      ZP2=ZZP*ZZP
      ZS2=ZZS*ZZS
C
      FHI=(1.+1./ZS2-1./ZZS)*EZS/R-
     @    C2*(1./ZP2-1./ZZP)*EZP/R
      CAPPA=   (1.+3./ZS2-3./ZZS)*EZS/R-
     @      C2*(1.+3./ZP2-3./ZZP)*EZP/R
      FHIDR=(-2.+ZZS+3./ZZS-3./ZS2)*EZS/R**2-
     @       C2*(-1.+3./ZZP-3./ZP2)*EZP/R**2
      CAPPADR=   (ZZS-4.+9./ZZS-9./ZS2)*EZS/R**2-
     @        C2*(ZZP-4.+9./ZZP-9./ZP2)*EZP/R**2
C
C   COMPUTE TENSORS U(),T() AND TS()
C
      AA=FHIDR-CAPPA/R
      BB=4.*CAPPA/R-2.*CAPPADR
      CC=(C1-2.)*(AA+0.5*BB-3.*CAPPA/R)-2.*CAPPA/R
C
      DO 10 II=1,3
      DO 10 JJ=1,3
      U(JJ,II)=DELTA(JJ,II)*FHI-CAPPA*RD(II)*RD(JJ)
      U(JJ,II)=U(JJ,II)*C3
      T(JJ,II)=AA*(RDN*DELTA(JJ,II)+RD(II)*ETA(JJ))+
     1RD(II)*RD(JJ)*RDN*BB+RD(JJ)*ETA(II)*CC
      T(JJ,II)=T(JJ,II)*C4
      TS(JJ,II)=RDN*(C6*DELTA(JJ,II)+3.*RD(II)*RD(JJ))+
     1C6*(ETA(JJ)*RD(II)-ETA(II)*RD(JJ))
      TS(JJ,II)=C5*TS(JJ,II)/R**2
   10 CONTINUE
C
C  WHEN ICODE .GT. 0 , COMPUTE TDIF()
C
C  TDIF() IS COMPUTED FROM THE SERIES EXPANSION OF THE DIFFERENCE
C  BETWEEN THE DYNAMIC AND STATIC TRACTION TENSORS.
C
      IF (ICODE) 700,700,30
   30 IF (CABS(FR*R/CP)-0.4) 50,50,40
C
C  THE VALUES OF THE TRACTION TENSOR ARE FAR FROM BEING SINGULAR
C
   40 DO 45 II=1,3
      DO 45 JJ=1,3
   45 TDIF(JJ,II)=T(JJ,II)-TS(JJ,II)
      RETURN
C
C  THE MAXIMUM VALUE OF THE FREQUENCY IS LIMITED FOR AN
C  ADEQUATE CONVERGENCE OF THE SERIES
C
   50 IF (CABS(FR*R/CS)-2.*PI) 55,55,56
   56 STOP 'FUNTHEH : FREQUENCY TOO HIGH'
   55 AA=(0.,0.)
      BB=(0.,0.)
      CC=(0.,0.)
      FACTORIAL=1.
      DO 70 NA=0,100
      IF (NA.NE.0) FACTORIAL=FACTORIAL*NA
      TA=(NA+2.*(1.+C7**(NA+4)))*ZZS**NA/
     1((NA+2.)*(NA+4.)*FACTORIAL)
      AA=AA+TA
      TTA=CABS(TA)
      ABA=CABS(AA)
      IF((TTA/ABA).LE.TOL) GO TO 100
   70 CONTINUE
C
  100 FACTORIAL=1.
      DO 80 NB=0,100
      IF (NB.NE.0) FACTORIAL=FACTORIAL*NB
      TB=2.*(NB-1.)*(1.-C7**(NB+4))*ZZS**NB/
     1((NB+2.)*(NB+4.)*FACTORIAL)
      BB=BB+TB
      TTB=CABS(TB)
      ABB=CABS(BB)
      IF(((TTB/ABB).LE.TOL).AND.(NB.NE.1)) GO TO 101
   80 CONTINUE
C
  101 FACTORIAL=1.
      DO 90 NC=0,100
      IF (NC.NE.0) FACTORIAL=FACTORIAL*NC
      TC=(-2.+(NC+4.)*C7**(NC+2)-2.*(NC+3.)*C7**(NC+4))*ZZS**NC/
     1((NC+2.)*(NC+4.)*FACTORIAL)
      CC=CC+TC
      TTC=CABS(TC)
      ABC=CABS(CC)
      IF((TTC/ABC).LE.TOL) GO TO 102
   90 CONTINUE
C
  102 AA=ZKS*ZKS*AA
      BB=-ZKS*ZKS*BB
      CC=ZKS*ZKS*CC
C
      DO 110 II=1,3
      DO 110 JJ=1,3
      TDIF(JJ,II)=AA*(RDN*DELTA(JJ,II)+RD(II)*ETA(JJ))+
     1RD(II)*RD(JJ)*RDN*BB+RD(JJ)*ETA(II)*CC
  110 TDIF(JJ,II)=TDIF(JJ,II)*C4
C
  700 RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE LOCIN8(XJ1,XJ2,XJ3,HW,GW,HS,HDIF,NODO,NCONE)
C
C  THIS SUBROUTINE COMPUTES THE CONTRIBUTIONS TO THE H AND G MATRICES
C  OF AN ELEMENT WHICH CONTAINS THE COLLOCATION POINT
C
C  THE ELEMENT IS SUBDIVIDED INTO TRIANGLES DRAWING LINES FROM THE
C  COLLOCATION POINT.
C  INTEGRATION OVER THE RESULTING TRIANGLES IS DONE BY STANDARD GAUSS
C  QUADRATURE CORRESPONDING TO A COLAPSED QUADRILATERAL DOMAIN BY
C  CALLING SUBROUTINE TRILOC
C
      COMPLEX HW(3,27),GW(3,27),HDIF(3,3),CP,CS,GE,C1,C2,C3
      DIMENSION XI(3),V1(3),V2(3)
      DIMENSION XJ1(9),XJ2(9),XJ3(9),HS(3,27),DELTA(3,3)
      DIMENSION GI1(9),GI2(9),GII1(15),GII2(15)
      COMMON N,NE,L,CP,CS,GE,RO,XNU,FR,INP,IPR
      COMMON C1,C2,C3,C4,C5,C6,DELTA
C
      DO 5 I=1,3*NCONE
      DO 5 J=1,3
      HS(J,I)=0.
      HW(J,I)=(0.,0.)
    5 GW(J,I)=(0.,0.)
      DO 6 I=1,3
      DO 6 J=1,3
    6 HDIF(I,J)=(0.,0.)
C
      IF (9-NCONE) 100,100,110
C
C  QUADRILATERAL ELEMENT
C
C  SET THE VALUES OF THE HOMOGENEOUS COORDINATES FOR NODES 1 TO 9
C
  100 GI1(1)=-1.
      GI1(2)=0.
      GI1(3)=1.
      GI1(4)=1.
      GI1(5)=1.
      GI1(6)=0.
      GI1(7)=-1.
      GI1(8)=-1.
      GI1(9)=0.
      GI2(1)=-1.
      GI2(2)=-1.
      GI2(3)=-1.
      GI2(4)=0.
      GI2(5)=1.
      GI2(6)=1.
      GI2(7)=1.
      GI2(8)=0.
      GI2(9)=0.
      DO 15 I=1,8
      GII1(I)=GI1(I)
   15 GII2(I)=GI2(I)
C
C  STORE THE VALUES OF GI1,GI2 OF THE NODE 1 TO 7 ALSO AS 9 TO 15
C
      DO 17 I=1,7
      GII1(8+I)=GI1(I)
   17 GII2(8+I)=GI2(I)
C
C  STORE THE VALUES OF GI1,GI2 OF THE COLLOCATION NODE IN V1(1),
C  V2(1), AND ITS CARTESIAN COORDINATES IN XI(I)
C
      V1(1)=GI1(NODO)
      V2(1)=GI2(NODO)
      XI(1)=XJ1(NODO)
      XI(2)=XJ2(NODO)
      XI(3)=XJ3(NODO)
C
C  SUBDIVISION OF THE QUADRILATERAL ELEMENT INTO 2,3 OR 4 TRIANGLES
C  DEPENDING ON THE COLLOCATION POINT
C
      GO TO(1,2,1,2,1,2,1,2,9),NODO
    1 AREA=2.
      DO 11 K=1,2
      M=2*K
      V1(2)=GII1(NODO+M)
      V2(2)=GII2(NODO+M)
      V1(3)=GII1(NODO+M+2)
      V2(3)=GII2(NODO+M+2)
   11 CALL TRILOC8(XI,XJ1,XJ2,XJ3,HW,GW,HS,HDIF,AREA,V1,V2,NODO,9)
      GO TO 555
    2 DO 22 K=1,3
      M=2*K-1
      V1(2)=GII1(NODO+M)
      V2(2)=GII2(NODO+M)
      V1(3)=GII1(NODO+M+2)
      V2(3)=GII2(NODO+M+2)
      AREA=0.5*(V1(2)*V2(3)+V1(1)*V2(2)+V1(3)*V2(1)-
     1          V1(2)*V2(1)-V1(3)*V2(2)-V1(1)*V2(3))
   22 CALL TRILOC8(XI,XJ1,XJ2,XJ3,HW,GW,HS,HDIF,AREA,V1,V2,NODO,9)
      GO TO 555
    9 AREA=1.
      DO 99 K=1,4
      M=2*K-1
      V1(2)=GII1(M)
      V2(2)=GII2(M)
      V1(3)=GII1(M+2)
      V2(3)=GII2(M+2)
   99 CALL TRILOC8(XI,XJ1,XJ2,XJ3,HW,GW,HS,HDIF,AREA,V1,V2,NODO,9)
      GO TO 555
C
C  TRIANGULAR ELEMENT
C
C SET THE VALUES OF THE HOMOGENEOUS COORDINATES FOR NODES 1 TO 6
C
  110 GI1(1)=1.
      GI1(2)=0.
      GI1(3)=0.
      GI1(4)=0.5
      GI1(5)=0
      GI1(6)=0.5
      GI2(1)=0.
      GI2(2)=1.
      GI2(3)=0.
      GI2(4)=0.5
      GI2(5)=0.5
      GI2(6)=0.
C
C  STORE THE VALUES OF GI1,GI2 OF THE COLLOCATION NODE IN V1(1),
C  V2(1), AND ITS CARTESIAN COORDINATES IN XI(I)
C
      V1(1)=GI1(NODO)
      V2(1)=GI2(NODO)
      XI(1)=XJ1(NODO)
      XI(2)=XJ2(NODO)
      XI(3)=XJ3(NODO)
C
C  SUBDIVISION OF THE TRIANGULAR ELEMENT INTO 2 TRIANGLES
C  WHEN THE COLLOCATION POINT IS A MID-SIDE NODE. WHEN THE
C  COLLOCATION NODE IS A CORNER NO SUBDIVISION EXIST
C
      GO TO(10,20,30,40,50,60),NODO
   10 AREA=0.5
      V1(2)=GI1(2)
      V2(2)=GI2(2)
      V1(3)=GI1(3)
      V2(3)=GI2(3)
      CALL TRILOC8(XI,XJ1,XJ2,XJ3,HW,GW,HS,HDIF,AREA,V1,V2,1,6)
      GO TO 555
   20 AREA=0.5
      V1(2)=GI1(3)
      V2(2)=GI2(3)
      V1(3)=GI1(1)
      V2(3)=GI2(1)
      CALL TRILOC8(XI,XJ1,XJ2,XJ3,HW,GW,HS,HDIF,AREA,V1,V2,2,6)
      GO TO 555
   30 AREA=0.5
      V1(2)=GI1(1)
      V2(2)=GI2(1)
      V1(3)=GI1(2)
      V2(3)=GI2(2)
      CALL TRILOC8(XI,XJ1,XJ2,XJ3,HW,GW,HS,HDIF,AREA,V1,V2,3,6)
      GO TO 555
   40 AREA=0.25
      V1(2)=GI1(2)
      V2(2)=GI2(2)
      V1(3)=GI1(3)
      V2(3)=GI2(3)
      CALL TRILOC8(XI,XJ1,XJ2,XJ3,HW,GW,HS,HDIF,AREA,V1,V2,4,6)
      V1(2)=GI1(3)
      V2(2)=GI2(3)
      V1(3)=GI1(1)
      V2(3)=GI2(1)
      CALL TRILOC8(XI,XJ1,XJ2,XJ3,HW,GW,HS,HDIF,AREA,V1,V2,4,6)
      GO TO 555
   50 AREA=0.25
      V1(2)=GI1(3)
      V2(2)=GI2(3)
      V1(3)=GI1(1)
      V2(3)=GI2(1)
      CALL TRILOC8(XI,XJ1,XJ2,XJ3,HW,GW,HS,HDIF,AREA,V1,V2,5,6)
      V1(2)=GI1(1)
      V2(2)=GI2(1)
      V1(3)=GI1(2)
      V2(3)=GI2(2)
      CALL TRILOC8(XI,XJ1,XJ2,XJ3,HW,GW,HS,HDIF,AREA,V1,V2,5,6)
      GO TO 555
   60 AREA=0.25
      V1(2)=GI1(1)
      V2(2)=GI2(1)
      V1(3)=GI1(2)
      V2(3)=GI2(2)
      CALL TRILOC8(XI,XJ1,XJ2,XJ3,HW,GW,HS,HDIF,AREA,V1,V2,6,6)
      V1(2)=GI1(2)
      V2(2)=GI2(2)
      V1(3)=GI1(3)
      V2(3)=GI2(3)
      CALL TRILOC8(XI,XJ1,XJ2,XJ3,HW,GW,HS,HDIF,AREA,V1,V2,6,6)
  555 RETURN
      END
C----------------------------------------------------------------------
      SUBROUTINE TRILOC8(XI,XJ1,XJ2,XJ3,HW,GW,HS,HDIF,AREA,V1,V2,
     1NODO,NCONE)
C
C  THIS SUBROUTINE DOES A NUMERICAL INTEGRATION OVER A TRIANGLE
C  WHEN THE COLLOCATION POINT IS ONE OF THE CORNERS.
C
C  THE TRIANGLE IS TRANSFORMED INTO A SQUARE DOMAIN AND A DOUBLE GAUSS
C  QUADRATURE IS USED.
C
      COMPLEX HW(3,27),GW(3,27),HDIF(3,3),CP,CS,GE,C1,C2,C3
      COMPLEX T(3,3),TDIF(3,3),U(3,3)
      DIMENSION XI(3),X(3),XJ1(9),XJ2(9),XJ3(9),GI(6),OME(6),TS(3,3)
      DIMENSION F(9),FD1(9),FD2(9),DC(9,9),DCT(9,9),V1(3),V2(3)
      DIMENSION AUX(9),BUX(9),CUX(9),HS(3,27),ETA(3),P12(9),DELTA(3,3)
      COMMON N,NE,L,CP,CS,GE,RO,XNU,FR,INP,IPR
      COMMON C1,C2,C3,C4,C5,C6,DELTA
      DATA GI/0.238619186083197,-0.238619186083197,0.661209386466265,
     1       -0.661209386466265,0.932469514203152,-0.932469514203152/
      DATA OME/0.467913934572691,0.467913934572691,0.360761573048139,
     1         0.360761573048139,0.171324492379170,0.171324492379170/
C
      DO 400 I=1,6
      DO 400 J=1,6
C
C  COMPUTE THE NATURAL COORDINATES OF THE GAUSS POINTS IN THE SQUARE
C
      SS1=0.5*(GI(I)+1.)
      SS2=0.5*(GI(J)+1.)
      G1=(1.-SS1)*V1(1)+SS1*(1.-SS2)*V1(2)+SS1*SS2*V1(3)
      G2=(1.-SS1)*V2(1)+SS1*(1.-SS2)*V2(2)+SS1*SS2*V2(3)
C
C  COMPUTE VALUES OF THE SHAPE FUNCTIONS AND SHAPE FUNCTIONS
C  DERIVATIVES WITH RESPECT TO THE NATURAL COORDINATES
C  AT GAUSS POINTS
C
      IF (9-NCONE) 100,100,110
C
C  THE ORIGINAL DOMAIN IS A QUADRILATERAL
C
  100 F(1)=0.25*G1*G2*(G1-1.)*(G2-1.)
      F(2)=0.5*(1.-G1**2)*G2*(G2-1.)
      F(3)=0.25*G1*G2*(G1+1.)*(G2-1.)
      F(4)=0.5*G1*(G1+1.)*(1.-G2**2)
      F(5)=0.25*G1*G2*(G1+1.)*(G2+1.)
      F(6)=0.5*(1.-G1**2)*G2*(G2+1.)
      F(7)=0.25*G1*G2*(G1-1.)*(G2+1.)
      F(8)=0.5*G1*(G1-1.)*(1.-G2**2)
      F(9)=(1.-G1**2)*(1.-G2**2)
C
      FD1(1)=0.25*G2*(G2-1.)*(2.*G1-1.)
      FD1(2)=-G1*G2*(G2-1.)
      FD1(3)=0.25*G2*(G2-1.)*(2.*G1+1.)
      FD1(4)=0.5*(1.-G2**2)*(2.*G1+1.)
      FD1(5)=0.25*G2*(G2+1.)*(2.*G1+1.)
      FD1(6)=-G1*G2*(G2+1.)
      FD1(7)=0.25*G2*(G2+1.)*(2.*G1-1.)
      FD1(8)=0.5*(1.-G2**2)*(2.*G1-1.)
      FD1(9)=-2.*G1*(1.-G2**2)
C
      FD2(1)=0.25*G1*(G1-1.)*(2.*G2-1.)
      FD2(2)=0.5*(1.-G1**2)*(2.*G2-1.)
      FD2(3)=0.25*G1*(G1+1.)*(2.*G2-1.)
      FD2(4)=-G1*G2*(G1+1.)
      FD2(5)=0.25*G1*(G1+1.)*(2.*G2+1.)
      FD2(6)=0.5*(1.-G1**2)*(2.*G2+1.)
      FD2(7)=0.25*G1*(G1-1.)*(2.*G2+1.)
      FD2(8)=-G1*G2*(G1-1.)
      FD2(9)=-2.*G2*(1.-G1**2)
      GO TO 150
C
C  THE ORIGINAL DOMAIN IS A TRIANGLE
C
  110 G3=1.-G1-G2
      F(1)=G1*(2.*G1-1.)
      F(2)=G2*(2.*G2-1.)
      F(3)=G3*(2.*G3-1.)
      F(4)=4.*G1*G2
      F(5)=4.*G2*G3
      F(6)=4.*G3*G1
C
      FD1(1)=4.*G1-1.
      FD1(2)=0.
      FD1(3)=1.-4.*G3
      FD1(4)=4.*G2
      FD1(5)=-4.*G2
      FD1(6)=4.*(G3-G1)
      FD2(1)=0.
      FD2(2)=4.*G2-1.
      FD2(3)=1.-4.*G3
      FD2(4)=4.*G1
      FD2(5)=4.*(G3-G2)
      FD2(6)=-4.*G1
C
C  JACOBIAN AND NORMAL VECTOR FOR EACH GAUSS POINT
C
  150 DO 20 IN=1,NCONE
      DO 20 JN=1,NCONE
      DC(IN,JN)=FD1(IN)*FD2(JN)
   20 DCT(IN,JN)=FD2(IN)*FD1(JN)
      DO 30 M=1,NCONE
      AUX(M)=0.
      BUX(M)=0.
   30 CUX(M)=0.
      DO 40 JK=1,NCONE
      DO 40 IK=1,NCONE
      AUX(JK)=AUX(JK)+XJ2(IK)*(DC(IK,JK)-DCT(IK,JK))
      BUX(JK)=BUX(JK)+XJ3(IK)*(DC(IK,JK)-DCT(IK,JK))
   40 CUX(JK)=CUX(JK)+XJ1(IK)*(DC(IK,JK)-DCT(IK,JK))
      S1=0.
      S2=0.
      S3=0.
      DO 50 M=1,NCONE
      S1=S1+AUX(M)*XJ3(M)
      S2=S2+BUX(M)*XJ1(M)
   50 S3=S3+CUX(M)*XJ2(M)
      XJA=SQRT(S1**2+S2**2+S3**2)
      ETA(1)=S1/XJA
      ETA(2)=S2/XJA
      ETA(3)=S3/XJA
C
C  CARTESIAN CO-ORDINATES OF THE GAUSS POINT
C
      DO 60 IN=1,3
   60 X(IN)=0.
      DO 70 IK=1,NCONE
      X(1)=X(1)+F(IK)*XJ1(IK)
      X(2)=X(2)+F(IK)*XJ2(IK)
   70 X(3)=X(3)+F(IK)*XJ3(IK)
C
      DO 80 IK=1,NCONE
   80 P12(IK)=F(IK)*XJA*(GI(I)+1.)*AREA*0.25*OME(I)*OME(J)
      CALL FUNTHEH(U,T,TS,XI(1),XI(2),XI(3),ETA,X(1),X(2),X(3)
     1,1,TDIF)
C
C  SUMMATION OVER THE GAUSS POINTS TO COMPUTE HW, GW, HS AND HDIF
C
      DO 90 KK=1,NCONE
      DO 90 II=1,3
      DO 90 JJ=1,3
      IND=3*KK+II-3
      GW(JJ,IND)=GW(JJ,IND)+U(JJ,II)*P12(KK)
      HW(JJ,IND)=HW(JJ,IND)+T(JJ,II)*P12(KK)
      HS(JJ,IND)=HS(JJ,IND)+TS(JJ,II)*P12(KK)
   90 CONTINUE
      DO 160 II=1,3
      DO 160 JJ=1,3
  160 HDIF(JJ,II)=HDIF(JJ,II)+TDIF(JJ,II)*P12(NODO)
C
  400 CONTINUE
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE INTER8(FI,DFI,KODE,BC,CX,CY,CZ,X,Y,Z,DSOL,NCONEC,
     1KCONEC,NESIM,KE)
C
C  THIS SUBROUTINE COMPUTES THE VALUES OF THE DISPLACEMENT AT INTERNAL
C  POINTS
C
      COMPLEX FI(1),DFI(27,1),DSOL(1),BC(27,1),HW(3,27),GW(3,27)
      COMPLEX CP,CS,GE,C1,C2,C3
      DIMENSION KODE(1),X(1),Y(1),Z(1),CX(1),CY(1),CZ(1),XI(3),HS(3,27)
      DIMENSION ETA1(36),ETA2(36),ETA3(36),XJA(36),FF(9,36),DELTA(3,3)
      DIMENSION XJ1(9),XJ2(9),XJ3(9),XG1(36),XG2(36),XG3(36)
      DIMENSION NCONEC(1),KCONEC(9,1),NESIM(1),KE(5,1)
      COMMON N,NE,L,CP,CS,GE,RO,XNU,FR,INP,IPR
      COMMON C1,C2,C3,C4,C5,C6,DELTA
C
C  REARRANGE FI AND DFI ARRAYS TO STORE ALL THE VALUES OF THE
C  DISPLACEMENT IN FI AND ALL THE VALUES OF TRACTION IN DFI
C
      DO 5 IE=1,NE
      DO 6 LD=1,3
      IF (KODE(3*IE-3+LD)) 10,10,11
   10 DO 12 K=1,NCONEC(IE)
      IK=KCONEC(K,IE)
   12 DFI(3*K-3+LD,IE)=FI(3*IK-3+LD)*GE
      GO TO 6
   11 DO 13 K=1,NCONEC(IE)
   13 DFI(3*K-3+LD,IE)=BC(3*K-3+LD,IE)
    6 CONTINUE
    5 CONTINUE
C
      DO 20 IN=1,N
      DO 15 LD=1,3
      DO 16 J=1,NESIM(IN)
      IF (KODE(3*KE(J,IN)-3+LD)) 17,17,16
   17 DO 25 M=1,NCONEC(KE(J,IN))
      IF (IN-KCONEC(M,KE(J,IN))) 25,26,25
   26 FI(3*IN-3+LD)=BC(3*M-3+LD,KE(J,IN))
      GO TO 15
   25 CONTINUE
   16 CONTINUE
   15 CONTINUE
   20 CONTINUE
C
C  COMPUTE THE VALUES OF DISPLACEMENT AT INTERNAL POINTS
C
      IF (L.LE.0) RETURN
C
      DO 40 I=1,3*L
   40 DSOL(I)=(0.,0.)
C
      DO 120 IE=1,NE
      DO 50 II=1,NCONEC(IE)
      IK=KCONEC(II,IE)
      XJ1(II)=X(IK)
      XJ2(II)=Y(IK)
   50 XJ3(II)=Z(IK)
      CALL ETAXJA(XJ1,XJ2,XJ3,ETA1,ETA2,ETA3,XJA,XG1,XG2,XG3,FF,
     1NCONEC(IE))
      DO 110 K=1,L
      XI(1)=CX(K)
      XI(2)=CY(K)
      XI(3)=CZ(K)
      CALL EXTIN8(ETA1,ETA2,ETA3,XJA,XG1,XG2,XG3,FF,XI,HW,GW,HS,
     1NCONEC(IE))
      DO 65 LD=1,3
      DO 60 M=1,NCONEC(IE)
      JK=KCONEC(M,IE)
      DSOL(3*K-2)=DSOL(3*K-2)+GW(1,3*M-3+LD)*DFI(3*M-3+LD,IE)-
     1HW(1,3*M-3+LD)*FI(3*JK-3+LD)
      DSOL(3*K-1)=DSOL(3*K-1)+GW(2,3*M-3+LD)*DFI(3*M-3+LD,IE)-
     1HW(2,3*M-3+LD)*FI(3*JK-3+LD)
   60 DSOL(3*K)=DSOL(3*K)+GW(3,3*M-3+LD)*DFI(3*M-3+LD,IE)-
     1HW(3,3*M-3+LD)*FI(3*JK-3+LD)
   65 CONTINUE
  110 CONTINUE
  120 CONTINUE
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE OUTPUT8(FI,DFI,DSOL,NCONEC,KCONEC)
C
C  THIS SUBROUTINE PRINTS THE VALUES OF DISPLACEMENTS AND TRACTIONS
C  AT BOUNDARY NODES. IT ALSO PRINTS THE VALUES OF DISPLACEMENTS AT
C  INTERNAL POINTS.
C
      COMPLEX FI(1),DFI(27,1),DSOL(1)
      COMPLEX CP,CS,GE,C1,C2,C3
      DIMENSION NCONEC(1),KCONEC(9,1),DELTA(3,3)
      COMMON N,NE,L,CP,CS,GE,RO,XNU,FR,INP,IPR
      COMMON C1,C2,C3,C4,C5,C6,DELTA
C
      WRITE(IPR,50)
   50 FORMAT(/,' ',79('*')/)
      WRITE(IPR,700) FR
  700 FORMAT(30X,'FREQUENCY = ',G14.7/)
      WRITE(IPR,100)
  100 FORMAT(' ',79('*')//1X,'RESULTS'//2X,'BOUNDARY NODES'//,
     1' NODE',6X,'DISPLACEMENT X',9X,'DISPLACEMENT Y',10X,
     2'DISPLACEMENT Z'/)
      DO 10 I=1,N
   10 WRITE(IPR,200) I,(FI(3*I-3+J),J=1,3)
  200 FORMAT(1X,I3,3(1X,G11.4,',',G11.4))
      WRITE(IPR,110)
  110 FORMAT(//2X,'BOUNDARY ELEMENTS'//1X,'ELEMENT',2X,'NODE',6X,
     1'TRACTION X',12X,'TRACTION Y',12X,'TRACTION Z'/)
      DO 20 IE=1,NE
      WRITE(IPR,120) IE
  120 FORMAT(/1X,I3)
      DO 30 K=1,NCONEC(IE)
      IK=KCONEC(K,IE)
   30 WRITE(IPR,130) IK,(DFI(3*K-3+J,IE),J=1,3)
   20 CONTINUE
  130 FORMAT(10X,I3,3(1X,G10.3,',',G10.3))
C
      IF (L.LE.0) GO TO 222
      WRITE(IPR,300)
  300 FORMAT(//2X,'INTERNAL POINTS',//'POINT',4X,
     1'DISPLACEMENT X',10X,'DISPLACEMENT Y',10X,'DISPLACEMENT Z'/)
      DO 60 K=1,L
   60 WRITE(IPR,200) K,(DSOL(3*K-3+J),J=1,3)
C
  222 WRITE(IPR,500)
  500 FORMAT(/' ',79('*'))
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE CSOLVER(A,B,N,NX)
C
      COMPLEX A(NX,NX),B(NX),X,CDOT
C
C     A=L*U  FACTORIZATION
C
      DO 30 K=1,N
      K1=K-1
      DO 10 I=K,N
      A(I,K)=A(I,K)-CDOT(K1,A(I,1),NX,A(1,K),1)
   10 CONTINUE
      X=A(K,K)
      IF(ABS(REAL(X))+ABS(AIMAG(X)).EQ.0.) THEN
      WRITE(6,*) ' SOLVER : STOP  SINGULARITY IN ROW ',K
      STOP ' SOLVER : STOP SINGULARITY '
      END IF
      X=(1.,0.)/X
      DO 20 J=K+1,N
      A(K,J)=X*(A(K,J)-CDOT(K1,A(K,1),NX,A(1,J),1))
   20 CONTINUE
   30 CONTINUE
C
C  FORWARD AND BACK SUBSTITUTION    L*U*X=B
C
C  SOLVE  L*Y=B
C
      DO 40 I=1,N
      B(I)=-(CDOT((I-1),A(I,1),NX,B,1)+B(I))/A(I,I)
   40 CONTINUE
C
C  SOLVE  U*X=Y
C
      DO 50 I=N,1,-1
      B(I)=-(CDOT((N-I),A(I,I+1),NX,B(I+1),1)+B(I))
   50 CONTINUE
C
      RETURN
      END
C-----------------------------------------------------------------------
      COMPLEX FUNCTION CDOT(N,U,IUX,V,IVX)
C
C THIS FUNCTION MAKES THE COMPLEX DOT PRODUCT OF TWO VECTORS
C
      COMPLEX U(1),V(1)
C
      CDOT=(0.,0.)
      IF(N.LE.0) RETURN
      IU=1
      IV=1
      IF(IUX.LT.0) IU=1-(N-1)*IUX
      IF(IVX.LT.0) IV=1-(N-1)*IVX
      DO 10 I=1,N
      CDOT=CDOT+U(IU)*V(IV)
      IU=IU+IUX
      IV=IV+IVX
10    CONTINUE
      RETURN
C
      END
