C------------------------------------------------------------------
C
      PROGRAM QUADTHSH
C
C  THIS IS A B.E. PROGRAM THAT USES (QUA)DRATIC ELEMENTS FOR THE
C  SOLUTION OF (D)YNAMIC (TH)REE-DIMENSIONAL (S)CALAR WAVE TIME
C  (H)ARMONIC PROBLEMS
C
C  PROGRAM NO. 6
C
C  SET MAXIMUM DIMENSIONS
C  NX= MAXIMUM DIMENSION OF THE SYSTEM OF EQUATIONS
C    = MAXIMUM NUMBER OF NODES
C  NNE=MAXIMUM NUMBER OF ELEMENTS
C  NPM=MAXIMUM NUMBER OF INTERNAL POINTS
C  NFM=MAXIMUM NUMBER OF FREQUENCIES
C
      PARAMETER(NX=100,NNE=50,NPM=20,NFM=10)
C
      CHARACTER*12 FILEIN,FILEOUT
C
      COMPLEX A(NX,NX),FI(NX),BC(9,NNE),DFI(9,NNE),DSOL(NPM)
      COMMON/MATA/ A
      COMMON/VECF/ FI
      COMMON N,NE,L,CW,FR,INP,IPR
      DIMENSION X(NX),Y(NX),Z(NX),CX(NPM),CY(NPM),CZ(NPM),KODE(NNE)
      DIMENSION AFR(NFM),NESIM(NX),KE(5,NX),NCONEC(NNE),KCONEC(9,NNE)
C
C  ASSIGN NUMBERS FOR INPUT AND OUTPUT FILES
C
      INP=5
      IPR=6
C
C  READ NAMES AND OPEN  FILES FOR INPUT AND OUTPUT
C
      WRITE(*,' (A) ') ' NAME OF INPUT FILE (MAX. 12 CHART.)'
      READ(*,'  (A) ')FILEIN
      OPEN(INP,FILE=FILEIN,STATUS='OLD')
      WRITE(*,' (A) ') ' NAME OF OUTPUT FILE (MAX. 12 CHART.)'
      READ(*,' (A) ')FILEOUT
      OPEN(IPR,FILE=FILEOUT,STATUS='NEW')
C
C  READ DATA
C
      CALL INPUT6(CX,CY,CZ,X,Y,Z,KODE,BC,NFR,AFR,NESIM,KE,NCONEC,
     1KCONEC)
      DO 60 I=1,NFR
      FR=AFR(I)
C
C  FORM SYSTEM OF EQUATIONS
C
      CALL GHMAT6(X,Y,Z,A,FI,BC,KODE,NCONEC,KCONEC,NESIM,KE,NX)
C
C  SOLVE SYSTEM OF EQUATIONS
C
      CALL CSOLVER(A,FI,N,NX)
C
C  COMPUTE PRESSURE VALUES AT INTERNAL POINTS
C
      CALL INTER6(FI,DFI,KODE,BC,CX,CY,CZ,X,Y,Z,DSOL,NCONEC,KCONEC,
     1NESIM,KE)
C
C  PRINT RESULTS AT BOUNDARY NODES AND INTERNAL POINTS
C
      CALL OUTPUT6(FI,DFI,DSOL,CX,CY,CZ,NCONEC,KCONEC)
   60 CONTINUE
C
C   CLOSE INPUT AND OUTPUT FILES
C
      CLOSE (INP)
      CLOSE (IPR)
      STOP
      END
C-----------------------------------------------------------------------
      SUBROUTINE INPUT6(CX,CY,CZ,X,Y,Z,KODE,BC,NFR,AFR,NESIM,KE,NCONEC,
     1KCONEC)
C
      CHARACTER*80 TITLE
      DIMENSION KODE(1),X(1),Y(1),Z(1),CX(1),CY(1),CZ(1),AFR(1)
      DIMENSION NESIM(1),KE(5,1),NCONEC(1),KCONEC(9,1)
      COMPLEX BC(9,1)
      COMMON N,NE,L,CW,FR,INP,IPR
C
C  N = NUMBER OF BOUNDARY NODES
C  NE= NUMBER OF BOUNDARY ELEMENTS
C  L = NUMBER OF INTERNAL POINTS
C  CW= WAVE VELOCITY
C  NFR= NUMBER OF FREQUENCIES TO BE ANALYSED
C  AFR= ARRAY FOR STORAGE OF FREQUENCIES
C  FR= FREQUENCY
C  NCONEC= ARRAY FOR STORAGE OF NUMBER OF NODES OF THE ELEMENT (6 OR 9)
C  KCONEC= MATRIX FOR STORAGE ELEMENT CONNECTIVITIES
C
      WRITE(IPR,100)
  100 FORMAT(' ',79('*'))
C
C  READ JOB TITLE
C
      READ(INP,'(A)') TITLE
      WRITE(IPR,'(A)') TITLE
C
C  READ NUMBER OF BOUNDARY NODES ,ELEMENTS, INTERNAL POINTS AND
C  FREQUENCIES; READ WAVE VELOCITY
C
      READ(INP,*)N,NE,L,NFR,CW
      WRITE(IPR,300)N,NE,L,NFR,CW
  300 FORMAT(//' DATA'//2X,
     1'NUMBER OF BOUNDARY NODES    =',I3/2X,
     2'NUMBER OF BOUNDARY ELEMENTS =',I3/2X,
     3'NUMBER OF INTERNAL POINTS   =',I3/2X,
     4'NUMBER OF FREQUENCIES       =',I3/2X,
     5'WAVE VELOCITY               =',G14.7/)
C
C  READ FREQUENCIES
C
      READ(INP,*) (AFR(I),I=1,NFR)
      WRITE(IPR,400) (AFR(I),I=1,NFR)
  400 FORMAT(/2X,'FREQUENCIES'/5(2X,G14.7))
C
C  READ BOUNDARY NODES COORDINATES
C
      WRITE(IPR,500)
  500 FORMAT(//2X,'BOUNDARY NODES COORDINATES'///4X,
     1'NODE',10X,'X',18X,'Y',17X,'Z'/)
      READ(INP,*) (IN,X(IN),Y(IN),Z(IN),I=1,N)
      WRITE(IPR,700) (I,X(I),Y(I),Z(I),I=1,N)
  700 FORMAT(5X,I3,3G19.7)
C
C  READ NUMBER OF NODES AND CONNECTIVITIES FOR EACH ELEMENT
C
      WRITE(IPR,740)
  740 FORMAT(//2X,'ELEMENT CONNECTIVITIES'///2X,'ELEMENT',1X,
     1'NO.CON.',24X,'NODES'/)
      DO 15 I=1,NE
      READ(INP,*) IE,NCONEC(IE),(KCONEC(J,IE),J=1,NCONEC(IE))
   15 WRITE(IPR,780) IE,NCONEC(IE),(KCONEC(J,IE),J=1,NCONEC(IE))
  780 FORMAT(4X,I3,4X,I2,2X,9(3X,I3))
C
C  READ BOUNDARY CONDITIONS IN BC() MATRIX;
C  IF KODE()=0 THE BC() VALUES ARE KNOWN POTENTIALS;
C  IF KODE()=1 THE BC() VALUES ARE KNOWN FLUXES.
C  NINE OR SIX BOUNDARY CONDITIONS ARE READ PER ELEMENT.
C  NODES BETWEEN TWO OR MORE ELEMENTS MAY HAVE DIFFERENT VALUES
C  OF THE FLUX BUT ONLY ONE VALUE OF THE POTENTIAL.
C
      WRITE(IPR,800)
  800 FORMAT(//2X,'BOUNDARY CONDITIONS'//1X,
     1'ELEMT.',1X,'CODE',22X,'PRESCRIBED VALUES',/)
      DO 30 I=1,NE
      READ(INP,*) IE,KODE(IE),(BC(J,IE),J=1,NCONEC(IE))
   30 WRITE(IPR,950) IE,KODE(IE),(BC(J,IE),J=1,NCONEC(IE))
  950 FORMAT(1X,I3,4X,I2,2X,3(1X,G10.3,',',G10.3)/
     1(12X,1X,G10.3,',',G10.3,1X,G10.3,',',G10.3,1X,G10.3,',',G10.3))
C
C  READ INTERNAL POINTS COORDINATES
C
      IF(L.EQ.0) GO TO 150
      READ(INP,*) (CX(I),CY(I),CZ(I),I=1,L)
C
C  DETERMINE THE NUMBER OF ELEMENTS TO WHICH EACH NODE BELONGS
C  AND DETERMINE THE ORDER NUMBERS OF THOSE ELEMENTS
C
  150 DO 40 I=1,N
   40 NESIM(I)=0
      DO 50 IE=1,NE
      DO 50 J=1,NCONEC(IE)
      IN=KCONEC(J,IE)
      NESIM(IN)=NESIM(IN)+1
   50 KE(NESIM(IN),IN)=IE
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE GHMAT6(X,Y,Z,A,FI,BC,KODE,NCONEC,KCONEC,NESIM,KE,NX)
C
C  THIS SUBROUTINE COMPUTES THE A MATRIX AND THE RIGHT HAND SIDE VECTOR
C  FI AND FORMS THE SYSTEM A X = F READY TO BE SOLVED
C
      COMPLEX A(NX,1),FI(1),BC(9,1),HW(9),GW(9),HDIF,D2,VP
      DIMENSION X(1),Y(1),Z(1),KODE(1),NESIM(1),KE(5,1),NCONEC(1)
      DIMENSION KCONEC(9,1),HS(9),XJ1(9),XJ2(9),XJ3(9),XI(3)
      DIMENSION ETA1(36),ETA2(36),ETA3(36),XJA(36)
      DIMENSION XG1(36),XG2(36),XG3(36),FF(9,36)
      COMMON N,NE,L,CW,FR,INP,IPR
      DO 20 I=1,N
      FI(I)=(0.,0.)
      DO 11 J=1,N
   11 A(J,I)=(0.,0.)
   20 CONTINUE
      HS11=0.
C
C  COMPUTE THE GW AND HW MATRICES FOR EACH BOUNDARY ELEMENT AND EACH
C  COLLOCATION POINT
C
      DO 500 IEJ=1,NE
C
C  STORE THE COORDINATES OF THE SIX OR NINE NODES OF THE 'IEJ'
C  ELEMENT IN 'XJ1, XJ2, XJ3'
C
      DO 30 M=1,NCONEC(IEJ)
      K=KCONEC(M,IEJ)
      XJ1(M)=X(K)
      XJ2(M)=Y(K)
   30 XJ3(M)=Z(K)
C
C  COMPUTE THE ARRAYS OF CARTESIAN COORDINATES 'XG1,XG2,XG3'
C  SHAPE FUNCTIONS 'FF', JACOBIANS 'XJA' AND COMPONENTS OF
C  THE OUTWARD NORMAL 'ETA' CORRESPONDING TO THE GAUSS POINTS,
C  FOR EACH ELEMENT
C
      CALL ETAXJA(XJ1,XJ2,XJ3,ETA1,ETA2,ETA3,XJA,XG1,XG2,XG3,FF,
     1NCONEC(IEJ))
      DO 400 IN=1,N
C
C  COMPUTE THE INTEGRALS OVER ELEMENT 'IEJ' FROM NODE 'IN'
C
      XI(1)=X(IN)
      XI(2)=Y(IN)
      XI(3)=Z(IN)
      NODO=0
      DO 60 I=1,NCONEC(IEJ)
      IF (IN.EQ.KCONEC(I,IEJ)) NODO=I
   60 CONTINUE
      IF(NODO) 55,55,65
   65 CALL LOCIN6(XJ1,XJ2,XJ3,HW,GW,HS,HDIF,NODO,NCONEC(IEJ))
      GO TO 120
   55 CALL EXTIN6(ETA1,ETA2,ETA3,XJA,XG1,XG2,XG3,FF,XI,HW,GW,HS,
     1NCONEC(IEJ))
C
C  COMPUTE CONTRIBUTION TO THE DIAGONAL COEFFICIENTS OF
C  MATRIX H (D1 OR D2)
C
  120 D1=0.
      D2=(0.,0.)
      IF(NODO) 75,75,76
C
C  THE COLLOCATION NODE DOES NOT BELONG TO THE INTEGRATION ELEMENT
C
   75 DO 90 I=1,NCONEC(IEJ)
   90 D1=D1-HS(I)
      IF (IN.EQ.1) HS11=HS11+D1
      GO TO 130
C
C  THE COLLOCATION NODE BELONGS TO THE INTEGRATION ELEMENT
C
   76 DO 100 I=1,NCONEC(IEJ)
      IF(I-NODO) 77,78,77
   77 D2=D2-HS(I)
      IF (IN.EQ.1) HS11=HS11-HS(I)
      GO TO 100
   78 D2=HDIF+D2
  100 CONTINUE
C
C  FORM SYSTEM OF EQUATIONS USING THE KODE()
C
  130 IF (KODE(IEJ)) 200,210,220
  200 WRITE(IPR,1000) IEJ
 1000 FORMAT(/2X,'NEGATIVE KODE OF THE ELEMENT ',I3)
      STOP
C
C  POTENTIAL IS PRESCRIBED ON ELEMENT IEJ
C
  210 DO 250 K=1,NCONEC(IEJ)
      IK=KCONEC(K,IEJ)
      A(IN,IK)=A(IN,IK)-GW(K)
      IF (IN-IK) 260,265,260
  260 FI(IN)=FI(IN)-HW(K)*BC(K,IEJ)
      GO TO 250
  265 FI(IN)=FI(IN)-D2   *BC(K,IEJ)
  250 CONTINUE
      GO TO 350
C
C  NORMAL FLUX IS PRESCRIBED ON ELEMENT IEJ
C
  220 DO 270 K=1,NCONEC(IEJ)
      IK=KCONEC(K,IEJ)
      KODE0=0
C
C  CHECK FOR ALL THE ELEMENTS TO WHICH NODE IK BELONGS
C
      DO 280 I=1,NESIM(IK)
      IF (KODE(KE(I,IK))) 280,295,280
  295 DO 300 M=1,NCONEC(KE(I,IK))
C
C  STORE IN VP THE PRESCRIBED POTENTIAL FOR THE NODE IK AS
C  PART OF ANOTHER ELEMENT. TO BE USED ALSO IN ELEMENT IEJ
C
      IF(IK-KCONEC(M,KE(I,IK))) 300,297,300
  297 VP=BC(M,KE(I,IK))
      KODE0=1
      GO TO 22
  300 CONTINUE
  280 CONTINUE
   22 IF (KODE0) 310,315,310
C
C  ALL THE ELEMENTS TO WHICH IK BELONGS HAVE PRESCRIBED FLUXES.
C  THE POTENTIAL IS THE UNKNOWN.
C
  315 FI(IN)=FI(IN)+GW(K)*BC(K,IEJ)
      IF (IN-IK) 320,325,320
  320 A(IN,IK)=A(IN,IK)+HW(K)
      GO TO 270
  325 A(IN,IN)=A(IN,IN)+D2
      GO TO 270
C
C  THE POTENTIAL IS ALSO PRESCRIBED FOR NODE IK. THE UNKNOWN IS
C  ANOTHER FLUX
C
  310 IF (IN-IK) 330,335,330
  330 FI(IN)=FI(IN)+GW(K)*BC(K,IEJ)-HW(K)*VP
      GO TO 270
  335 FI(IN)=FI(IN)+GW(K)*BC(K,IEJ)-D2   *VP
  270 CONTINUE
C
C  IF THE COLLOCATION POINT 'IN' DOES NOT BELONG TO THE INTEGRATION
C  ELEMENT  ADD D1 TO THE DIAGONAL COEFFICIENT OF MATIX H OR
C  -D1*BC TO THE RIGHT HAND SIDE VECTOR.
C
  350 IF (NODO) 400,340,400
  340 KODE0=0
      DO 380 I=1,NESIM(IN)
      IF (KODE(KE(I,IN))) 380,385,380
  385 DO 390 M=1,NCONEC(KE(I,IN))
      IF (IN-KCONEC(M,KE(I,IN))) 390,395,390
  395 VP=BC(M,KE(I,IN))
      KODE0=1
      GO TO 33
  390 CONTINUE
  380 CONTINUE
   33 IF (KODE0) 396,399,396
  399 A(IN,IN)=A(IN,IN)+D1
      GO TO 400
  396 FI(IN)=FI(IN)-D1*VP
  400 CONTINUE
  500 CONTINUE
C
C  ADD ONE TO THE DIAGONAL TERMS OF THE H MATRIX FOR EXTERNAL PROBLEMS
C
      IF (HS11) 510,520,520
  510 DO 600 IN=1,N
      KODE0=0
      DO 530 I=1,NESIM(IN)
      IF (KODE(KE(I,IN))) 530,545,530
  545 DO 550 M=1,NCONEC(KE(I,IN))
      IF (IN-KCONEC(M,KE(I,IN))) 550,555,550
  555 VP=BC(M,KE(I,IN))
      KODE0=1
      GO TO 535
  550 CONTINUE
  530 CONTINUE
  535 IF (KODE0) 560,565,560
  565 A(IN,IN)=A(IN,IN)+1.
      GO TO 600
  560 FI(IN)=FI(IN)-1.*VP
  600 CONTINUE
  520 CONTINUE
C
      RETURN
      END
C---------------------------------------------------------------------
      SUBROUTINE ETAXJA(XJ1,XJ2,XJ3,ETA1,ETA2,ETA3,XJA,XG1,XG2,XG3,
     1FF,NCONE)
C
C  THIS SUBROUTINE COMPUTES THE COMPONENTS OF THE NORMAL VECTORS,
C  COORDINATES,JACOBIANS AND SHAPE FUNCTIONS FOR THE GAUSS POINTS
C
      DIMENSION ETA1(1),ETA2(1),ETA3(1),XJA(1)
      DIMENSION XG1(1),XG2(1),XG3(1),FF(9,1)
      DIMENSION XJ1(1),XJ2(1),XJ3(1)
      DIMENSION FD1(9),FD2(9),DC(9,9),DCT(9,9)
      DIMENSION AUX(9),BUX(9),CUX(9)
      DIMENSION GI(6),RR(4),SS(4)
C
C  GI()      : GAUSS POINTS FOR QUADRILATERAL ELEMENTS
C  RR(),SS() : GAUSS POINTS FOR TRIANGULAR ELEMENTS
C
      DATA GI/0.238619186083197,-0.238619186083197,0.661209386466265,
     1        -0.661209386466265,0.932469514203152,-0.932469514203152/
      DATA RR/ .0694318422 , .3300094782 , .6699905218 , .9305681558 /
      DATA SS/ .0571041961 , .2768430136 , .5835904324 , .8602401357 /
C
      IF (9-NCONE) 1,1,2
    1 NPG=6
      GO TO 3
    2 NPG=4
    3 DO 10 I=1,NPG*NPG
      ETA1(I)=0.
      ETA2(I)=0.
      ETA3(I)=0.
   10 XJA(I)=0.
C
      DO  200 I=1,NPG
      DO  200 J=1,NPG
C
C  COMPUTE THE ORDER NUMBER OF THE GAUSS POINT
C
      K=NPG*I-NPG+J
C
      IF (9-NCONE) 100,100,110
C
C  COMPUTE VALUES OF THE SHAPE FUNCTIONS FOR QUADRILATERAL ELEMENTS
C
  100 G1=GI(I)
      G2=GI(J)
      FF(1,K)=0.25*G1*G2*(G1-1.)*(G2-1.)
      FF(2,K)=0.5*(1.-G1**2)*G2*(G2-1.)
      FF(3,K)=0.25*G1*G2*(G1+1.)*(G2-1.)
      FF(4,K)=0.5*G1*(G1+1.)*(1.-G2**2)
      FF(5,K)=0.25*G1*G2*(G1+1.)*(G2+1.)
      FF(6,K)=0.5*(1.-G1**2)*G2*(G2+1.)
      FF(7,K)=0.25*G1*G2*(G1-1.)*(G2+1.)
      FF(8,K)=0.5*G1*(G1-1.)*(1.-G2**2)
      FF(9,K)=(1.-G1**2)*(1.-G2**2)
C
C  COMPUTE SHAPE FUNCTIONS DERIVATIVES FOR QUADRILATERAL ELEMENTS
C
      FD1(1)=0.25*G2*(G2-1.)*(2.*G1-1.)
      FD1(2)=-G1*G2*(G2-1.)
      FD1(3)=0.25*G2*(G2-1.)*(2.*G1+1.)
      FD1(4)=0.5*(1.-G2**2)*(2.*G1+1.)
      FD1(5)=0.25*G2*(G2+1.)*(2.*G1+1.)
      FD1(6)=-G1*G2*(G2+1.)
      FD1(7)=0.25*G2*(G2+1.)*(2.*G1-1.)
      FD1(8)=0.5*(1.-G2**2)*(2.*G1-1.)
      FD1(9)=-2.*G1*(1.-G2**2)
C
      FD2(1)=0.25*G1*(G1-1.)*(2.*G2-1.)
      FD2(2)=0.5*(1.-G1**2)*(2.*G2-1.)
      FD2(3)=0.25*G1*(G1+1.)*(2.*G2-1.)
      FD2(4)=-G1*G2*(G1+1.)
      FD2(5)=0.25*G1*(G1+1.)*(2.*G2+1.)
      FD2(6)=0.5*(1.-G1**2)*(2.*G2+1.)
      FD2(7)=0.25*G1*(G1-1.)*(2.*G2+1.)
      FD2(8)=-G1*G2*(G1-1.)
      FD2(9)=-2.*G2*(1.-G1**2)
      GO TO 120
C
C  COMPUTE VALUES OF THE SHAPE FUNCTIONS FOR TRIANGULAR ELEMENTS
C
  110 G1=SS(J)
      G2=RR(I)*(1.-SS(J))
      G3=1.-G1-G2
      FF(1,K)=G1*(2.*G1-1.)
      FF(2,K)=G2*(2.*G2-1.)
      FF(3,K)=G3*(2.*G3-1.)
      FF(4,K)=4.*G1*G2
      FF(5,K)=4.*G2*G3
      FF(6,K)=4.*G3*G1
C
C  COMPUTE SHAPE FUNCTIONS DERIVATIVES FOR TRIANGULAR ELEMENTS
C
      FD1(1)=4.*G1-1.
      FD1(2)=0.
      FD1(3)=1.-4.*G3
      FD1(4)=4.*G2
      FD1(5)=-4.*G2
      FD1(6)=4.*(G3-G1)
C
      FD2(1)=0.
      FD2(2)=4.*G2-1.
      FD2(3)=1.-4.*G3
      FD2(4)=4.*G1
      FD2(5)=4.*(G3-G2)
      FD2(6)=-4.*G1
C
C
C  CARTESIAN CO-ORDINATES OF THE GAUSS POINT
C
  120 XG1(K)=0.
      XG2(K)=0.
      XG3(K)=0.
      DO 50 IK=1,NCONE
      XG1(K)=XG1(K)+FF(IK,K)*XJ1(IK)
      XG2(K)=XG2(K)+FF(IK,K)*XJ2(IK)
   50 XG3(K)=XG3(K)+FF(IK,K)*XJ3(IK)
C
C  JACOBIAN AND NORMAL VECTOR FOR THE GAUSS POINT
C
      DO 60 IN=1,NCONE
      DO 60 JN=1,NCONE
      DC(IN,JN)=FD1(IN)*FD2(JN)
   60 DCT(IN,JN)=FD2(IN)*FD1(JN)
      DO 70 M=1,NCONE
      AUX(M)=0.
      BUX(M)=0.
   70 CUX(M)=0.
      DO 80 JK=1,NCONE
      DO 80 IK=1,NCONE
      AUX(JK)=AUX(JK)+XJ2(IK)*(DC(IK,JK)-DCT(IK,JK))
      BUX(JK)=BUX(JK)+XJ3(IK)*(DC(IK,JK)-DCT(IK,JK))
   80 CUX(JK)=CUX(JK)+XJ1(IK)*(DC(IK,JK)-DCT(IK,JK))
      S1=0.
      S2=0.
      S3=0.
      DO 90 M=1,NCONE
      S1=S1+AUX(M)*XJ3(M)
      S2=S2+BUX(M)*XJ1(M)
   90 S3=S3+CUX(M)*XJ2(M)
      XJA(K)=SQRT(S1**2+S2**2+S3**2)
      ETA1(K)=S1/XJA(K)
      ETA2(K)=S2/XJA(K)
      ETA3(K)=S3/XJA(K)
C
  200 CONTINUE
      RETURN
      END
C ---------------------------------------------------------------------
      SUBROUTINE EXTIN6(ETA1,ETA2,ETA3,XJA,XG1,XG2,XG3,FF,XI,HW,GW,HS,
     1NCONE)
C
C  THIS SUBROUTINE COMPUTES THE CONTRIBUTIONS TO THE H AND G MATRICES
C  OF AN ELEMENT WHICH DOES NOT CONTAIN THE COLLOCATION POINT
C
      COMPLEX HW(9),GW(9),U,Q
      DIMENSION ETA1(36),ETA2(36),ETA3(36),ETA(3),XJA(36)
      DIMENSION XG1(36),XG2(36),XG3(36),FF(9,36)
      DIMENSION XI(3),HS(9),P12(9)
      DIMENSION OME(6),OMEI(4),OMEJ(4)
      COMMON N,NE,L,CW,FR,INP,IPR
C
C  OME()         : GAUSS WHEIGHTS FOR QUADRILATERAL ELEMENTS
C  OMEI(),OMEJ() : GAUSS WHEIGHTS FOR TRIANGULAR ELEMENTS
C
      DATA OME/0.467913934572691,0.467913934572691,0.360761573048139,
     1         0.360761573048139,0.171324492379170,0.171324492379170/
      DATA OMEI/.1739274226 , .3260725774 , .3260725774 , .1739274226/
      DATA OMEJ/.1355069134 , .2034645680 , .1298475476 , .0311809709/
C
      DO 10 IK=1,NCONE
      HW(IK)=(0.,0.)
      GW(IK)=(0.,0.)
   10 HS(IK)=0.
C
C  COMPUTE GW, HW AND HS
C
      IF (9-NCONE) 1,1,2
    1 NPG=6
      GO TO 3
    2 NPG=4
    3 DO 200 I=1,NPG
      DO 200 J=1,NPG
      K=NPG*I-NPG+J
      ETA(1)=ETA1(K)
      ETA(2)=ETA2(K)
      ETA(3)=ETA3(K)
C
      DO 30 IK=1,NCONE
      IF (9-NCONE) 40,40,50
   50 P12(IK)=FF(IK,K)*XJA(K)*OMEI(I)*OMEJ(J)
      GO TO 30
   40 P12(IK)=FF(IK,K)*XJA(K)*OME(I)*OME(J)
   30 CONTINUE
C
C  COMPUTE FUNDAMENTAL SOLUTION
C
      CALL FUNDA6(U,Q,QS,XI(1),XI(2),XI(3),ETA,XG1(K),XG2(K),XG3(K))
C
C  SUMMATION OVER THE GAUSS POINTS TO COMPUTE GW, HW AND HS
C
      DO 60 KK=1,NCONE
      GW(KK)=GW(KK)+U*P12(KK)
      HW(KK)=HW(KK)+Q*P12(KK)
   60 HS(KK)=HS(KK)+QS*P12(KK)
C
  200 CONTINUE
      RETURN
      END
C ---------------------------------------------------------------------
      SUBROUTINE FUNDA6(U,Q,QS,XP,YP,ZP,ETA,XG,YG,ZG)
C
C  THIS SUBROUTINE COMPUTES THE FUNDAMENTAL SOLUTION AT A GAUSS POINT
C
      COMPLEX U,Q,ZW
      DIMENSION ETA(3)
      COMMON N,NE,L,CW,FR,INP,IPR
C
      RD1=XG-XP
      RD2=YG-YP
      RD3=ZG-ZP
      R=SQRT(RD1**2+RD2**2+RD3**2)
      RDN=(RD1*ETA(1)+RD2*ETA(2)+RD3*ETA(3))/R
C
      ZW=CMPLX(0.,-FR*R/CW)
      U =CEXP(ZW) / (12.5663704*R)
      Q =(( ZW - (1.,0.))*U*CMPLX(RDN,0.)) / CMPLX(R,0.)
      QS = - RDN / (12.5663704*R*R)
C
      RETURN
      END
C ---------------------------------------------------------------------
      SUBROUTINE LOCIN6(XJ1,XJ2,XJ3,HW,GW,HS,HDIF,NODO,NCONE)
C
C  THIS SUBROUTINE COMPUTES THE CONTRIBUTIONS TO THE H AND G MATRICES
C  OF AN ELEMENT WHICH CONTAINS THE COLLOCATION POINT
C
C  THE ELEMENT IS SUBDIVIDED INTO TRIANGLES DRAWING LINES FROM THE
C  COLLOCATION POINT.
C  INTEGRATION OVER THE RESULTING TRIANGLE IS DONE USING THE STANDARD
C  GAUSS QUADRATURE CORRESPONDING TO A COLAPSED QUADRILATERAL DOMAIN
C  IN SUBROUTINE TRILOC
C
C  GI1() : FIRST HOMOGENEOUS COORDINATE
C  GI2() : SECOND HOMOGENEOUS COORDINATE
C
      COMPLEX HW(9),GW(9),HDIF
      DIMENSION XI(3),V1(3),V2(3)
      DIMENSION XJ1(9),XJ2(9),XJ3(9),HS(9)
      DIMENSION GI1(9),GI2(9),GII1(15),GII2(15)
      COMMON N,NE,L,CW,FR,INP,IPR
C
      DO 5 I=1,NCONE
      HS(I)=0.
      HW(I)=(0.,0.)
    5 GW(I)=(0.,0.)
      HDIF=(0.,0.)
C
      IF (9-NCONE) 100,100,110
C
C  QUADRILATERAL ELEMENT.
C
C  SET THE VALUES OF THE HOMOGENEOUS COORDINATES FOR NODES 1 TO 9
C
  100 GI1(1)=-1.
      GI1(2)=0.
      GI1(3)=1.
      GI1(4)=1.
      GI1(5)=1.
      GI1(6)=0.
      GI1(7)=-1.
      GI1(8)=-1.
      GI1(9)=0.
      GI2(1)=-1.
      GI2(2)=-1.
      GI2(3)=-1.
      GI2(4)=0.
      GI2(5)=1.
      GI2(6)=1.
      GI2(7)=1.
      GI2(8)=0.
      GI2(9)=0.
      DO 15 I=1,8
      GII1(I)=GI1(I)
   15 GII2(I)=GI2(I)
C
C  STORE THE VALUES OF GI1,GI2 OF NODES 1 TO 7 ALSO AS 9 TO 15
C
      DO 17 I=1,7
      GII1(8+I)=GI1(I)
   17 GII2(8+I)=GI2(I)
C
C  STORE THE VALUES OF GI1,GI2 OF THE COLLOCATION NODE IN V1(1),
C  V2(1) AND ITS CARTESIAN COORDINATES IN XI(I)
C
      V1(1)=GI1(NODO)
      V2(1)=GI2(NODO)
      XI(1)=XJ1(NODO)
      XI(2)=XJ2(NODO)
      XI(3)=XJ3(NODO)
C
C  SUBDIVISION OF THE QUADRILATERAL ELEMENT INTO 2, 3 OR 4 TRIANGLES
C  DEPENDING ON THE COLLOCATION POINT
C
      GO TO(1,2,1,2,1,2,1,2,9),NODO
    1 AREA=2.
      DO 11 K=1,2
      M=2*K
      V1(2)=GII1(NODO+M)
      V2(2)=GII2(NODO+M)
      V1(3)=GII1(NODO+M+2)
      V2(3)=GII2(NODO+M+2)
   11 CALL TRILOC6(XI,XJ1,XJ2,XJ3,HW,GW,HS,AREA,V1,V2,9)
      GO TO 555
    2 DO 22 K=1,3
      M=2*K-1
      V1(2)=GII1(NODO+M)
      V2(2)=GII2(NODO+M)
      V1(3)=GII1(NODO+M+2)
      V2(3)=GII2(NODO+M+2)
      AREA=0.5*(V1(2)*V2(3)+V1(1)*V2(2)+V1(3)*V2(1)-
     1          V1(2)*V2(1)-V1(3)*V2(2)-V1(1)*V2(3))
   22 CALL TRILOC6(XI,XJ1,XJ2,XJ3,HW,GW,HS,AREA,V1,V2,9)
      GO TO 555
    9 AREA=1.
      DO 99 K=1,4
      M=2*K-1
      V1(2)=GII1(M)
      V2(2)=GII2(M)
      V1(3)=GII1(M+2)
      V2(3)=GII2(M+2)
   99 CALL TRILOC6(XI,XJ1,XJ2,XJ3,HW,GW,HS,AREA,V1,V2,9)
      GO TO 555
C
C  TRIANGULAR ELEMENT.
C
C  SET THE VALUES OF THE HOMOGENEOUS COORDINATES FOR NODES 1 TO 6
C
  110 GI1(1)=1.
      GI1(2)=0.
      GI1(3)=0.
      GI1(4)=0.5
      GI1(5)=0
      GI1(6)=0.5
      GI2(1)=0.
      GI2(2)=1.
      GI2(3)=0.
      GI2(4)=0.5
      GI2(5)=0.5
      GI2(6)=0.
C
C  STORE THE VALUES OF GI1,GI2 OF THE COLLOCATION NODE IN V1(1),
C  V2(1) AND ITS CARTESIAN COORDINATES IN XI(I)
C
      V1(1)=GI1(NODO)
      V2(1)=GI2(NODO)
      XI(1)=XJ1(NODO)
      XI(2)=XJ2(NODO)
      XI(3)=XJ3(NODO)
C
C  SUBDIVISION OF THE TRIANGULAR ELEMENT INTO 2 TRIANGLES
C  WHEN THE COLLOCATION POINT IS  A MID-SIDE NODE.
C  WHEN THE COLLOCATION POINT IS A CORNER NO SUBDIVISION EXIST.
C
      GO TO(10,20,30,40,50,60),NODO
   10 AREA=0.5
      V1(2)=GI1(2)
      V2(2)=GI2(2)
      V1(3)=GI1(3)
      V2(3)=GI2(3)
      CALL TRILOC6(XI,XJ1,XJ2,XJ3,HW,GW,HS,AREA,V1,V2,6)
      GO TO 555
   20 AREA=0.5
      V1(2)=GI1(3)
      V2(2)=GI2(3)
      V1(3)=GI1(1)
      V2(3)=GI2(1)
      CALL TRILOC6(XI,XJ1,XJ2,XJ3,HW,GW,HS,AREA,V1,V2,6)
      GO TO 555
   30 AREA=0.5
      V1(2)=GI1(1)
      V2(2)=GI2(1)
      V1(3)=GI1(2)
      V2(3)=GI2(2)
      CALL TRILOC6(XI,XJ1,XJ2,XJ3,HW,GW,HS,AREA,V1,V2,6)
      GO TO 555
   40 AREA=0.25
      V1(2)=GI1(2)
      V2(2)=GI2(2)
      V1(3)=GI1(3)
      V2(3)=GI2(3)
      CALL TRILOC6(XI,XJ1,XJ2,XJ3,HW,GW,HS,AREA,V1,V2,6)
      V1(2)=GI1(3)
      V2(2)=GI2(3)
      V1(3)=GI1(1)
      V2(3)=GI2(1)
      CALL TRILOC6(XI,XJ1,XJ2,XJ3,HW,GW,HS,AREA,V1,V2,6)
      GO TO 555
   50 AREA=0.25
      V1(2)=GI1(3)
      V2(2)=GI2(3)
      V1(3)=GI1(1)
      V2(3)=GI2(1)
      CALL TRILOC6(XI,XJ1,XJ2,XJ3,HW,GW,HS,AREA,V1,V2,6)
      V1(2)=GI1(1)
      V2(2)=GI2(1)
      V1(3)=GI1(2)
      V2(3)=GI2(2)
      CALL TRILOC6(XI,XJ1,XJ2,XJ3,HW,GW,HS,AREA,V1,V2,6)
      GO TO 555
   60 AREA=0.25
      V1(2)=GI1(1)
      V2(2)=GI2(1)
      V1(3)=GI1(2)
      V2(3)=GI2(2)
      CALL TRILOC6(XI,XJ1,XJ2,XJ3,HW,GW,HS,AREA,V1,V2,6)
      V1(2)=GI1(2)
      V2(2)=GI2(2)
      V1(3)=GI1(3)
      V2(3)=GI2(3)
      CALL TRILOC6(XI,XJ1,XJ2,XJ3,HW,GW,HS,AREA,V1,V2,6)
  555 HDIF=HDIF+HW(NODO)-HS(NODO)
      RETURN
      END
C -------------------------------------------------------------------
      SUBROUTINE TRILOC6(XI,XJ1,XJ2,XJ3,HW,GW,HS,AREA,V1,V2,NCONE)
C
C  THIS SUBROUTINE DOES A NUMERICAL INTEGRATION OVER A TRIANGULAR
C  SUBELEMENT WHEN THE COLLOCATION POINT IS ONE OF THE CORNERS.
C
C  THE TRIANGLE IS TRANSFORMED INTO A SQUARE DOMAIN AND A DOUBLE GAUSS
C  QUADRATURE IS USED
C
      COMPLEX HW(9),GW(9),U,Q
      DIMENSION XI(3),X(3),XJ1(9),XJ2(9),XJ3(9),T(6),OME(6)
      DIMENSION F(9),FD1(9),FD2(9),DC(9,9),DCT(9,9),V1(3),V2(3)
      DIMENSION AUX(9),BUX(9),CUX(9),HS(9),ETA(3),P12(9)
      COMMON N,NE,L,CW,FR,INP,IPR
      DATA T/0.238619186083197,-0.238619186083197,0.661209386466265,
     1       -0.661209386466265,0.932469514203152,-0.932469514203152/
      DATA OME/0.467913934572691,0.467913934572691,0.360761573048139,
     1         0.360761573048139,0.171324492379170,0.171324492379170/
C
      DO 400 I=1,6
      DO 400 J=1,6
C
C  COMPUTE THE NATURAL CO-ORDINATES OF THE GAUSS POINTS
C
      SS1=0.5*(T(I)+1.)
      SS2=0.5*(T(J)+1.)
      G1=(1.-SS1)*V1(1)+SS1*(1.-SS2)*V1(2)+SS1*SS2*V1(3)
      G2=(1.-SS1)*V2(1)+SS1*(1.-SS2)*V2(2)+SS1*SS2*V2(3)
C
C  COMPUTE VALUES OF THE SHAPE FUNCTIONS AND SHAPE FUNCTIONS
C  DERIVATIVES WITH RESPECT TO THE NATURAL COORDINATES AT GAUSS POINTS
C
      IF (9-NCONE) 100,100,110
C
C  THE ORIGINAL DOMAIN IS QUADRILATERAL
C
  100 F(1)=0.25*G1*G2*(G1-1.)*(G2-1.)
      F(2)=0.5*(1.-G1**2)*G2*(G2-1.)
      F(3)=0.25*G1*G2*(G1+1.)*(G2-1.)
      F(4)=0.5*G1*(G1+1.)*(1.-G2**2)
      F(5)=0.25*G1*G2*(G1+1.)*(G2+1.)
      F(6)=0.5*(1.-G1**2)*G2*(G2+1.)
      F(7)=0.25*G1*G2*(G1-1.)*(G2+1.)
      F(8)=0.5*G1*(G1-1.)*(1.-G2**2)
      F(9)=(1.-G1**2)*(1.-G2**2)
C
      FD1(1)=0.25*G2*(G2-1.)*(2.*G1-1.)
      FD1(2)=-G1*G2*(G2-1.)
      FD1(3)=0.25*G2*(G2-1.)*(2.*G1+1.)
      FD1(4)=0.5*(1.-G2**2)*(2.*G1+1.)
      FD1(5)=0.25*G2*(G2+1.)*(2.*G1+1.)
      FD1(6)=-G1*G2*(G2+1.)
      FD1(7)=0.25*G2*(G2+1.)*(2.*G1-1.)
      FD1(8)=0.5*(1.-G2**2)*(2.*G1-1.)
      FD1(9)=-2.*G1*(1.-G2**2)
C
      FD2(1)=0.25*G1*(G1-1.)*(2.*G2-1.)
      FD2(2)=0.5*(1.-G1**2)*(2.*G2-1.)
      FD2(3)=0.25*G1*(G1+1.)*(2.*G2-1.)
      FD2(4)=-G1*G2*(G1+1.)
      FD2(5)=0.25*G1*(G1+1.)*(2.*G2+1.)
      FD2(6)=0.5*(1.-G1**2)*(2.*G2+1.)
      FD2(7)=0.25*G1*(G1-1.)*(2.*G2+1.)
      FD2(8)=-G1*G2*(G1-1.)
      FD2(9)=-2.*G2*(1.-G1**2)
      GO TO 150
C
C  THE ORIGINAL DOMAIN IS A TRIANGLE
C
  110 G3=1.-G1-G2
      F(1)=G1*(2.*G1-1.)
      F(2)=G2*(2.*G2-1.)
      F(3)=G3*(2.*G3-1.)
      F(4)=4.*G1*G2
      F(5)=4.*G2*G3
      F(6)=4.*G3*G1
C
      FD1(1)=4.*G1-1.
      FD1(2)=0.
      FD1(3)=1.-4.*G3
      FD1(4)=4.*G2
      FD1(5)=-4.*G2
      FD1(6)=4.*(G3-G1)
C
      FD2(1)=0.
      FD2(2)=4.*G2-1.
      FD2(3)=1.-4.*G3
      FD2(4)=4.*G1
      FD2(5)=4.*(G3-G2)
      FD2(6)=-4.*G1
C
C  JACOBIAN AND NORMAL VECTOR FOR EACH GAUSS POINT
C
  150 DO 20 IN=1,NCONE
      DO 20 JN=1,NCONE
      DC(IN,JN)=FD1(IN)*FD2(JN)
   20 DCT(IN,JN)=FD2(IN)*FD1(JN)
      DO 30 M=1,NCONE
      AUX(M)=0.
      BUX(M)=0.
   30 CUX(M)=0.
      DO 40 JK=1,NCONE
      DO 40 IK=1,NCONE
      AUX(JK)=AUX(JK)+XJ2(IK)*(DC(IK,JK)-DCT(IK,JK))
      BUX(JK)=BUX(JK)+XJ3(IK)*(DC(IK,JK)-DCT(IK,JK))
   40 CUX(JK)=CUX(JK)+XJ1(IK)*(DC(IK,JK)-DCT(IK,JK))
      S1=0.
      S2=0.
      S3=0.
      DO 50 M=1,NCONE
      S1=S1+AUX(M)*XJ3(M)
      S2=S2+BUX(M)*XJ1(M)
   50 S3=S3+CUX(M)*XJ2(M)
C
      XJA=SQRT(S1**2+S2**2+S3**2)
      ETA(1)=S1/XJA
      ETA(2)=S2/XJA
      ETA(3)=S3/XJA
C
C  CARTESIAN CO-ORDINATES OF THE GAUSS POINT
C
      DO 60 IN=1,3
   60 X(IN)=0.
      DO 70 IK=1,NCONE
      X(1)=X(1)+F(IK)*XJ1(IK)
      X(2)=X(2)+F(IK)*XJ2(IK)
   70 X(3)=X(3)+F(IK)*XJ3(IK)
      CALL FUNDA6(U,Q,QS,XI(1),XI(2),XI(3),ETA,X(1),
     1X(2),X(3))
C
C  SUMMATION OVER THE GAUSS POINTS TO COMPUTE HW, GW AND HS
C
      DO 80 IK=1,NCONE
   80 P12(IK)=F(IK)*XJA*(T(I)+1.)*AREA*0.25*OME(I)*OME(J)
      DO 90 KK=1,NCONE
      GW(KK)=GW(KK)+U*P12(KK)
      HW(KK)=HW(KK)+Q*P12(KK)
   90 HS(KK)=HS(KK)+QS*P12(KK)
  400 CONTINUE
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE INTER6(FI,DFI,KODE,BC,CX,CY,CZ,X,Y,Z,DSOL,NCONEC,
     1KCONEC,NESIM,KE)
C
C  THIS SUBROUTINE COMPUTES THE VALUES OF THE PRESSURES AT INTERNAL
C  POINTS
C
      COMPLEX FI(1),DFI(9,1),DSOL(1),BC(9,1),HW(9),GW(9)
      DIMENSION KODE(1),X(1),Y(1),Z(1),CX(1),CY(1),CZ(1),XI(3),HS(9)
      DIMENSION ETA1(36),ETA2(36),ETA3(36),XJA(36),FF(9,36)
      DIMENSION XJ1(9),XJ2(9),XJ3(9),XG1(36),XG2(36),XG3(36)
      DIMENSION NCONEC(1),KCONEC(9,1),NESIM(1),KE(5,1)
      COMMON N,NE,L,CW,FR,INP,IPR
C
C  REARRANGE FI AND DFI ARRAYS TO STORE ALL THE VALUES OF THE
C  POTENTIAL IN FI AND ALL THE VALUES OF THE FLUX IN DFI
C
      DO 5 IE=1,NE
      IF (KODE(IE)) 6,6,7
    6 DO 8 K=1,NCONEC(IE)
      IK=KCONEC(K,IE)
    8 DFI(K,IE)=FI(IK)
      GO TO 5
    7 DO 9 K=1,NCONEC(IE)
    9 DFI(K,IE)=BC(K,IE)
    5 CONTINUE
C
      DO 10 IN=1,N
      DO 15 J=1,NESIM(IN)
      IF (KODE(KE(J,IN))) 20,20,15
   20 DO 25 M=1,NCONEC(KE(J,IN))
      IF (IN-KCONEC(M,KE(J,IN))) 25,26,25
   26 FI(IN)=BC(M,KE(J,IN))
      GO TO 10
   25 CONTINUE
   15 CONTINUE
   10 CONTINUE
C
C  COMPUTE THE POTENTIAL VALUES AT INTERNAL POINTS
C
      IF (L.LE.0) RETURN
C
      DO 40 I=1,L
   40 DSOL(I)=(0.,0.)
C
      DO 120 IE=1,NE
      DO 50 II=1,NCONEC(IE)
      IK=KCONEC(II,IE)
      XJ1(II)=X(IK)
      XJ2(II)=Y(IK)
   50 XJ3(II)=Z(IK)
      CALL ETAXJA(XJ1,XJ2,XJ3,ETA1,ETA2,ETA3,XJA,XG1,XG2,XG3,FF,
     1NCONEC(IE))
      DO 110 K=1,L
      XI(1)=CX(K)
      XI(2)=CY(K)
      XI(3)=CZ(K)
      CALL EXTIN6(ETA1,ETA2,ETA3,XJA,XG1,XG2,XG3,FF,XI,HW,GW,HS,
     1NCONEC(IE))
      DO 60 M=1,NCONEC(IE)
      JK=KCONEC(M,IE)
   60 DSOL(K)=DSOL(K)+GW(M)*DFI(M,IE)-HW(M)*FI(JK)
  110 CONTINUE
  120 CONTINUE
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE OUTPUT6(FI,DFI,DSOL,CX,CY,CZ,NCONEC,KCONEC)
C
C  THIS SUBROUTINE PRINTS THE VALUES OF THE POTENTIAL AND ITS NORMAL
C  DERIVATIVES AT BOUNDARY NODES. IT ALSO PRINTS THE VALUES OF POTENTIAL
C  AT INTERNAL POINTS.
C
      COMPLEX FI(1),DFI(9,1),DSOL(1)
      DIMENSION NCONEC(1),KCONEC(9,1),CX(1),CY(1),CZ(1)
      COMMON N,NE,L,CW,FR,INP,IPR
      WRITE(IPR,50)
   50 FORMAT(/,' ',79('*')/)
      WRITE(IPR,700) FR
  700 FORMAT(30X,'FREQUENCY = ',G14.7/)
      WRITE(IPR,100)
  100 FORMAT(' ',79('*')//1X,'RESULTS'//2X,'BOUNDARY NODES'//
     110X,' NODE ',10X,'  POTENTIAL ',/)
      DO 10 I=1,N
   10 WRITE(IPR,200) I,FI(I)
  200 FORMAT(9X,I5,9X,G11.4,',',G11.4)
      WRITE(IPR,110)
  110 FORMAT(//2X,'BOUNDARY ELEMENTS'//15X,'ELEMENT ',27X,'FLUX',/)
      DO 20 IE=1,NE
      WRITE(IPR,120) IE
  120 FORMAT(/15X,I4,5X,'  NODES '/)
      DO 30 K=1,NCONEC(IE)
      IK=KCONEC(K,IE)
   30 WRITE(IPR,130) IK,DFI(K,IE)
   20 CONTINUE
  130 FORMAT(27X,I4,8X,G11.4,',',G11.4)
C
      IF (L.LE.0) GO TO 222
      WRITE(IPR,300)
  300 FORMAT(//2X,'INTERNAL POINTS',//5X,'X',12X,'Y',12X,'Z',18X,
     1'POTENTIAL',/)
      DO 60 K=1,L
   60 WRITE(IPR,400) CX(K),CY(K),CZ(K),DSOL(K)
  400 FORMAT(3(2X,G11.4),3X,G11.4,',',G11.4)
C
  222 WRITE(IPR,500)
  500 FORMAT(/' ',79('*'))
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE CSOLVER(A,B,N,NX)
C
      COMPLEX A(NX,NX),B(NX),X,CDOT
C
C     A=L*U  FACTORIZATION
C
      DO 30 K=1,N
      K1=K-1
      DO 10 I=K,N
      A(I,K)=A(I,K)-CDOT(K1,A(I,1),NX,A(1,K),1)
   10 CONTINUE
      X=A(K,K)
      IF(ABS(REAL(X))+ABS(AIMAG(X)).EQ.0.) THEN
      WRITE(6,*) ' SOLVER : STOP  SINGULARITY IN ROW ',K
      STOP ' SOLVER : STOP SINGULARITY '
      END IF
      X=(1.,0.)/X
      DO 20 J=K+1,N
      A(K,J)=X*(A(K,J)-CDOT(K1,A(K,1),NX,A(1,J),1))
   20 CONTINUE
   30 CONTINUE
C
C  FORWARD AND BACK SUBSTITUTION    L*U*X=B
C
C  SOLVE  L*Y=B
C
      DO 40 I=1,N
      B(I)=-(CDOT((I-1),A(I,1),NX,B,1)+B(I))/A(I,I)
   40 CONTINUE
C
C  SOLVE  U*X=Y
C
      DO 50 I=N,1,-1
      B(I)=-(CDOT((N-I),A(I,I+1),NX,B(I+1),1)+B(I))
   50 CONTINUE
C
      RETURN
      END
C-----------------------------------------------------------------------
      COMPLEX FUNCTION CDOT(N,U,IUX,V,IVX)
C
C THIS FUNCTION MAKES THE COMPLEX DOT PRODUCT OF TWO VECTORS
C
      COMPLEX U(1),V(1)
C
      CDOT=(0.,0.)
      IF(N.LE.0) RETURN
      IU=1
      IV=1
      IF(IUX.LT.0) IU=1-(N-1)*IUX
      IF(IVX.LT.0) IV=1-(N-1)*IVX
      DO 10 I=1,N
      CDOT=CDOT+U(IU)*V(IV)
      IU=IU+IUX
      IV=IV+IVX
10    CONTINUE
      RETURN
C
      END
